/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef __CAMERAAPPLICATION__
#define __CAMERAAPPLICATION__

#include "../../main/application.hpp"
#include "../../main/mmboxstate.hpp"

// used interfaces
#include "nmm/interfaces/base/INode.hpp"
#include "nmm/interfaces/video/viscacamera/IVISCACamera.hpp"

namespace NMM {

  class CameraState : public MMBoxState {
  public:
    CameraState(string, MMBoxApplication*);
    ~CameraState();

        Result receiveMessage( string message );
    Result doCreate( string );
    Result doKill();
    Result doInitialize( string );
    Result doUninitialize();
  
    /* here all used "events" must be entered */
    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Stop();
    Result Back();
    void updateColorBar();
    Result colorKeyRed();
    Result colorKeyGreen();
    
    void disconnectSinkSynchronizer(ISinkSynchronizer* tmp_sync);
    void connectSinkSynchronizer(ISinkSynchronizer* tmp_sync);
  private:
    GraphDescription graph;
    //    ISinkSynchronizer_var sink_synchronizer;
    INode* display;
    INode* osd_node;
    NodeDescription* playback_nd;
    NodeDescription* osdNode_nd;
    ISinkSynchronizer_var sink_synchronizer;
    // own nodes
    INode* camera;

    // interfaces
    IVISCACamera* camera_interface;

    // moving & zooming
    bool move_up, move_down, move_left, move_right, zoom_in, zoom_out;

    void showAlert(string title, string message);
    
    void requestNodes(string);
    void cleanup();
  };
}

#endif
