/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de> 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "DistTaskState.hpp"
#include "../../main/application.hpp"
#include "nmm/base/ProxyObject.hpp"

namespace NMM {
  DistTaskState::DistTaskState( string id, MMBoxApplication* app) : DistMMBoxState(id, app)
  {
    setNeedAudio(true);
    setNeedVideo(true);
  } 
 
  DistTaskState::~DistTaskState()
  {
  } 

  Result DistTaskState::Back() 
  {
    return goToLastState();
  }
  
  Result DistTaskState::doCreate( string ) 
  {
    app->getOsdManager()->setPause(false);
    app->setUseExternalSynchronizer(true);
    
    ISynchronizedSink_var audio_sync(app->getAudioNode()->getParentObject()->getCheckedInterface<ISynchronizedSink>());
    ISinkController_var audio_sync_controller(audio_sync->getController());
    app->getSynchronizer()->removeSinkController(audio_sync_controller.get());
    audio_sync->setSynchronized(true);

    ISynchronizedSink_var video_sync(app->getDisplayNode()->getParentObject()->getCheckedInterface<ISynchronizedSink>());
    ISinkController_var video_sync_controller(video_sync->getController());
    app->getSynchronizer()->removeSinkController(video_sync_controller.get());
    video_sync->setSynchronized(true);

    return SUCCESS;
  }

  Result DistTaskState::doInitialize( string ) 
  {
    return SUCCESS;
  }

  Result DistTaskState::doUninitialize() 
  {
    return SUCCESS;
  }

  Result DistTaskState::doKill() 
  {
    ISynchronizedSink_var audio_sync(app->getAudioNode()->getParentObject()->getCheckedInterface<ISynchronizedSink>());
    ISinkController_var audio_sync_controller(audio_sync->getController());
    audio_sync->setSynchronized(false);

    ISynchronizedSink_var video_sync(app->getDisplayNode()->getParentObject()->getCheckedInterface<ISynchronizedSink>());
    ISinkController_var video_sync_controller(video_sync->getController());
    video_sync->setSynchronized(false);

    app->getOsdManager()->setPause(true);
    app->setUseExternalSynchronizer(false);

    app->getOsdManager()->update();

    return SUCCESS;
  }

}
