#include "EPGHandler.hpp"

#include <fstream>
#include <stdio.h>

#include "nmm/utils/utils.hpp"

namespace NMM{
  EPGHandler::EPGHandler(ITVGuide *itvg)
    : WebRequestHandler("epg"),
      itvguide(itvg)
  {}
  EPGHandler::~EPGHandler(){}
  
  void EPGHandler::doGet (WebServerRequest *pRequest, WebServerResponse *pResponse){
    //cerr << "Method  : " << pRequest->getMethod() << endl;
    //cerr << "Protocol: " << pRequest->getProtocol() << endl;
    //cerr << "URI     : " << pRequest->getRequestURI() << endl;
    //cerr << "Query   : " << pRequest->getQueryString() << endl;
    //cerr << "RemAddr : " << pRequest->getRemoteAddress() << endl;

    if(!itvguide){
      pResponse->write("<html><head><title>MMBox-TVGuide</title></head><body>no TVGuide available<br></body></html>");
      return;
    }
    
    
    if(!pRequest->getQueryString().empty()){
      string requestOption = pRequest->getQueryString();
      requestOption = requestOption.substr(0,requestOption.find_first_of('='));

      time_t now = time(NULL); 
      struct tm *t =localtime(&now); //make sure, the timezone-vaiable is initialized
      char tmp[6];
      double d = -1*timezone/60/60*100;
      if(t->tm_isdst) d += 100;
      sprintf(tmp, " %0+5.0f", d);
      string tz = string(tmp);

      if(requestOption == "channels"){

      }else if(requestOption== "event"){
        
        pResponse->write("<html><head><title>Test-Site</title></head><body>Test Successful <br> <a href='/'>back</a></body></html>");
        
      }else if(requestOption== "date"){
        pResponse->write("<html>");
        pResponse->write("<head>");
        //pResponse->write("<title>Test-Site</title>");

        ifstream f((getNMMResourceDir() + "/mmbox/webhandler/epg.css").c_str());
        char line[1000];
        
        if(f.fail()){
        }else{
          while(!f.eof()){
            f.getline(line, 1000);
            pResponse->write(string(line));
        }
          f.close(); 
        }

        pResponse->write("<script type=\"text/javascript\">");
        pResponse->write("<!--");
        pResponse->write("function open_window(URI){");
        pResponse->write("F1 = window.open(URI,\"Fenster1\",\"width=310,height=400,left=0,top=0\");");
        pResponse->write("}");
        pResponse->write("//-->");
        pResponse->write("</script>");

        pResponse->write("</head>");
        pResponse->write("<body>");

        pResponse->write("<h1>MMBox - TVGuide</h1>");

        pResponse->write("<table id='all'>");
        pResponse->write("<tr>");
        pResponse->write("<th></th>");

        pResponse->write("<th colspan=\"60\" >00:00</th>");
        pResponse->write("<th colspan=\"60\" >01:00</th>");
        pResponse->write("<th colspan=\"60\" >02:00</th>");
        pResponse->write("<th colspan=\"60\" >03:00</th>");
        pResponse->write("<th colspan=\"60\" >04:00</th>");
        pResponse->write("<th colspan=\"60\" >05:00</th>");
        pResponse->write("<th colspan=\"60\" >06:00</th>");
        pResponse->write("<th colspan=\"60\" >07:00</th>");
        pResponse->write("<th colspan=\"60\" >08:00</th>");
        pResponse->write("<th colspan=\"60\" >09:00</th>");
        pResponse->write("<th colspan=\"60\" >10:00</th>");
        pResponse->write("<th colspan=\"60\" >11:00</th>");
        pResponse->write("<th colspan=\"60\" >12:00</th>");
        pResponse->write("<th colspan=\"60\" >13:00</th>");
        pResponse->write("<th colspan=\"60\" >14:00</th>");
        pResponse->write("<th colspan=\"60\" >15:00</th>");
        pResponse->write("<th colspan=\"60\" >16:00</th>");
        pResponse->write("<th colspan=\"60\" >17:00</th>");
        pResponse->write("<th colspan=\"60\" >18:00</th>");
        pResponse->write("<th colspan=\"60\" >19:00</th>");
        pResponse->write("<th colspan=\"60\" >20:00</th>");
        pResponse->write("<th colspan=\"60\" >21:00</th>");
        pResponse->write("<th colspan=\"60\" >22:00</th>");
        pResponse->write("<th colspan=\"60\" >23:00</th>");

        pResponse->write("</tr>");

        string date = pRequest->getQueryString().substr(pRequest->getQueryString().find_first_of('=')+1);
        
        list<ChannelDescription> channels = itvguide->getChannelsList();
        string channelColor = "color1";
        string channelColorOld = "color2";

        for(list<ChannelDescription>::iterator i = channels.begin(); i != channels.end(); i++){
          pResponse->write("<tr id='" + channelColor + "'>");
          string stmp = channelColor; channelColor = channelColorOld; channelColorOld = stmp;

          pResponse->write("<td id=\"channel\" >" + i->getChannelNames().front() + "</td>");

          //fill the firts cells from midnight to start of first program with last Event, if available
          time_t midnight = stringToTime(date + "0000");
          struct tm *midnight_tm = localtime(&midnight);
          midnight_tm->tm_hour = 0;
          midnight_tm->tm_min  = 0;
          midnight_tm->tm_sec  = 0;
          midnight = mktime(midnight_tm);

          string filled = "filled";

          list<ProgramDescription> programme = itvguide->getEventsOnDayOnChannel(date + tz, i->getID());
          if(programme.empty()) continue; 

          char chMidnight[20];
          time_t beforeMidnight = midnight - 60;
          strftime(chMidnight, sizeof(chMidnight), "%Y%m%d%H%M", localtime(&(beforeMidnight)));
          ProgramDescription midnightPD = itvguide->getEvent(string(chMidnight)+tz, string(chMidnight)+tz, i->getID());
          midnightPD.localizeTimes();
          ProgramDescription firstPD = programme.front();
          firstPD.localizeTimes();
          time_t firstProgramStart = stringToTime(firstPD.getStartTime());
          int colspan = (firstProgramStart - midnight) / 60;
          if(midnightPD.empty()){
            pResponse->write("<td id='empty' colspan='" + string(itoa(colspan)) + "'>" + "</td>");
          }else{
            if(firstProgramStart != midnight){
              pResponse->write("<td id='" + filled + "' colspan='" + string(itoa(colspan)) + "'>" + midnightPD.getTitles().front() +  "</td>");
            }
          }

          time_t lastStopTime = 0;
          for(list<ProgramDescription>::iterator j = programme.begin(); j != programme.end(); j++){
            j->localizeTimes();
            //calculate colspan
            time_t startTime = stringToTime(j->getStartTime());
            time_t stopTime = 0;
            if((j->getEndTime()).empty()){
              list<ProgramDescription>::iterator k = j; k++;
              if(k != programme.end()){
                ProgramDescription p = *k;
                p.localizeTimes();
                stopTime = stringToTime(p.getStartTime());
              }else{
                stopTime = midnight + (24 * 3600);
              }
              
            }else{
              stopTime = stringToTime(j->getEndTime());
            }

            if(stopTime > (midnight + (24 * 3600))){
              stopTime = midnight + (24 * 3600);
            }

            colspan = (stopTime - startTime) / 60;

            if((lastStopTime != 0) && (startTime - lastStopTime) > 60){
              //determine Sendeschluss
              pResponse->write("<td id='empty' colspan='" + string(itoa((startTime - lastStopTime) / 60)) + "'>" + "</td>");
            }
            lastStopTime = stopTime;

            pResponse->write("<td id='" + filled + "' colspan='" + string(itoa(colspan)) + "'>" 
                             + "<a href='javascript:open_window(\"/epg?description="
                             + j->getStartTime().substr(0,12) + ":" 
                             + (j->getEndTime().empty() ? "" : j->getEndTime().substr(0,12)) 
                             + ":" + j->getChannelID()
                             + "\")'>"
                             + j->getTitles().front() + "</a></td>");
            
          }
          pResponse->write("</tr>");

        }
        pResponse->write("</table>");

        pResponse->write("</body>");
        pResponse->write("</html>");
        
      }else if(requestOption== "calendar"){
        ifstream f((getNMMResourceDir() + "/mmbox/webhandler/calendar.html").c_str());
        char line[1000];
        
        while(!f.eof()){
          f.getline(line, 1000);
          pResponse->write(string(line) + "\n");
        }
        f.close(); 
        
      }else if(requestOption== "description"){

        try{
          string requestParams = pRequest->getQueryString().substr(pRequest->getQueryString().find_first_of('=')+1);
          size_t pos = requestParams.find_first_of(':');
          size_t oldPos = 0;
          string startTime = requestParams.substr(oldPos,pos);
          oldPos = pos+1;
          pos = requestParams.find_first_of(':', oldPos);
          string stopTime  = requestParams.substr(oldPos, pos-oldPos);
          
          pos =  requestParams.find_last_of(':');
          string channel   = requestParams.substr(pos+1);
                    
          ProgramDescription pd = itvguide->getEvent(startTime + tz, stopTime + tz, channel);
          pd.localizeTimes();
          pResponse->write("<html><head>");
          pResponse->write("<title>Description-Site</title>");
          
          ifstream f((getNMMResourceDir() + "/mmbox/webhandler/epg.css").c_str());
          char line[1000];
          
          if(f.fail()){
          }else{
            while(!f.eof()){
              f.getline(line, 1000);
              pResponse->write(string(line));
            }
            f.close(); 
          }

          pResponse->write("</head>");
          pResponse->write("<body>");
          
          pResponse->write("<table id='desc'>");
        
          pResponse->write("<tr>");        
          pResponse->write("<td id='date'>");
          string date = pd.getStartTime();
          pResponse->write(date.substr(6,2) + "." + date.substr(4,2) + "." + date.substr(0,4));
          pResponse->write("</td>");
          pResponse->write("</tr>");
          
          pResponse->write("<tr>");
          pResponse->write("<td id='time'>");
          pResponse->write(date.substr(8,2) + ":" + date.substr(10,2));
          if(!pd.getEndTime().empty()){
            date = pd.getEndTime();
            
            pResponse->write(" - " + date.substr(8,2) + ":" + date.substr(10,2));
          }
          pResponse->write("</td>");
          pResponse->write("</tr>");
          
          pResponse->write("<tr>");
          pResponse->write("<td id='channel'>" + itvguide->getChannelById(pd.getChannelID()) + "</td>");
          pResponse->write("</tr>");
          pResponse->write("<tr>");
          pResponse->write("<td id='titles'>");
          list<string> titles = pd.getTitles();
          for(list<string>::iterator i = titles.begin(); i != titles.end(); i++){
            pResponse->write( *i + "<br>");
          }
          pResponse->write("</td>");
          pResponse->write("</tr>");
          
          pResponse->write("<tr>");
          pResponse->write("<td id='desc'>");
          pResponse->write(pd.getDescription());
          pResponse->write("</td>");
          pResponse->write ("</tr>");
          
          string channelName = itvguide->getChannelById(channel);
          if(!channelName.empty()){
            pResponse->write("<tr>");
            pResponse->write("<td>");
            pResponse->write("<a href='/record?program=" + startTime + ":" + stopTime + ":" + channelName + "'>Record</a>");
            pResponse->write("</td>");
            pResponse->write("<td style='text-align:right'>");
            pResponse->write("<a href='javascript:this.close()'>Close</a>");
            pResponse->write("</td>");
            pResponse->write("</tr>");
          }
          
          pResponse->write("</table>");
          pResponse->write("</body></html>");
              
        }catch(const Exception &e){
          pResponse->write("<html><head><title>Extended Description</title></head><body>No Information found <br> <a href='/'>back</a></body></html>");
        }catch(...){
          pResponse->write("<html><head><title>Extended Description</title></head><body>No Information found <br> <a href='/'>back</a></body></html>");
        }
      }else{          
        pResponse->write("<html><head><title>Test-Site</title></head><body>Test failed <br> <a href='/'>back</a></body></html>");
      }

    }
    
  }

  void EPGHandler::doPost(WebServerRequest *, WebServerResponse *){
  }
}
