/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        David Maass <dama@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_TRANSCODECONFIG_HPP
#define _NMM_TRANSCODECONFIG_HPP

#include "nmm/plugins/video/osd/manager/OSDElements/OSDElementComposit.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/listunit.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/doublelistunit.hpp"

#include "nmm/base/registry/GraphDescription.hpp"

#include "../main/PlaylistTemplateFunctions.hpp"
#include "../main/TranscodeTemplateFunctions.hpp"
#include "../main/application.hpp"

#include "nmm/utils/DirReader.hpp"
#include "nmm/utils/gdparse/ParserStructs.hpp"

namespace NMM {
  class TranscodeConfig:
    public OSDElementComposit {
  public:
    TranscodeConfig(IOSDManager* manager, IWidgetFactory* factory, TranscodeTemplateFunctions* state);
    ~TranscodeConfig();
    
    Result selectLeft();
    Result selectRight();
    Result selectUp();
    Result selectDown();
    Result select();

    Result colorKeyRed();
    Result colorKeyGreen();
    Result colorKeyBlue();

    void reset();

    void setRootpath(string);
    void setApplication(MMBoxApplication*);

    void setWinProperties(int nr, win_properties_t* prop);
    void createWindows();
    void deleteWindows();
    void selectWindow(int winid);
    void enable();
    void setConfig(MMBoxConfig* config_, map<string,string> so) {config=config_;shown_options=so;}
    void analyzeDir();
    void globalOptions();
    void show() {OSDElementComposit::show();updateColorBar();manager->update();}
    void updateColorBar();

    //returns URL to the script, which should be called after transcoding the files
    //returns "" if no script available
    string getScript();

  protected:
    void analyzeNodeConfig();
    void selectupdown();

  private:
    MMBoxConfig*                            config;
    ListUnit                                *gd_win,*conf1_win,*comment_win;
    TranscodeTemplateFunctions*             state;
    DoubleListUnit*                         conf2_win;
    DirReader                               dirreader;
    win_properties_t                        winprops[4];
    vector<DirReader::directory_t>          dir;
    vector<string>                          commentVector;
    vector<string>                          option_backup;
    map<string,string>                      shown_options;
    vector<GraphDescription*>               graphVector;
    vector<list<NodeConfiguration>* >       configVector; 
    vector<SequenceValue<NodeDescription> > nodedescriptionVector;
    MMBoxApplication*                       app;
    string                                  rootpath;

    enum {NORMAL=0,CONFIG}                          mode;
    enum myfocus {JOBS=0,COMMENT,NODES,CONFIGNODES} focus;
  };
}

#endif
