/*
 * Qwt Widget Library (http://qwt.sf.net)
 * Copyright (C) 1997   Josef Wilgen
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
 * USA.
 */

#ifndef QWT_DIMAP_H
#define QWT_DIMAP_H

class QwtDiMap {

 private:

	// PRIVATE DATA
	double d_x1, d_x2;  // double interval boundaries
	int d_y1,d_y2;      // integer interval boundaries
	double d_cnv;       // conversion factor
	int d_log;	   // logarithmic scale?
	
	// PRIVATE MEMBERS
	void newFactor();	

 public:
	
	// PUBLIC DATA
	static const double LogMin;
	static const double LogMax;

	// PUBLIC MEMBERS
	QwtDiMap();
	QwtDiMap(int i1, int i2, double d1, double d2);
	~QwtDiMap();
	
	int transform(double x) const;
	double xTransform(double x) const;
	double invTransform(int i) const;
	int limTransform(double x) const;

	void setIntRange(int i1, int i2);
	void setDblRange(double d1, double d2,int lg = 0);

	bool contains(double x);
	bool contains(int x);
	
	double d1() const {return d_x1;}
	double d2() const {return d_x2;}
	int i1() const {return d_y1;}
	int i2() const {return d_y2;}
	inline bool logarithmic() const {return d_log;}
	
};

#endif






