/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <iostream>
#include "PluginView.hpp"
#include <qlistview.h>
#include <qlayout.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>

#include "nmm/base/registry/PluginManager.hpp"
#include "nmm/interfaces/base/registry/IPluginManager.hpp"
#include "nmm/base/registry/LocalNodeFactory.hpp"
#include "nmm/base/registry/LocalRegistry.hpp"
#include "nmm/base/BaseExceptions.hpp"
#include "nmm/base/NMMApplication.hpp"



PluginView::PluginView(NMM::NMMApplication& na, QWidget *parent, const char *name ) : View(parent,name) {
  __p_manager     = &na.getPluginManager();
  allocate();
  initConnect();
  updateView();
}
PluginView::~PluginView(){
 
}

void PluginView::allocate() {
  __current_item  = 0;
  __top_layout    = new QGridLayout(this, 1, 1);
  __list_view     = new QListView(this);
  __list_view->addColumn("Plugin");
  __list_view->addColumn("active");
  __top_layout->addWidget(__list_view,0,0);
  __plugin_menu = new QPopupMenu(this);
  __plugin_menu->insertItem("Load Plugin", this, SLOT(loadPlugin()), 0, 0);
  __plugin_menu->insertItem("Unload Plugin", this,  SLOT(unloadPlugin()), 0, 1);
  __plugin_menu->insertSeparator();
  __plugin_menu->insertItem("Init Registry", this, SLOT(initRegistry()), 0, 2);
  __plugin_menu->insertItem("Init Factory", this, SLOT(initFactory()), 0, 3);
}

void PluginView::updateView() {
  __list_view->clear();
  list<string> plugins = __p_manager->getPlugins();
  for (list<string>::iterator it = plugins.begin(); it != plugins.end(); ++it) {
    new QListViewItem(__list_view, it->c_str());
  }
}

void PluginView::initConnect() {
  connect(__list_view, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint &, int)), SLOT(openDialog(QListViewItem*, const QPoint &, int)));
}

void PluginView::openDialog(QListViewItem* item, const QPoint& point, int coord) {
  __current_item = item;
//  if (__p_manager->isPluginUsed(item->text(0).latin1())) {
//    __plugin_menu->setItemEnabled(2, true);
//    __plugin_menu->setItemEnabled(3, true);
//  } else {
//    __plugin_menu->setItemEnabled(2, false);
//    __plugin_menu->setItemEnabled(3, false);
//  }

  __plugin_menu->popup(point, coord);
}
void PluginView::loadPlugin() {
  try {
    if (__current_item) {
      __p_manager->loadLibrary(__current_item->text(0).latin1());
    }
  } catch (const NMM::Exception& e ) {
    QMessageBox::critical(this, "Plugin error", e.getComment().c_str());
    cerr << e << endl;
  }
}
void PluginView::unloadPlugin() {
  if (__current_item) {
    __p_manager->removeLibrary(__current_item->text(0).latin1());
  }
}

void PluginView::initFactory() {
  try {
    if (__current_item) {
    //  __p_manager->activateFactoryPart(__current_item->text(0).latin1(), NMM::LocalNodeFactory::getInstance());
    }
  } catch (const NMM::Exception& e) {
    QMessageBox::critical(this, "Plugin error", e.getComment().c_str());
    cerr << e << endl;
  }
}

void PluginView::initRegistry() {
  try {
    if (__current_item) {
     // __p_manager->activateRegistryPart(__current_item->text(0).latin1(), NMM::LocalRegistry::getInstance());
    }
  } catch (const NMM::Exception& e) {
    QMessageBox::critical(this, "Plugin error", e.getComment().c_str());
    cerr << e << endl;
  }
}
