/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christian Gerstner <gerstner@graphics.cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_VCORE_MDIWINDOW_HPP_
#define _NMM_VCORE_MDIWINDOW_HPP_ _NMM_VCORE_MDIWINDOW_HPP_

#include <qmainwindow.h>
#include <qworkspace.h>
#include <qtoolbar.h>
#include <qcombobox.h>
#include <qtoolbutton.h>
#include <qbuttongroup.h>

namespace NMM {
  class NMMApplication;
}

class MDIWindow : public QMainWindow
{
  Q_OBJECT
private:
  NMM::NMMApplication& app;
  
  QWorkspace* workspace;
  QToolBar* vcoremode;
  QComboBox* mode;
  QToolBar* vncmenu;
  QToolButton* tbundo;
  QToolButton* tbredo;
  QToolButton* tbclear;
  QButtonGroup* colorgroup;
  QToolButton* tbblue;
  QToolButton* tbgreen;
  QToolButton* tbred;
  QToolButton* tbblack;
  QToolButton* tbcustom;
  QButtonGroup* drawtypegroup;
  QToolButton* tbpointer;
  QToolButton* tbfreehand;
  QToolButton* tbrectangle;
  QToolButton* tbline;

public:
  QToolBar* getToolBarVNCMenu();
  QButtonGroup* getButtonGroupColor();
  QButtonGroup* getButtonGroupDrawType();
  QComboBox* getComboBoxMode();
  QToolButton* getToolButtonRed();
  QToolButton* getToolButtonGreen();
  QToolButton* getToolButtonBlue();
  QToolButton* getToolButtonBlack();
  QToolButton* getToolButtonCustom();
  QToolButton* getToolButtonUndo();
  QToolButton* getToolButtonRedo();
  QToolButton* getToolButtonClear();
  void closeEvent( QCloseEvent* ce );

  /**
   * The main widget for the application holding all needed
   * buttons, editlines, sliders, switchboxes, ...
   */
  MDIWindow(NMM::NMMApplication& app_, QWidget *parent = 0, const char* name = "MDIWindow", WFlags f = 0);
  ~MDIWindow();
    
  QWorkspace* getWorkspace(){	return workspace; };

public slots:

   void doNothing(){};

signals:

  void mdiWindowClosed();  
};
#endif //_NMM_VCORE_MDIWINDOW_HPP_
