/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_INPUTJACK_HPP_
#define _NMM_INPUTJACK_HPP_

#include "Jack.hpp"
#include "nmm/interfaces/base/IInputJackImpl.hpp"
#include "nmm/interfaces/base/IOutputJack.hpp"

namespace NMM {

  class Buffer;
  class Node;
  class Message;
  class OutputJack;
  class OutputJack;
  class ReceiverStrategy;
  class CommunicationChannel;
  class IOutputJack;
  class TransportStrategy;

  /**
   * This is the (abstract) superclass for all output jacks.
   */
  class InputJack : public Jack, public IInputJackImpl
  {

  private:
    // mutable OutputJack *partner;           ///< The jack this jack is connected to.
    mutable IOutputJack_var data_partner;     ///< The jack this jack should exchange buffers with.
    InputJack  *data_jack;                    ///< The data jack of this jack.
    TransportStrategy* __transport_strategy;  ///< stores the pointer to the TransportStrategy if it is used to send messages
  public:

    /**
     * The main constructor. This constructor only sets the parent,
     * not the partner.
     * \param node The parent node.
     */
    InputJack(Node* parent, const string& jack_tag = "default", const string& name = "InputJack");

    /**
     * The main destructor.
     */
    virtual ~InputJack();
    /**
     * A method to obtain a pointer to the data jack of this jack.
     * \returns a pointer to the data jack of this jack, or NULL if
     * this jack has no data jack (only in special cases). If a jack
     * is its own data jack, it should return a pointer to itself, not
     * NULL.
     */
    virtual const InputJack* getDataJack() const {return data_jack;}
    virtual       InputJack* getDataJack()       {return data_jack;}

    /**
     * Connect to a partner jack.
     * \param jack partner jack.
     */
    virtual void connectTo(const OutputJack* jack) const
      throw (const BindingException&);

    /**
     * Returns true, if this Jack is connected, otherwise false.
     * \return true, if this Jack is connected, otherwise false.
     * \note Still public for MPEGDemuxNode
     */
    virtual bool isConnected() const;

   protected:

    virtual CommunicationChannel* connectTo(const IOutputJack& jack)
      throw (const BindingException&);

    /**
     * Disconnect from partner jack.
     */
    virtual void disconnect();

    /**
     * A method to obtain a pointer to the partner of this jack.
     * \returns a pointer to the partner jack of this jack, or NULL
     * if this jack has no partner.
     */
    virtual IOutputJack* getDataPartner();

    virtual ITransportStrategy* getTransportStrategy();

    virtual void setTransportStrategy(TransportStrategy* strategy)
      throw (const NMM::BindingException&);
    virtual void setTransportStrategy(const TSAddress*)
      throw (const NMM::BindingException&);
    
    virtual void connectToJack();
    
    virtual TSAddress* getDefaultAddress(const string& strategy_type) const;
  };

}

#endif // _NMM_INPUTJACK_HPP_
