/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *                    Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#include "Message.hpp"
namespace NMM {

  Message::Message(Type t) : SerializedValue("Message", "NMM"), label(0), mutex(ThreadMutex::RECURSIVE)
  {
    // time is initialized by its standard constructor
    setDirection(DOWNSTREAM);
    type.setValue(t);
    setMandatoryBit(false);
    ref_count = 0;

    registerValues();
  }

  Message::Message(const Message& m) : SerializedValue(m), label(m.label), mutex(ThreadMutex::RECURSIVE)
  {
    setDirection(m.getDirection());
    type.setValue(m.getType());
    time.setValue(m.getTimestamp());
    setMandatoryBit(m.getMandatoryBit());
    ref_count = 0;

    registerValues();
  }


  Message::~Message()
  {
  }

  void Message::release() const 
  {
    mutex.lock();
    ref_count--;
    if (ref_count <= 0) {
      mutex.unlock();
      erase();
    }
    else {
      mutex.unlock();
    }
  }

  void Message::increaseCount(unsigned int ref) const 
  {
    mutex.lock();
    ref_count += ref;
    mutex.unlock();
  }
  
  void Message::setType(Type new_type) {
    type.setValue(new_type);
  }

  void Message::registerValues() {
    insertArgument(&direction);
    insertArgument(&type);
    insertArgument(&time);
    insertArgument(&mandatory);
    insertArgument(&label);
  }

  Message& Message::operator=(const Message& message) {
    direction = message.getDirection();
    time      = message.getTimestamp();
    mandatory = message.getMandatoryBit();
    return *this;
  }

  OValueStream& operator<<(OValueStream& ov, const Message::Type& t) {
    ov << reinterpret_cast<const int&>(t);
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, Message::Type& t) {
    iv >> reinterpret_cast<int&>(t);
    return iv;
  }

  OValueStream& operator<<(OValueStream& ov, const Message::Direction& t) {
    ov << reinterpret_cast<const int&>(t);
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, Message::Direction& t) {
    iv >> reinterpret_cast<int&>(t);
    return iv;
  }
  
}
