/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_NMMAPPLICATION_HPP
#define NMM_NMMAPPLICATION_HPP

#include "nmm/base/registry/ClientRegistry.hpp"

namespace NMM {
class IRegistry;
class IPluginManager;
class IServerRegistryImpl;
class TCPAcceptor;

/** The NMMApplication instantiate a Server-Registry in the local address space, to simulate a distributed.
  */
class NMMApplication {
public:
/** @name Constructor & Destructor
  */
//@{
/** Constructor
  */
  NMMApplication(int argc, char** argv, bool is_local = true, bool enable_port_listening = true);

/** Destructor
  */
  virtual ~NMMApplication();
//@}

/** @name ClientRegistry & PluginManager
  */
//@{
/** This method returns the ClientRegistry, which can be used to search and request Nodes
  */
  virtual       ClientRegistry& getRegistry();
/** This method returns the ClientRegistry, which can be used to search and request Nodes
  */
  virtual const ClientRegistry& getRegistry() const;

/** This method returns the PluginManager, which can be used to load plugins
  */
  virtual       IPluginManager& getPluginManager();

/** This method returns the PluginManager, which can be used to load plugins
  */
  virtual const IPluginManager& getPluginManager() const;

/** This method returns the ServerRegistry, if the application uses a own ServerRegistry.
  * If not, this method returns NULL
  */
  virtual       IServerRegistryImpl* getServerRegistry();

/** This method returns the ServerRegistry, if the application uses a own ServerRegistry.
  * If not, this method returns NULL
  */
  virtual const IServerRegistryImpl* getServerRegistry() const;

/** enables access to the applications server registry
  */
  void enableServerRegistry(int port);

/** reads the plugin information of the server registry from given file.
  * By default the file ~/.nmm/plugins.<hostname> is used.
  */
  void readRegistryConfig(const string& file = "");

/** writes the plugin information of the server registry to a given file.
  * By default the file ~/.nmm/plugins.<hostname> is used.
  */
  void writeRegistryConfig(const string& file = "");

  void loadPlugins();
//@}
protected:
  void setServerRegistry(IServerRegistryImpl*);

private:
  auto_ptr<ClientRegistry>       __client_registry;     ///< pointer to the ClientRegistry
  auto_ptr<IServerRegistryImpl>  __server_registry;     ///< pointer to the ServerRegistry
  auto_ptr<TCPAcceptor>          __default_acceptor;    ///< The default acceptor.
  auto_ptr<TCPAcceptor>          __registry_acceptor;   ///< accpetor for the server registry
  IPluginManager_var             __plugin_manager;      ///< Pointer to the PluginManager

public: //static members
  static int                     argc;                  ///< argc from main-method
  static char**                  argv;                  ///< argv from main-method
};


}

#endif


