/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_USAGE_HPP
#define NMM_USAGE_HPP NMM_USAGE_HPP

#include "nmm/interfaces/base/IUsageImpl.hpp"
#include "nmm/utils/thread/Thread.hpp"

#include <map>

namespace NMM{

  /**
   *Class to store the usage of a node. This can be used to prioritize applications wich want to use this node
   */
  class Usage :
    public IUsageImpl {
  private:
    typedef pair<int, const IUsageNotify*> usermode_t;

    map<IUsage::usage_t, list<usermode_t> > usages;

    ThreadMutex usage_mutex;
  public:
    Usage();
    ~Usage();

  protected: 

    virtual void setUsage(const IUsage::usage_t &mode, const IUsageNotify* user);
    virtual void unsetUsage(const IUsage::usage_t &mode, const IUsageNotify* user);
    virtual IUsage::usage_t getUsage();

    virtual int getRank(const IUsage::usage_t &mode, const IUsageNotify* user);

    virtual Result isMaster(const IUsage::usage_t &mode, const IUsageNotify* user);
  };
}

#endif
