/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "WParameter.hpp"
#include "nmm/base/serialize/ValueStream.hpp"

namespace NMM 
{
  WParameter::WParameter(const string & pName) : Parameter(pName) 
  {
    m_parameter_type = Parameter::WILDCARD;
  }

  void WParameter::addValue(void */*pValue*/)
  {
    return;
  }

  void WParameter::showParameter() const
  {
    GLOBAL_DEBUG_STREAM2( "Parameter-Name  : " << m_parameter_name << endl);
    GLOBAL_DEBUG_STREAM2( "Parameter-Type  : WILDCARD" << endl);
    GLOBAL_DEBUG_STREAM2( "Parameter-Weight: " << m_parameter_weight << endl);
    GLOBAL_DEBUG_STREAM2( "value : WILDCARD " << "weight: 1" << endl);
  }
  
  ostream& WParameter::showParameter(ostream& os) const
  {
    os << m_parameter_name << "[weight:" << m_parameter_weight << "]=wildcard(*[1])";

    return os;
  }
  
  ostream& WParameter::showParameterVerbose(ostream& os) const
  {
    os << "Parameter-Name  : " << m_parameter_name << endl;
    os << "Parameter-Type  : WILDCARD" << endl;
    os << "Parameter-Weight: " << m_parameter_weight << endl;
    os << "value : WILDCARD " << "weight: 1" << endl;
    return os;
  }
    
  bool WParameter::isValueIncluded(void */*pValue*/) const
  {
    return true;
  }

  void * WParameter::nextValue() const
  {
    return NULL;
  }
  
  bool WParameter::isValueIncluded(void */*lValue*/, void */*hValue*/) const
  {
    return true;
  }

  Parameter * WParameter::duplicate() const
  {
    WParameter *tempParameter = new WParameter(m_parameter_name);
    return tempParameter;
  }

  uint WParameter::getNumberOfValues() const
  {
    return 1;
  }

  void * WParameter::getFirstValue() const
  {
    return NULL;
  }

  float WParameter::getWeight(void */*value*/) const
  {
    return DEFAULT_WEIGHT;
  }

  bool WParameter::isParameterIdentical(const Parameter *reference) const
  {
    if (reference->getParameterType() == Parameter::WILDCARD)
      return true;
    else
      return reference->isParameterIdentical(this);
  }

  bool WParameter::isParameterMatching(const Parameter *reference) const
  {
    if (reference->getParameterType() == Parameter::WILDCARD)
      return true;
    else 
      return reference->isParameterMatching(this);
  }

  Parameter *WParameter::getIntersection(const Parameter *reference) const
  {
   
    if (reference->getParameterType() == Parameter::WILDCARD)
      {
	WParameter * new_parameter = new WParameter(m_parameter_name);
	return new_parameter;
      }
    else
      {
	return reference->getIntersection(this);
      }
  }

  float WParameter::getStandardizedWeight() const
  {
    return DEFAULT_WEIGHT * m_parameter_weight;
  }

  data_value WParameter::getDefaultValue() const
  {
    return m_value;
  }

  void WParameter::fromStream(IValueStream& iv) {
    if ( (iv.getCurrentType() != "WParameter") || (iv.getCurrentNamespace() != "NMM")) {
      GLOBAL_ERROR_STREAM ("Invalid type or namespace:" << endl);
      GLOBAL_ERROR_STREAM ("Current type : " << iv.getCurrentType() << "                Expected type: WParameter" << endl);
      GLOBAL_ERROR_STREAM ("Current namespace : " << iv.getCurrentNamespace() << "      Expected namespace: NMM " << endl);

      iv.nextElement();
      return;
    }

    if (!iv.goDown()) {
      GLOBAL_ERROR_STREAM ("WParameter is empty" << endl);
      iv.nextElement();
      return;
    }

    string p_name;
    float weight = 0.0;

    iv >> p_name;
    iv >> weight;

    setParameterName(p_name);
    setParameterWeight(weight);

    iv.goUp();
    return;
  }

  void WParameter::toStream(OValueStream& ov) const  {
    ov.beginType("WParameter", "NMM");

    ov << getParameterName();
    ov << getParameterWeight();

    ov.endType();
  }

}
