/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_WPARAMETER_HPP
#define _NMM_WPARAMETER_HPP

#include "Parameter.hpp"

namespace NMM {

  /**
     This class stores the Wildcart-Value.
     There 's no real storing for values, its just in virtual value.
     That means a matching with this parameter always returns true.
  **/
  
  class WParameter : public Parameter {

  private:
    
    data_value m_value;

  public:
    /**
       The default constructor.
       \param string parameter_name: the name of the parameter.
    */
    WParameter(const string & parameter_name = "");
    
    /**
       The defautl destructor
    */
    virtual ~WParameter() {};
    
    virtual void addValue(void *pValue);
    
    virtual void showParameter() const;
  
    virtual ostream& showParameter(ostream&) const;
  
    virtual ostream& showParameterVerbose(ostream&) const;

    virtual bool isValueIncluded(void *pValue) const;
    
    virtual bool isValueIncluded(int32_t /*value*/) const {return true;};
    virtual bool isValueIncluded(float /*value*/) const {return true;};
    virtual bool isValueIncluded(const string & ) const {return true;};
    virtual bool isValueIncluded(Format * ) const {return true;};
    
    virtual bool isValueIncluded(void */*lValue*/, void */*hValue*/) const;

    virtual bool isValueIncluded(float , float ) const {return true;};
    virtual bool isValueIncluded(int32_t , int32_t ) const {return true;};

    virtual bool isRangeMatching(float , float ) const {return true;};
    virtual bool isRangeMatching(int32_t , int32_t ) const {return true;};

  
    virtual void *nextValue() const;

    virtual Parameter * duplicate() const;

    virtual uint getNumberOfValues() const;

    virtual void *getFirstValue() const;

    virtual float getWeight(void * value) const;

    virtual bool isParameterIdentical(const Parameter *reference) const;

    virtual bool isParameterMatching(const Parameter * reference) const;

    virtual Parameter *getIntersection(const Parameter *reference) const;

    virtual float getStandardizedWeight() const;

    virtual data_value getDefaultValue() const;

    virtual void fromStream(IValueStream& ) ;
    virtual void toStream(OValueStream&) const  ;

  };
}

#endif
