/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <unistd.h> // for close()
#include <iostream>
#include "TCPAcceptor.hpp"
#include "TCPStrategy.hpp"
#include "nmm/base/registry/ReferenceManager.hpp"
#include "nmm/base/registry/LocalRegistry.hpp"
#include "nmm/base/registry/PluginManager.hpp"
#include "nmm/base/TSFactory.hpp"
#include "nmm/utils/utils.hpp"

namespace NMM {

TCPAcceptor::TCPAcceptor(unsigned int port) {
  addr.setPort(port);
  open(addr);
  SNetReactor::getInstance().registerSocket( getSocket(), this);

  /*Begin> Register the default address for tcp connections */
  TCPAddress address;
  address.getINetAddr().setPort(port);
  address.getINetAddr().setAddress(NMM::getHostname().c_str());

  STSFactory::getInstance().setDefaultAddress(TCPStrategy::getStaticStrategyType(), &address);
  /*End< */
}

TCPAcceptor::TCPAcceptor(unsigned int begin_port, unsigned int end_port ) {
  addr.setPort(begin_port);
  tryOpen(addr, begin_port, end_port);
  SNetReactor::getInstance().registerSocket( getSocket(), this);

  /*Begin> Register the default address for tcp connections */
  TCPAddress address;
  address.getINetAddr().setPort(addr.getPort());
  address.getINetAddr().setAddress(NMM::getHostname().c_str());

  STSFactory::getInstance().setDefaultAddress(TCPStrategy::getStaticStrategyType(), &address);
  /*End< */
}



TCPAcceptor::~TCPAcceptor() {
  SNetReactor::getInstance().unregisterSocket( getSocket() );
  ::close( getSocket() );
}

NetHandler::NetResult TCPAcceptor::receiveData(int) {
  GLOBAL_MESSAGE_STREAM("TCPConnection established" << endl);
  
  /* create a new TCPStrategy */
  TCPStrategy* strategy = new TCPStrategy();
  /* This object receives the first event */
  strategy->setEventReceiver(this);

  /* accept the connection */
  accept(strategy->getStreamSocket());

  ITCPControl_var itcp (strategy->getInterface<ITCPControl>());
  itcp->enableNoDelay(true);

  /* register the Socket by the NetReactor */
  SNetReactor::getInstance().internalRegisterSocket( strategy->getStreamSocket().getSocket(), strategy);
  return VOID;
}

}//end namespace


