/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#include "NodeFactory.hpp"
#include <algorithm>

namespace NMM {

NodeFactory::NodeFactory(const char* name, NodeFactory* parent)
  : NamedObject(name), __parent(parent), __factory_id(name)
{
  /* If a parent is set register the Factory */
  if (parent)
    parent->addNodeFactory(*this);
}

NodeFactory::~NodeFactory() {
  DEBUG_STREAM(">Destroy " << getName() << " : " << this << endl);
  if (getParent())
    getParent()->removeNodeFactory(*this);

  /*Begin> reset the parent of all children to 0*/
  list<NodeFactory*> child_list = getChildList();
  for (list<NodeFactory*>::iterator it = child_list.begin(); it != child_list.end(); ++it)
  {
    (*it)->setParent(0);
  }
  /*End< */

  DEBUG_STREAM("<Destroy " << getName() << " : " << this << endl);
}


void  NodeFactory::setParent(NodeFactory* factory) {
  if (getParent()) {
    getParent()->removeNodeFactory(*this);
  }
  __parent = factory;
}

NodeFactory* NodeFactory::findNodeFactory(const NodeFactoryID& factory_id) {
  if (factory_id == getFactoryID())
    return this;

  NodeFactory* tmp_factory = 0;
  for (list<NodeFactory*>::iterator it = __child_list.begin(); it != __child_list.end(); ++it)
  {
    tmp_factory = (*it)->findNodeFactory(factory_id);
    if (tmp_factory)
      return tmp_factory;
  }

  return 0;
}

void NodeFactory::removeNodeFactory(NodeFactory& fac) {
  __child_list.remove(&fac);
}

void NodeFactory::addNodeFactory(NodeFactory& fac) {
  if (this == &fac)
    return;

  /* Begin> insert the NodeFactory in the child list if it does not exist */
  if (find(__child_list.begin(), __child_list.end(), &fac) != __child_list.end()) {
    WARNING_STREAM("Factory is already child of this NodeFactory" << endl);
    return;
  } else {
    __child_list.push_back(&fac);
  }
  /* End< */
}


}//end namespace
