/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_REGISTRYMEMENTO_HPP
#define NMM_REGISTRYMEMENTO_HPP

#include <string>

#include "nmm/base/serialize/ComplexValue.hpp"

#include "RegistryTypedef.hpp"
#include "NodeDescription.hpp"

namespace NMM {

class MementoStruct : public SerializedValue {
public:
  MementoStruct();
  MementoStruct(const MementoStruct&);
  
  virtual MementoStruct* copy()                 const { return new MementoStruct(*this); }
  
  const NodeDescription& getNodeDescription()   const { return __node_description; }
  const ObjectID&        getObjectID()          const { return __object_id.getRValue() ; }
  unsigned int           getMaxInstanceCount()  const { return __max_instance_count.getRValue(); }
  unsigned int           getPluginIndex()       const { return __plugin_index.getValue(); }

  void setNodeDescription( const NodeDescription& nd )        { __node_description = nd; }
  void setObjectID( const ObjectID& object_id )               { __object_id.setValue( object_id ); }
  void setMaxInstanceCount( unsigned int max_instance_count)  { __max_instance_count.setValue( max_instance_count); }
  void setPluginIndex( unsigned int index)                    { __plugin_index.setValue( index ); }

private:
  void registerAttributes();
private:
  NodeDescription         __node_description;
  TValue<ObjectID>        __object_id;
  TValue<unsigned int>    __max_instance_count;
  TValue<unsigned int>    __plugin_index;
};

class RegistryMemento : public SerializedValue {
public:                          
  RegistryMemento();
  RegistryMemento(const RegistryMemento& rm);
  virtual RegistryMemento* copy()             const { return new RegistryMemento(*this); }
 
  bool isPluginRequired()                     const { return ! __plugin.getRValue().empty(); }
  const string& getPluginName()               const { return __plugin.getRValue(); }
  void setPluginName(const string& plugin)          { __plugin.setValue(plugin); }

  const RegistryID& getRegistryID()           const { return __registry_id.getRValue(); }
  void setRegistryID(const RegistryID& reg_id)      { __registry_id.setValue(reg_id); }

  const list<MementoStruct>& getMementoList() const { return __memento_list.getRValue(); } 
  void addMementoStruct(const MementoStruct& memento_struct) { __memento_list.getRValue().push_back(memento_struct); }
private:
  void registerAttributes(); 
private:
  TValue<string>                   __plugin;
  TValue<RegistryID>               __registry_id;
  TValue< list<MementoStruct> >    __memento_list;
};

OValueStream& operator<<(OValueStream& ov, const list<MementoStruct>& memento_list);
IValueStream& operator>>(IValueStream& iv, list<MementoStruct>& memento_list);

}//end namespace
#endif

