/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <iostream> 
#include "QualityParameter.hpp"

namespace NMM {

QualityParameter::QualityParameter() : SerializedValue("QualityParameter", "NMM") {
  registerValues();
  parameter_type = NIL;
  i_val = 0;
  f_val = 0.0F;
  s_val = string();
}

QualityParameter::QualityParameter(int val) : SerializedValue("QualityParameter", "NMM")  {
  registerValues();
  parameter_type = INT32;
  i_val = val;
}

QualityParameter::QualityParameter(float val) : SerializedValue("QualityParameter", "NMM")  {
  registerValues();
  parameter_type = FLOAT;
  f_val = val;
}

QualityParameter::QualityParameter(string val) : SerializedValue("QualityParameter", "NMM") {
  registerValues();
  parameter_type = STRING;
  s_val = val;
}

QualityParameter::QualityParameter(const QualityParameter& qp) : SerializedValue(qp) {
  registerValues();
  *this = qp;
}
    
int    QualityParameter::getIntValue() {
  if (parameter_type.getRValue() != INT32) {
    return 0;
  }
  
  return i_val.getRValue();
}

float  QualityParameter::getFloatValue() {
  if (parameter_type.getRValue() != FLOAT) {
    return 0;
  }
  
  return f_val.getRValue();
}

string QualityParameter::getStringValue() {
  if (parameter_type.getRValue() != STRING) {
    return 0;
  }
  
  return s_val.getRValue();
}

bool QualityParameter::operator< (const QualityParameter& B) {
  // float and int32 can be compared 
  // strings are always larger as float or int32
  
  // identical types:
  if (parameter_type.getRValue() == B.parameter_type.getRValue()) {
    switch (parameter_type.getRValue()) {
      case NIL:
        return false; // no comparison possible
      case INT32:
        return (i_val.getRValue() < B.i_val.getRValue());
      case FLOAT:
        return (f_val.getRValue() < B.f_val.getRValue());
      case STRING:
        return (s_val.getRValue() < B.s_val.getRValue());
    }
  }
      
  // different types      
  if (parameter_type.getRValue() == INT32) {
    if (B.parameter_type.getRValue() == FLOAT) {
      return (static_cast<float>(i_val.getRValue()) < B.f_val.getRValue());
    }
		if (B.parameter_type.getRValue() == STRING) {
			return true;
		}
	}
	
	if (parameter_type.getRValue() == FLOAT) {
		if (B.parameter_type.getRValue() == INT32) {
			return (f_val.getRValue() < static_cast<float>(B.i_val.getRValue()));
		}
		if (B.parameter_type.getRValue() == STRING) {
			return true;
		}
	}
	
	if (parameter_type.getRValue() == STRING) {
		return false;
	}
	
	cerr << "QualityParameter comparison: did I forget a case?\n";
	
	return false;
}

QualityParameter& QualityParameter::operator-= (const QualityParameter& A) {
  if (parameter_type.getRValue() != A.parameter_type.getRValue()) {
    return *this; // only process parameteres of the same type
  }
  
  
  switch (parameter_type.getRValue()) {
    case INT32:
      i_val.getRValue() -= A.i_val.getRValue();
      break;
    case FLOAT:
      f_val.getRValue() -= A.f_val.getRValue();
      break;
    default:
      return *this;
  }
  return *this;
}

void QualityParameter::registerValues() {
  insertArgument(&i_val);
  insertArgument(&f_val);
  insertArgument(&s_val);
  insertArgument(&parameter_type);
}


ostream& operator<< (ostream& stream, const QualityParameter& qp) {
  switch (qp.parameter_type.getRValue()) {
    case QualityParameter::NIL:
      stream << "''" ;
      break;
    case QualityParameter::INT32:
      stream << qp.i_val.getRValue();
      break;
    case QualityParameter::FLOAT:
      stream << qp.f_val.getRValue();
      break;
    case QualityParameter::STRING:
      stream << qp.s_val.getRValue();
      break;  
  }
  return stream;
}

}
