/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Eric Peters <epeters@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include <iostream>
#include "DiffTimer.hpp"

namespace NMM {

  void DiffTimer::reset() {
    start_time = this->getTime();
    DEBUG_STREAM2 (" Reset. " << endl);
    DEBUG_STREAM2 (" Time: " << start_time.sec << endl);
  };

  DiffTimer::DiffTimer()
    : TimedElement(), NamedObject("Timer") {
    this->reset();
  };

  DiffTimer::~DiffTimer() {};

  float DiffTimer::getDiff() {
    float secs = this->getTime().sec - start_time.sec;
    float nanos = this->getTime().nsec - start_time.nsec;
    if (nanos < 0) {
      nanos = 1000000000 + nanos;
      secs--;
    }
    secs = ((int)((secs + (nanos/1000000000))*1000))/1000.0;
    return secs;
  }
    
}

