/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "UserTime.hpp"
#include "nmm/base/serialize/Value.hpp"

namespace NMM {

  UserTime::UserTime(const Time& time) {
    hour = time.sec / 3600;
    min  = (time.sec - hour * 3600) / 60;
    sec  = time.sec - hour * 3600 - min * 60;
    msec = time.nsec / 1000000L;
  };
    
  UserTime::UserTime(const Interval& interval) {
    hour = interval.sec / 3600;
    min  = (interval.sec - hour * 3600) / 60;
    sec  = interval.sec - hour * 3600 - min * 60;
    msec = interval.nsec / 1000000L;
  };
  
  void UserTime::output(std::ostream& stream, bool with_seconds, bool with_microseconds) const {
    stream.width(2);
    stream.fill('0');
    stream << hour << ":";
    stream.width(2);
    stream.fill('0');
    stream << min;
    if(with_seconds) {
      stream << ":";
      stream.width(2);
      stream.fill('0');
      stream << sec;
    }
    if(with_microseconds) {
      stream << "+";
      stream.width(3);
      stream.fill('0');
      stream << msec;
      stream << "msec";
    }
  }
  
  ostream& operator<< (ostream& stream, const UserTime& time) {
    time.output(stream, true, true);
    return stream;
  }
  
  
  OValueStream& operator<<(OValueStream& ov, const UserTime& u_time) {
    ov.beginType("UserTime", "NMM");
    ov << u_time.hour;       //!< The hours
    ov << u_time.min;        //!< The minutes
    ov << u_time.sec;        //!< The seconds
    ov << u_time.msec;       //!< The milliseconds
    ov.endType();
    return ov;
  }
  
  IValueStream& operator>>(IValueStream& ivs, UserTime& u_time) {
    if ( (ivs.getCurrentType() != "UserTime") || (ivs.getCurrentNamespace() != "NMM")) {
      GLOBAL_ERROR_STREAM (" SerializedValue > Invalid type or namespace:" << endl);
      GLOBAL_ERROR_STREAM (" SerializedValue > Current type : " << ivs.getCurrentType() << "                Expected type: UserTime" << endl);
      GLOBAL_ERROR_STREAM (" SerializedValue > Current namespace : " << ivs.getCurrentNamespace() << "      Expected namespace: NMM" << endl);
      ivs.nextElement();
      return ivs;
    }
    
    if (!ivs.goDown()) {
      GLOBAL_ERROR_STREAM ("UserTime has no values");
      ivs.nextElement();
      return ivs;
    }
    
    ivs >> u_time.hour;       //!< The hours
    ivs >> u_time.min;        //!< The minutes
    ivs >> u_time.sec;        //!< The seconds
    ivs >> u_time.msec;       //!< The milliseconds
    
    ivs.goUp();
    return ivs;
  }
  
  TValueConstructor<UserTime> user_time_prototype("UserTime", "NMM");
  
}
