
#ifndef NMM_IMMBoxState_HPP
#define NMM_IMMBoxState_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/base/registry/NodeDescription.hpp"
#include "nmm/interfaces/base/INode.hpp"
#include "nmm/interfaces/video/osd/manager/ITextView.hpp"

namespace NMM
{

  class IMMBoxState : virtual public Interface { 
  public:
    IMMBoxState() {  setTypeInfo(TypeInfo(IMMBoxState::IMMBoxState_type, "NMM")); };

    IMMBoxState(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IMMBoxState::IMMBoxState_type, "NMM")); setEventReceiver(parent); };

    virtual ~IMMBoxState() {};

    static const char* IMMBoxState_type;

    virtual  string getInterfaceType() const { return IMMBoxState_type; }

    Interface* copy() const { return new IMMBoxState(*this); }
    static const char* initialize_event;
    static const char* uninitialize_event;
    static const char* create_event;
    static const char* kill_event;

    virtual Result initialize(const string& message = "")
      throw(const Exception&);

    virtual Result uninitialize()
      throw(const Exception&);

    virtual Result create(const string& message = "")
      throw(const Exception&);

    virtual Result kill()
      throw(const Exception&);


  };

  typedef auto_ptr<IMMBoxState> IMMBoxState_var;



  class IDistMMBoxState : virtual public IMMBoxState { 
  public:
    IDistMMBoxState() {  setTypeInfo(TypeInfo(IDistMMBoxState::IDistMMBoxState_type, "NMM")); };

    IDistMMBoxState(NMMObject& parent) : IMMBoxState(parent) {  setTypeInfo(TypeInfo(IDistMMBoxState::IDistMMBoxState_type, "NMM")); setEventReceiver(parent); };

    virtual ~IDistMMBoxState() {};

    static const char* IDistMMBoxState_type;

    virtual  string getInterfaceType() const { return IDistMMBoxState_type; }

    Interface* copy() const { return new IDistMMBoxState(*this); }
    static const char* createNode_event;
    static const char* getAudioNode_event;
    static const char* getOsdNode_event;
    static const char* getDisplayNode_event;
    static const char* setNeedVideo_event;
    static const char* setNeedAudio_event;

    virtual INode* createNode(const NodeDescription& ndescr)
      throw(const Exception&);

    virtual INode* getAudioNode()
      throw(const Exception&);

    virtual INode* getOsdNode()
      throw(const Exception&);

    virtual INode* getDisplayNode()
      throw(const Exception&);

    virtual void setNeedVideo(const bool& b)
      throw(const Exception&);

    virtual void setNeedAudio(const bool& b)
      throw(const Exception&);


  };

  typedef auto_ptr<IDistMMBoxState> IDistMMBoxState_var;



  class IDistMP3State : virtual public IDistMMBoxState { 
  public:
    IDistMP3State() {  setTypeInfo(TypeInfo(IDistMP3State::IDistMP3State_type, "NMM")); };

    IDistMP3State(NMMObject& parent) : IDistMMBoxState(parent) {  setTypeInfo(TypeInfo(IDistMP3State::IDistMP3State_type, "NMM")); setEventReceiver(parent); };

    virtual ~IDistMP3State() {};

    static const char* IDistMP3State_type;

    virtual  string getInterfaceType() const { return IDistMP3State_type; }

    Interface* copy() const { return new IDistMP3State(*this); }
    static const char* getMPEGAudioDecodeNode_event;
    static const char* getMPEGAudioInputJack_event;
    static const char* getPlaybackNode_event;
    static const char* addTextLine_event;
    static const char* selectTextLine_event;

    virtual INode* getMPEGAudioDecodeNode()
      throw(const Exception&);

    virtual IInputJack* getMPEGAudioInputJack()
      throw(const Exception&);

    virtual INode* getPlaybackNode()
      throw(const Exception&);

    virtual void addTextLine(const string& line)
      throw(const Exception&);

    virtual void selectTextLine(const unsigned int& num)
      throw(const Exception&);


  };

  typedef auto_ptr<IDistMP3State> IDistMP3State_var;




}

#endif
