
#ifndef NMM_IIcecastImpl_HPP
#define NMM_IIcecastImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IIcecast.hpp"

#include "nmm/interfaces/base/INodeImpl.hpp"

namespace NMM
{
  /** Interface for IceCast sink node.
   */

  class IIcecastImpl  : public virtual NMMObject { 
  public:
    IIcecastImpl() { addInterface(new IIcecast(*this)); registerEvents(); };

    virtual ~IIcecastImpl() {};

  protected:
    /** Set the IP address of the IceCast streaming server.
     */

    virtual Result setServerIP(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the port of the IceCast streaming server.
     */

    virtual Result setServerPort(const int& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the source password for the IceCast streaming server.
     */

    virtual Result setServerPassword(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the user name for the IceCast streaming server.
     */

    virtual Result setUser(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the mount point.
     */

    virtual Result setMountpoint(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the source title.
     */

    virtual Result setTitle(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set the source name.
     */

    virtual Result setStreamName(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set the source genre.
     */

    virtual Result setGenre(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set the source description.
     */

    virtual Result setDescription(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set the source URL.
     */

    virtual Result setURL(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set public mode.
     */

    virtual Result setPublic(const int& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set whether to send title.
     */

    virtual Result setSendTitle(const bool& value) /* limited(INode::INITIALIZED) */ = 0;


  private:
    void registerEvents();
  };

}

#endif
