
#include "ILADSPA.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  bool ILADSPA::addLADSPAPlugin(const string& value, string& name, unsigned int& execpos)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<string > value_value(value, false);
    TInOutValue<string > name_value;
    name_value.setValue(name);
    TInOutValue<unsigned int > execpos_value;
    execpos_value.setValue(execpos);
    Event event("ILADSPA::addLADSPAPlugin", &value_value, &name_value, &execpos_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    name = name_value.getRValue();
    execpos = execpos_value.getRValue();
    return return_value.getRValue();
  }
  
  void ILADSPA::removeLADSPAPluginByName(const string& name)
      throw(const Exception&)
  {
    TInValue<string > name_value(name, false);
    Event event("ILADSPA::removeLADSPAPluginByName", &name_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ILADSPA::removeLADSPAPluginByIndex(const unsigned int& index)
      throw(const Exception&)
  {
    TInValue<unsigned int > index_value(index, false);
    Event event("ILADSPA::removeLADSPAPluginByIndex", &index_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int ILADSPA::getLADSPAPluginCount()
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("ILADSPA::getLADSPAPluginCount");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ILADSPA::setConfigFilename(const string& value)
      throw(const Exception&)
  {
    TInValue<string > value_value(value, false);
    Event event("ILADSPA::setConfigFilename", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ILADSPA::setConfigProfileByName(const string& name, const string& profile)
      throw(const Exception&)
  {
    TInValue<string > name_value(name, false);
    TInValue<string > profile_value(profile, false);
    Event event("ILADSPA::setConfigProfileByName", &name_value, &profile_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ILADSPA::setConfigProfileByIndex(const unsigned int& index, const string& profile)
      throw(const Exception&)
  {
    TInValue<unsigned int > index_value(index, false);
    TInValue<string > profile_value(profile, false);
    Event event("ILADSPA::setConfigProfileByIndex", &index_value, &profile_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ILADSPA::resetToBaseValuesByName(const string& name)
      throw(const Exception&)
  {
    TInValue<string > name_value(name, false);
    Event event("ILADSPA::resetToBaseValuesByName", &name_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ILADSPA::resetToBaseValuesByIndex(const unsigned int& index)
      throw(const Exception&)
  {
    TInValue<unsigned int > index_value(index, false);
    Event event("ILADSPA::resetToBaseValuesByIndex", &index_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ILADSPA::setControlInValueByName(const string& name, const unsigned int& port, const float& value)
      throw(const Exception&)
  {
    TInValue<string > name_value(name, false);
    TInValue<unsigned int > port_value(port, false);
    TInValue<float > value_value(value, false);
    Event event("ILADSPA::setControlInValueByName", &name_value, &port_value, &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ILADSPA::setControlInValueByIndex(const unsigned int& index, const unsigned int& port, const float& value)
      throw(const Exception&)
  {
    TInValue<unsigned int > index_value(index, false);
    TInValue<unsigned int > port_value(port, false);
    TInValue<float > value_value(value, false);
    Event event("ILADSPA::setControlInValueByIndex", &index_value, &port_value, &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  float ILADSPA::getControlInValueByName(const string& name, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    TInValue<string > name_value(name, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::getControlInValueByName", &name_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  float ILADSPA::getControlInValueByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    TInValue<unsigned int > index_value(index, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::getControlInValueByIndex", &index_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ILADSPA::getControlInNameByName(const string& name, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<string > name_value(name, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::getControlInNameByName", &name_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ILADSPA::getControlInNameByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<unsigned int > index_value(index, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::getControlInNameByIndex", &index_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ILADSPA::setControlInBaseByName(const string& name, const unsigned int& port, const float& value)
      throw(const Exception&)
  {
    TInValue<string > name_value(name, false);
    TInValue<unsigned int > port_value(port, false);
    TInValue<float > value_value(value, false);
    Event event("ILADSPA::setControlInBaseByName", &name_value, &port_value, &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ILADSPA::setControlInBaseByIndex(const unsigned int& index, const unsigned int& port, const float& value)
      throw(const Exception&)
  {
    TInValue<unsigned int > index_value(index, false);
    TInValue<unsigned int > port_value(port, false);
    TInValue<float > value_value(value, false);
    Event event("ILADSPA::setControlInBaseByIndex", &index_value, &port_value, &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  float ILADSPA::getControlInBaseByName(const string& name, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    TInValue<string > name_value(name, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::getControlInBaseByName", &name_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  float ILADSPA::getControlInBaseByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    TInValue<unsigned int > index_value(index, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::getControlInBaseByIndex", &index_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int ILADSPA::getLADSPAPluginIndex(const string& name)
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > name_value(name, false);
    Event event("ILADSPA::getLADSPAPluginIndex", &name_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ILADSPA::getLADSPAPluginName(const unsigned int& index)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<unsigned int > index_value(index, false);
    Event event("ILADSPA::getLADSPAPluginName", &index_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int ILADSPA::getControlInCountByName(const string& name)
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > name_value(name, false);
    Event event("ILADSPA::getControlInCountByName", &name_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int ILADSPA::getControlInCountByIndex(const unsigned int& index)
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<unsigned int > index_value(index, false);
    Event event("ILADSPA::getControlInCountByIndex", &index_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool ILADSPA::hasControlInUpperBoundByName(const string& name, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<string > name_value(name, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::hasControlInUpperBoundByName", &name_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool ILADSPA::hasControlInUpperBoundByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<unsigned int > index_value(index, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::hasControlInUpperBoundByIndex", &index_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool ILADSPA::hasControlInLowerBoundByName(const string& name, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<string > name_value(name, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::hasControlInLowerBoundByName", &name_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool ILADSPA::hasControlInLowerBoundByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<unsigned int > index_value(index, false);
    TInValue<unsigned int > port_value(port, false);
    Event event("ILADSPA::hasControlInLowerBoundByIndex", &index_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ILADSPA::getControlInBoundsByName(const string& name, const unsigned int& port, float& lower, float& upper)
      throw(const Exception&)
  {
    TInValue<string > name_value(name, false);
    TInValue<unsigned int > port_value(port, false);
    TOutValue<float > lower_value(lower);
    TOutValue<float > upper_value(upper);
    Event event("ILADSPA::getControlInBoundsByName", &name_value, &port_value, &lower_value, &upper_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    lower = lower_value.getRValue();
    upper = upper_value.getRValue();
  }
  
  void ILADSPA::getControlInBoundsByIndex(const unsigned int& index, const unsigned int& port, float& lower, float& upper)
      throw(const Exception&)
  {
    TInValue<unsigned int > index_value(index, false);
    TInValue<unsigned int > port_value(port, false);
    TOutValue<float > lower_value(lower);
    TOutValue<float > upper_value(upper);
    Event event("ILADSPA::getControlInBoundsByIndex", &index_value, &port_value, &lower_value, &upper_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    lower = lower_value.getRValue();
    upper = upper_value.getRValue();
  }
  
  string ILADSPA::getLADSPAPluginRealNameByName(const string& name)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<string > name_value(name, false);
    Event event("ILADSPA::getLADSPAPluginRealNameByName", &name_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ILADSPA::getLADSPAPluginRealNameByIndex(const unsigned int& index)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<unsigned int > index_value(index, false);
    Event event("ILADSPA::getLADSPAPluginRealNameByIndex", &index_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ILADSPA::getLADSPAPluginLabelByName(const string& name)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<string > name_value(name, false);
    Event event("ILADSPA::getLADSPAPluginLabelByName", &name_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ILADSPA::getLADSPAPluginLabelByIndex(const unsigned int& index)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<unsigned int > index_value(index, false);
    Event event("ILADSPA::getLADSPAPluginLabelByIndex", &index_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ILADSPA::getLADSPAPluginLibraryNameByName(const string& name)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<string > name_value(name, false);
    Event event("ILADSPA::getLADSPAPluginLibraryNameByName", &name_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ILADSPA::getLADSPAPluginLibraryNameByIndex(const unsigned int& index)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<unsigned int > index_value(index, false);
    Event event("ILADSPA::getLADSPAPluginLibraryNameByIndex", &index_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ILADSPA::ILADSPA_type = "ILADSPA";
  TIFactoryObject<ILADSPA > __ILADSPA_prototype("ILADSPA");


  const char* ILADSPA::addLADSPAPlugin_event = "ILADSPA::addLADSPAPlugin";
  const char* ILADSPA::removeLADSPAPluginByName_event = "ILADSPA::removeLADSPAPluginByName";
  const char* ILADSPA::removeLADSPAPluginByIndex_event = "ILADSPA::removeLADSPAPluginByIndex";
  const char* ILADSPA::getLADSPAPluginCount_event = "ILADSPA::getLADSPAPluginCount";
  const char* ILADSPA::setConfigFilename_event = "ILADSPA::setConfigFilename";
  const char* ILADSPA::setConfigProfileByName_event = "ILADSPA::setConfigProfileByName";
  const char* ILADSPA::setConfigProfileByIndex_event = "ILADSPA::setConfigProfileByIndex";
  const char* ILADSPA::resetToBaseValuesByName_event = "ILADSPA::resetToBaseValuesByName";
  const char* ILADSPA::resetToBaseValuesByIndex_event = "ILADSPA::resetToBaseValuesByIndex";
  const char* ILADSPA::setControlInValueByName_event = "ILADSPA::setControlInValueByName";
  const char* ILADSPA::setControlInValueByIndex_event = "ILADSPA::setControlInValueByIndex";
  const char* ILADSPA::getControlInValueByName_event = "ILADSPA::getControlInValueByName";
  const char* ILADSPA::getControlInValueByIndex_event = "ILADSPA::getControlInValueByIndex";
  const char* ILADSPA::getControlInNameByName_event = "ILADSPA::getControlInNameByName";
  const char* ILADSPA::getControlInNameByIndex_event = "ILADSPA::getControlInNameByIndex";
  const char* ILADSPA::setControlInBaseByName_event = "ILADSPA::setControlInBaseByName";
  const char* ILADSPA::setControlInBaseByIndex_event = "ILADSPA::setControlInBaseByIndex";
  const char* ILADSPA::getControlInBaseByName_event = "ILADSPA::getControlInBaseByName";
  const char* ILADSPA::getControlInBaseByIndex_event = "ILADSPA::getControlInBaseByIndex";
  const char* ILADSPA::getLADSPAPluginIndex_event = "ILADSPA::getLADSPAPluginIndex";
  const char* ILADSPA::getLADSPAPluginName_event = "ILADSPA::getLADSPAPluginName";
  const char* ILADSPA::getControlInCountByName_event = "ILADSPA::getControlInCountByName";
  const char* ILADSPA::getControlInCountByIndex_event = "ILADSPA::getControlInCountByIndex";
  const char* ILADSPA::hasControlInUpperBoundByName_event = "ILADSPA::hasControlInUpperBoundByName";
  const char* ILADSPA::hasControlInUpperBoundByIndex_event = "ILADSPA::hasControlInUpperBoundByIndex";
  const char* ILADSPA::hasControlInLowerBoundByName_event = "ILADSPA::hasControlInLowerBoundByName";
  const char* ILADSPA::hasControlInLowerBoundByIndex_event = "ILADSPA::hasControlInLowerBoundByIndex";
  const char* ILADSPA::getControlInBoundsByName_event = "ILADSPA::getControlInBoundsByName";
  const char* ILADSPA::getControlInBoundsByIndex_event = "ILADSPA::getControlInBoundsByIndex";
  const char* ILADSPA::getLADSPAPluginRealNameByName_event = "ILADSPA::getLADSPAPluginRealNameByName";
  const char* ILADSPA::getLADSPAPluginRealNameByIndex_event = "ILADSPA::getLADSPAPluginRealNameByIndex";
  const char* ILADSPA::getLADSPAPluginLabelByName_event = "ILADSPA::getLADSPAPluginLabelByName";
  const char* ILADSPA::getLADSPAPluginLabelByIndex_event = "ILADSPA::getLADSPAPluginLabelByIndex";
  const char* ILADSPA::getLADSPAPluginLibraryNameByName_event = "ILADSPA::getLADSPAPluginLibraryNameByName";
  const char* ILADSPA::getLADSPAPluginLibraryNameByIndex_event = "ILADSPA::getLADSPAPluginLibraryNameByIndex";


}


