
#include "ICompositeStrategy.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  ITransportStrategy* ICompositeStrategy::getBufferChildStrategy() const
      throw(const Exception&)
  {
    TOutValue<ITransportStrategy* > return_value;
    Event event("ICompositeStrategy::getBufferChildStrategy");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* ICompositeStrategy::getEventChildStrategy() const
      throw(const Exception&)
  {
    TOutValue<ITransportStrategy* > return_value;
    Event event("ICompositeStrategy::getEventChildStrategy");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ICompositeStrategy::setMode(const ICompositeStrategy::Mode& mode)
      throw(const Exception&)
  {
    TInValue<ICompositeStrategy::Mode > mode_value(mode, false);
    Event event("ICompositeStrategy::setMode", &mode_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ICompositeStrategy::setLossInterval(const unsigned int& loss_interval)
      throw(const Exception&)
  {
    TInValue<unsigned int > loss_interval_value(loss_interval, false);
    Event event("ICompositeStrategy::setLossInterval", &loss_interval_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ICompositeStrategy::connectToCompositeBuffer() 
      throw(const Exception&, const BindingException&)
  {
    Event event("ICompositeStrategy::connectToCompositeBuffer");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void ICompositeStrategy::connectToCompositeEvent() 
      throw(const Exception&, const BindingException&)
  {
    Event event("ICompositeStrategy::connectToCompositeEvent");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void ICompositeStrategy::eventReceived(const int& stream_counter) /* in-stream */ 
      throw(const Exception&)
  {
    TInValue<int > stream_counter_value(stream_counter, false);
    Event event("ICompositeStrategy::eventReceived", &stream_counter_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* ICompositeStrategy::ICompositeStrategy_type = "ICompositeStrategy";
  TIFactoryObject<ICompositeStrategy > __ICompositeStrategy_prototype("ICompositeStrategy");
  TValueConstructor< ICompositeStrategy::Mode > __ICompositeStrategy_Mode_prototype(TypeInfo("ICompositeStrategy::Mode", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const ICompositeStrategy::Mode& t)
  {
    ov.beginType(TypeInfo("ICompositeStrategy::Mode", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, ICompositeStrategy::Mode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("ICompositeStrategy::Mode", "NMM")) {
      throw InvalidTypeException(TypeInfo("ICompositeStrategy::Mode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* ICompositeStrategy::getBufferChildStrategy_event = "ICompositeStrategy::getBufferChildStrategy";
  const char* ICompositeStrategy::getEventChildStrategy_event = "ICompositeStrategy::getEventChildStrategy";
  const char* ICompositeStrategy::setMode_event = "ICompositeStrategy::setMode";
  const char* ICompositeStrategy::setLossInterval_event = "ICompositeStrategy::setLossInterval";
  const char* ICompositeStrategy::connectToCompositeBuffer_event = "ICompositeStrategy::connectToCompositeBuffer";
  const char* ICompositeStrategy::connectToCompositeEvent_event = "ICompositeStrategy::connectToCompositeEvent";
  const char* ICompositeStrategy::eventReceived_event = "ICompositeStrategy::eventReceived";
  Event* ICompositeStrategy::create_eventReceived(const int& stream_counter)
  {
    TInValue<int >* stream_counter_value = new TInValue<int >();
    stream_counter_value->setValue(stream_counter);

    return new Event("ICompositeStrategy::eventReceived", stream_counter_value);
  }



}


