
#include "IConnectionManager.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IConnectionManager::handleConnect(const TSAddress* remote_addr, TSAddress*& local_addr) const 
      throw(const Exception&, const BindingException&)
  {
    TInValue<TSAddress* > remote_addr_value (const_cast<TSAddress* >(remote_addr), false);
    TOutValue<TSAddress* > local_addr_value(local_addr);
    Event event("IConnectionManager::handleConnect", &remote_addr_value, &local_addr_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    local_addr = local_addr_value.getRValue();
  }
  
  Result IConnectionManager::sendReliableEvent(const NMMReference& ts_reference, Event& e) const 
      throw(const Exception&, const BindingException&)
  {
    TOutValue<Result > return_value;
    TInValue<NMMReference > ts_reference_value(ts_reference, false);
    TInOutValue<Event > e_value;
    e_value.setValue(e);
    Event event("IConnectionManager::sendReliableEvent", &ts_reference_value, &e_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    e = e_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IConnectionManager::sendReliableConfigEvent(const NMMReference& receiver, Event& e) const 
      throw(const Exception&, const BindingException&)
  {
    TOutValue<Result > return_value;
    TInValue<NMMReference > receiver_value(receiver, false);
    TInOutValue<Event > e_value;
    e_value.setValue(e);
    Event event("IConnectionManager::sendReliableConfigEvent", &receiver_value, &e_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    e = e_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IConnectionManager::sendReliableInstreamMessage(const NMMReference& receiver, CEvent& e) const 
      throw(const Exception&, const BindingException&)
  {
    TOutValue<Result > return_value;
    TInValue<NMMReference > receiver_value(receiver, false);
    TInOutValue<CEvent > e_value;
    e_value.setValue(e);
    Event event("IConnectionManager::sendReliableInstreamMessage", &receiver_value, &e_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    e = e_value.getRValue();
    return return_value.getRValue();
  }
  
  list<string >  IConnectionManager::getSupportedStrategies(const Format& format) const
      throw(const Exception&)
  {
    TOutValue<list<string >  > return_value;
    TInValue<Format > format_value(format, false);
    Event event("IConnectionManager::getSupportedStrategies", &format_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IConnectionManager::IConnectionManager_type = "IConnectionManager";
  TIFactoryObject<IConnectionManager > __IConnectionManager_prototype("IConnectionManager");


  const char* IConnectionManager::handleConnect_event = "IConnectionManager::handleConnect";
  const char* IConnectionManager::sendReliableEvent_event = "IConnectionManager::sendReliableEvent";
  const char* IConnectionManager::sendReliableConfigEvent_event = "IConnectionManager::sendReliableConfigEvent";
  const char* IConnectionManager::sendReliableInstreamMessage_event = "IConnectionManager::sendReliableInstreamMessage";
  const char* IConnectionManager::getSupportedStrategies_event = "IConnectionManager::getSupportedStrategies";


}


