
#include "IQueued.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  unsigned int IQueued::getSize(const string& id /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > id_value(id, false);
    Event event("IQueued::getSize", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IQueued::getMaxSize(const string& id /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > id_value(id, false);
    Event event("IQueued::getMaxSize", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IQueued::setMaxSize(const unsigned int& size, const string& id /* = "default" */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned int > size_value(size, false);
    TInValue<string > id_value(id, false);
    Event event("IQueued::setMaxSize", &size_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IQueued::Mode IQueued::getMode(const string& id /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<IQueued::Mode > return_value;
    TInValue<string > id_value(id, false);
    Event event("IQueued::getMode", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IQueued::setMode(const IQueued::Mode& mode, const string& id /* = "default" */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<IQueued::Mode > mode_value(mode, false);
    TInValue<string > id_value(id, false);
    Event event("IQueued::setMode", &mode_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IQueued::IQueued_type = "IQueued";
  TIFactoryObject<IQueued > __IQueued_prototype("IQueued");
  TValueConstructor< IQueued::Mode > __IQueued_Mode_prototype(TypeInfo("IQueued::Mode", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IQueued::Mode& t)
  {
    ov.beginType(TypeInfo("IQueued::Mode", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IQueued::Mode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IQueued::Mode", "NMM")) {
      throw InvalidTypeException(TypeInfo("IQueued::Mode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* IQueued::getSize_event = "IQueued::getSize";
  const char* IQueued::getMaxSize_event = "IQueued::getMaxSize";
  const char* IQueued::setMaxSize_event = "IQueued::setMaxSize";
  const char* IQueued::getMode_event = "IQueued::getMode";
  const char* IQueued::setMode_event = "IQueued::setMode";
  
  
  unsigned int IBiDirQueuedNode::getDownstreamSize(const string& id /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::getDownstreamSize", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IBiDirQueuedNode::getDownstreamMaxSize(const string& id /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::getDownstreamMaxSize", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IQueued::Mode IBiDirQueuedNode::getDownstreamMode(const string& id /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<IQueued::Mode > return_value;
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::getDownstreamMode", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setDownstreamMaxSize(const unsigned int& size, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned int > size_value(size, false);
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::setDownstreamMaxSize", &size_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setDownstreamMode(const IQueued::Mode& mode, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    TInValue<IQueued::Mode > mode_value(mode, false);
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::setDownstreamMode", &mode_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::unlimitDownstreamSize() /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IBiDirQueuedNode::unlimitDownstreamSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::limitDownstreamSize() /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IBiDirQueuedNode::limitDownstreamSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IBiDirQueuedNode::getUpstreamSize(const string& id /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::getUpstreamSize", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IBiDirQueuedNode::getUpstreamMaxSize(const string& id /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::getUpstreamMaxSize", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IQueued::Mode IBiDirQueuedNode::getUpstreamMode(const string& id /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<IQueued::Mode > return_value;
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::getUpstreamMode", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setUpstreamMaxSize(const unsigned int& size, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned int > size_value(size, false);
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::setUpstreamMaxSize", &size_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setUpstreamMode(const IQueued::Mode& mode, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    TInValue<IQueued::Mode > mode_value(mode, false);
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::setUpstreamMode", &mode_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setMaxSize(const unsigned int& size, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned int > size_value(size, false);
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::setMaxSize", &size_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setMode(const IQueued::Mode& mode, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    TInValue<IQueued::Mode > mode_value(mode, false);
    TInValue<string > id_value(id, false);
    Event event("IBiDirQueuedNode::setMode", &mode_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IBiDirQueuedNode::IBiDirQueuedNode_type = "IBiDirQueuedNode";
  TIFactoryObject<IBiDirQueuedNode > __IBiDirQueuedNode_prototype("IBiDirQueuedNode");


  const char* IBiDirQueuedNode::getDownstreamSize_event = "IBiDirQueuedNode::getDownstreamSize";
  const char* IBiDirQueuedNode::getDownstreamMaxSize_event = "IBiDirQueuedNode::getDownstreamMaxSize";
  const char* IBiDirQueuedNode::getDownstreamMode_event = "IBiDirQueuedNode::getDownstreamMode";
  const char* IBiDirQueuedNode::setDownstreamMaxSize_event = "IBiDirQueuedNode::setDownstreamMaxSize";
  const char* IBiDirQueuedNode::setDownstreamMode_event = "IBiDirQueuedNode::setDownstreamMode";
  const char* IBiDirQueuedNode::unlimitDownstreamSize_event = "IBiDirQueuedNode::unlimitDownstreamSize";
  Event* IBiDirQueuedNode::create_unlimitDownstreamSize()
  {

    return new Event("IBiDirQueuedNode::unlimitDownstreamSize");
  }

  const char* IBiDirQueuedNode::limitDownstreamSize_event = "IBiDirQueuedNode::limitDownstreamSize";
  Event* IBiDirQueuedNode::create_limitDownstreamSize()
  {

    return new Event("IBiDirQueuedNode::limitDownstreamSize");
  }

  const char* IBiDirQueuedNode::getUpstreamSize_event = "IBiDirQueuedNode::getUpstreamSize";
  const char* IBiDirQueuedNode::getUpstreamMaxSize_event = "IBiDirQueuedNode::getUpstreamMaxSize";
  const char* IBiDirQueuedNode::getUpstreamMode_event = "IBiDirQueuedNode::getUpstreamMode";
  const char* IBiDirQueuedNode::setUpstreamMaxSize_event = "IBiDirQueuedNode::setUpstreamMaxSize";
  const char* IBiDirQueuedNode::setUpstreamMode_event = "IBiDirQueuedNode::setUpstreamMode";
  const char* IBiDirQueuedNode::setMaxSize_event = "IBiDirQueuedNode::setMaxSize";
  const char* IBiDirQueuedNode::setMode_event = "IBiDirQueuedNode::setMode";


}


