
#ifndef NMM_IQueued_HPP
#define NMM_IQueued_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "INode.hpp"

namespace NMM
{

  class IQueued : virtual public Interface { 
  public:    /** Specifies what to do when the queue is full and a new buffer
     *	arrives. 
     */

  enum Mode { 
    DEFAULT_MODE = 1 , 
    MODE_SUSPEND = 1 , 
    MODE_KEEP_OLDEST , 
    MODE_KEEP_NEWEST  };

    IQueued() {  setTypeInfo(TypeInfo(IQueued::IQueued_type, "NMM")); };

    IQueued(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IQueued::IQueued_type, "NMM")); setEventReceiver(parent); };

    virtual ~IQueued() {};

    static const char* IQueued_type;

    virtual  string getInterfaceType() const { return IQueued_type; }

    Interface* copy() const { return new IQueued(*this); }
    static const char* getSize_event;
    static const char* getMaxSize_event;
    static const char* setMaxSize_event;
    static const char* getMode_event;
    static const char* setMode_event;

    virtual unsigned int getSize(const string& id = "default") const
      throw(const Exception&);

    virtual unsigned int getMaxSize(const string& id = "default") const
      throw(const Exception&);

    virtual Result setMaxSize(const unsigned int& size, const string& id = "default")
      throw(const Exception&);

    virtual IQueued::Mode getMode(const string& id = "default") const
      throw(const Exception&);

    virtual Result setMode(const IQueued::Mode& mode, const string& id = "default")
      throw(const Exception&);


  };

  typedef auto_ptr<IQueued> IQueued_var;

  OValueStream& operator<<(OValueStream& ov, const IQueued::Mode& t);
  IValueStream& operator>>(IValueStream& iv, IQueued::Mode& t);


  class IBiDirQueuedNode : virtual public Interface { 
  public:
    IBiDirQueuedNode() {  setTypeInfo(TypeInfo(IBiDirQueuedNode::IBiDirQueuedNode_type, "NMM")); };

    IBiDirQueuedNode(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IBiDirQueuedNode::IBiDirQueuedNode_type, "NMM")); setEventReceiver(parent); };

    virtual ~IBiDirQueuedNode() {};

    static const char* IBiDirQueuedNode_type;

    virtual  string getInterfaceType() const { return IBiDirQueuedNode_type; }

    Interface* copy() const { return new IBiDirQueuedNode(*this); }
    static const char* getDownstreamSize_event;
    static const char* getDownstreamMaxSize_event;
    static const char* getDownstreamMode_event;
    static const char* setDownstreamMaxSize_event;
    static const char* setDownstreamMode_event;
    static const char* unlimitDownstreamSize_event;
    static Event* create_unlimitDownstreamSize();
    static const char* limitDownstreamSize_event;
    static Event* create_limitDownstreamSize();
    static const char* getUpstreamSize_event;
    static const char* getUpstreamMaxSize_event;
    static const char* getUpstreamMode_event;
    static const char* setUpstreamMaxSize_event;
    static const char* setUpstreamMode_event;
    static const char* setMaxSize_event;
    static const char* setMode_event;

    virtual unsigned int getDownstreamSize(const string& id = "default") const
      throw(const Exception&);

    virtual unsigned int getDownstreamMaxSize(const string& id = "default") const
      throw(const Exception&);

    virtual IQueued::Mode getDownstreamMode(const string& id = "default") const
      throw(const Exception&);

    virtual Result setDownstreamMaxSize(const unsigned int& size, const string& id = "default") /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&);

    virtual Result setDownstreamMode(const IQueued::Mode& mode, const string& id = "default") /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&);
  /* instream (protected) and non-instream (public) */    
    virtual Result unlimitDownstreamSize() /* in-stream */ 
      throw(const Exception&);
    
    virtual Result limitDownstreamSize() /* in-stream */ 
      throw(const Exception&);
  /* non-instream (public) */    
    virtual unsigned int getUpstreamSize(const string& id = "default") const
      throw(const Exception&);

    virtual unsigned int getUpstreamMaxSize(const string& id = "default") const
      throw(const Exception&);

    virtual IQueued::Mode getUpstreamMode(const string& id = "default") const
      throw(const Exception&);

    virtual Result setUpstreamMaxSize(const unsigned int& size, const string& id = "default") /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&);

    virtual Result setUpstreamMode(const IQueued::Mode& mode, const string& id = "default") /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&);

    virtual Result setMaxSize(const unsigned int& size, const string& id = "default") /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&);

    virtual Result setMode(const IQueued::Mode& mode, const string& id = "default") /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&);


  };

  typedef auto_ptr<IBiDirQueuedNode> IBiDirQueuedNode_var;




}

#endif
