
#ifndef NMM_ITransportStrategyConfig_HPP
#define NMM_ITransportStrategyConfig_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class Format;
  /** Transport strategy configuration interface.
   * This interface is for transport strategies that need special configuration and have only limited
   * support for sending and receiving NMM messages. It provides methods for configuring the transport
   * strategy and querying for its capabilities.
   */

  class ITransportStrategyConfig : virtual public Interface { 
  public:
    ITransportStrategyConfig() {  setTypeInfo(TypeInfo(ITransportStrategyConfig::ITransportStrategyConfig_type, "NMM")); };

    ITransportStrategyConfig(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITransportStrategyConfig::ITransportStrategyConfig_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITransportStrategyConfig() {};

    static const char* ITransportStrategyConfig_type;

    virtual  string getInterfaceType() const { return ITransportStrategyConfig_type; }

    Interface* copy() const { return new ITransportStrategyConfig(*this); }
    static const char* configureForInputJack_event;
    static const char* configureForOutputJack_event;
    /** This method configures the transport strategy for an input jack.
     * Some protocols need special configuration that depends on the stream format and whether data is sent or
     * received. This method configures transport strategies for such special protocols for use with input jacks.
     * The method must be called again whenever the stream format changes.
     * @param format Stream format
     */

    virtual void configureForInputJack(const Format& format)
      throw(const Exception&);
    /** This method configures the transport strategy for an output jack.
     * Some protocols need special configuration that depends on the stream format and whether data is sent or
     * received. This method configures transport strategies for such special protocols for use with output jacks.
     * The method must be called again whenever the stream format changes.
     * @param format Stream format
     */

    virtual void configureForOutputJack(const Format& format)
      throw(const Exception&);


  };

  typedef auto_ptr<ITransportStrategyConfig> ITransportStrategyConfig_var;




}

#endif
