
#ifndef NMM_IUsage_HPP
#define NMM_IUsage_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/base/registry/RegistryStructs.hpp"
#include "nmm/interfaces/base/IUsageNotify.hpp"

namespace NMM
{

  class IUsage : virtual public Interface { 
  public:
  enum usage_t { 
    UNKNOWN = -1 , 
    DVBEPG = 0 , 
    LIVETV , 
    RECORDER  };

    IUsage() {  setTypeInfo(TypeInfo(IUsage::IUsage_type, "NMM")); };

    IUsage(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IUsage::IUsage_type, "NMM")); setEventReceiver(parent); };

    virtual ~IUsage() {};

    static const char* IUsage_type;

    virtual  string getInterfaceType() const { return IUsage_type; }

    Interface* copy() const { return new IUsage(*this); }
    static const char* setUsage_event;
    static const char* unsetUsage_event;
    static const char* getUsage_event;
    static const char* getRank_event;
    static const char* isMaster_event;
    /**
     *Set the Usage of the node.
     *@param usage usage of the node
     *@param user set the IUsageNotify-interface of this user so it will be notified, if its status (master or slave) changes
     */

    virtual void setUsage(const IUsage::usage_t& usage, const IUsageNotify* user)
      throw(const Exception&);
    /**
     *Unset the Usage of the node.
     *@param usage usage of the node
     *@param user set the IUsageNotify-interface of this user to identify the user
     */

    virtual void unsetUsage(const IUsage::usage_t& usage, const IUsageNotify* user)
      throw(const Exception&);
    /**
     *Get the current Usage of this node
     *@return usage current usage
     */

    virtual IUsage::usage_t getUsage()
      throw(const Exception&);
    /**
     *Get the number of users wich have rewuested this node before the given user with the given usage
     *@param usage usage
     *@param user the user wich makes the request
     *@return int rank
     */

    virtual int getRank(const IUsage::usage_t& usage, const IUsageNotify* user)
      throw(const Exception&);
    /**
     *Checks, if the given user will be Master with the given usage
     *@param mode usage
     *@param user user-identification
     *@return SUCCESS if the user will be master
     *@return FAILED if the user will be slave
     */

    virtual Result isMaster(const IUsage::usage_t& mode, const IUsageNotify* user)
      throw(const Exception&);


  };

  typedef auto_ptr<IUsage> IUsage_var;

  OValueStream& operator<<(OValueStream& ov, const IUsage::usage_t& t);
  IValueStream& operator>>(IValueStream& iv, IUsage::usage_t& t);



}

#endif
