
#include "IResourceCollector.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IResourceCollector::enableResourceProbing()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IResourceCollector::enableResourceProbing");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IResourceCollector::disableResourceProbing()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IResourceCollector::disableResourceProbing");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IResourceCollector::setBufferProbeInterval(const unsigned int& value)
      throw(const Exception&)
  {
    TInValue<unsigned int > value_value(value, false);
    Event event("IResourceCollector::setBufferProbeInterval", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IResourceCollector::restrictCPUUsage(const float& level)
      throw(const Exception&)
  {
    TInValue<float > level_value(level, false);
    Event event("IResourceCollector::restrictCPUUsage", &level_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Interval IResourceCollector::getUsedCPUTime()
      throw(const Exception&)
  {
    TOutValue<Interval > return_value;
    Event event("IResourceCollector::getUsedCPUTime");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval IResourceCollector::getUsedTime()
      throw(const Exception&)
  {
    TOutValue<Interval > return_value;
    Event event("IResourceCollector::getUsedTime");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned long IResourceCollector::getMaxReceivedBufferSize()
      throw(const Exception&)
  {
    TOutValue<unsigned long > return_value;
    Event event("IResourceCollector::getMaxReceivedBufferSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IResourceCollector::getCPUUsage(const bool& since_start /* = false */)
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<bool > since_start_value(since_start, false);
    Event event("IResourceCollector::getCPUUsage", &since_start_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IResourceCollector::setCPU(const string& ci)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > ci_value(ci, false);
    Event event("IResourceCollector::setCPU", &ci_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IResourceCollector::loadMesh(const string& s_mesh)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > s_mesh_value(s_mesh, false);
    Event event("IResourceCollector::loadMesh", &s_mesh_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IResourceCollector::loadMeshes(const string& format_id, const string& s_mesh)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > format_id_value(format_id, false);
    TInValue<string > s_mesh_value(s_mesh, false);
    Event event("IResourceCollector::loadMeshes", &format_id_value, &s_mesh_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IResourceCollector::getMesh(string& s_mesh)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TOutValue<string > s_mesh_value(s_mesh);
    Event event("IResourceCollector::getMesh", &s_mesh_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    s_mesh = s_mesh_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IResourceCollector::savePoints(const string& filename)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > filename_value(filename, false);
    Event event("IResourceCollector::savePoints", &filename_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  float IResourceCollector::getUsage(const Format& description)
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    TInValue<Format > description_value(description, false);
    Event event("IResourceCollector::getUsage", &description_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  float IResourceCollector::getActualUsage()
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    Event event("IResourceCollector::getActualUsage");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  float IResourceCollector::getActualBandwidth()
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    Event event("IResourceCollector::getActualBandwidth");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  float IResourceCollector::getActualBufferSize()
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    Event event("IResourceCollector::getActualBufferSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  float IResourceCollector::getCPUperBuffer()
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    Event event("IResourceCollector::getCPUperBuffer");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IResourceCollector::getVertex(const Format& description, RMVertex& vertex)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Format > description_value(description, false);
    TOutValue<RMVertex > vertex_value(vertex);
    Event event("IResourceCollector::getVertex", &description_value, &vertex_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    vertex = vertex_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IResourceCollector::updateVertex(const Format& description, const float& cpu_usage, const float& bandwidth)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Format > description_value(description, false);
    TInValue<float > cpu_usage_value(cpu_usage, false);
    TInValue<float > bandwidth_value(bandwidth, false);
    Event event("IResourceCollector::updateVertex", &description_value, &cpu_usage_value, &bandwidth_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IResourceCollector::addVertex(const Format& description, const float& cpu_usage, const float& bandwidth)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Format > description_value(description, false);
    TInValue<float > cpu_usage_value(cpu_usage, false);
    TInValue<float > bandwidth_value(bandwidth, false);
    Event event("IResourceCollector::addVertex", &description_value, &cpu_usage_value, &bandwidth_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IResourceCollector::setupMesh(const Format& dimensions)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Format > dimensions_value(dimensions, false);
    Event event("IResourceCollector::setupMesh", &dimensions_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool IResourceCollector::getActualFormat(Format& ret_format)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TOutValue<Format > ret_format_value(ret_format);
    Event event("IResourceCollector::getActualFormat", &ret_format_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    ret_format = ret_format_value.getRValue();
    return return_value.getRValue();
  }

  const char* IResourceCollector::IResourceCollector_type = "IResourceCollector";
  TIFactoryObject<IResourceCollector > __IResourceCollector_prototype("IResourceCollector");


  const char* IResourceCollector::enableResourceProbing_event = "IResourceCollector::enableResourceProbing";
  const char* IResourceCollector::disableResourceProbing_event = "IResourceCollector::disableResourceProbing";
  const char* IResourceCollector::setBufferProbeInterval_event = "IResourceCollector::setBufferProbeInterval";
  const char* IResourceCollector::restrictCPUUsage_event = "IResourceCollector::restrictCPUUsage";
  const char* IResourceCollector::getUsedCPUTime_event = "IResourceCollector::getUsedCPUTime";
  const char* IResourceCollector::getUsedTime_event = "IResourceCollector::getUsedTime";
  const char* IResourceCollector::getMaxReceivedBufferSize_event = "IResourceCollector::getMaxReceivedBufferSize";
  const char* IResourceCollector::getCPUUsage_event = "IResourceCollector::getCPUUsage";
  const char* IResourceCollector::setCPU_event = "IResourceCollector::setCPU";
  const char* IResourceCollector::loadMesh_event = "IResourceCollector::loadMesh";
  const char* IResourceCollector::loadMeshes_event = "IResourceCollector::loadMeshes";
  const char* IResourceCollector::getMesh_event = "IResourceCollector::getMesh";
  const char* IResourceCollector::savePoints_event = "IResourceCollector::savePoints";
  const char* IResourceCollector::getUsage_event = "IResourceCollector::getUsage";
  const char* IResourceCollector::getActualUsage_event = "IResourceCollector::getActualUsage";
  const char* IResourceCollector::getActualBandwidth_event = "IResourceCollector::getActualBandwidth";
  const char* IResourceCollector::getActualBufferSize_event = "IResourceCollector::getActualBufferSize";
  const char* IResourceCollector::getCPUperBuffer_event = "IResourceCollector::getCPUperBuffer";
  const char* IResourceCollector::getVertex_event = "IResourceCollector::getVertex";
  const char* IResourceCollector::updateVertex_event = "IResourceCollector::updateVertex";
  const char* IResourceCollector::addVertex_event = "IResourceCollector::addVertex";
  const char* IResourceCollector::setupMesh_event = "IResourceCollector::setupMesh";
  const char* IResourceCollector::getActualFormat_event = "IResourceCollector::getActualFormat";


}


