
#ifndef NMM_IAudioHandoverSynchronizerImpl_HPP
#define NMM_IAudioHandoverSynchronizerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IAudioHandoverSynchronizer.hpp"

#include "ISinkSynchronizerImpl.hpp"
#include "IMultiAudioSinkSynchronizerImpl.hpp"
#include "ISwitchImpl.hpp"

namespace NMM
{
  /**
   * Audio-audio sink synchronizer.
   * Interface for a synchronizer which performs inter-stream synchronization between
   * an audio and a seconed audio sink.
   */

  class IAudioHandoverSynchronizerImpl  : virtual public ISinkSynchronizerImpl, virtual public IMultiAudioSinkSynchronizerImpl { 
  public:
    IAudioHandoverSynchronizerImpl() { addInterface(new IAudioHandoverSynchronizer(*this)); registerEvents(); };

    virtual ~IAudioHandoverSynchronizerImpl() {};

  protected:
    /**
     * Set the audio sink controller.
     * This sets the controller for the audio sink.
     * @param controller controller for this sink
     */

    virtual Result setFirstSinkController(const ISinkController* controller) = 0;
    /**
     * Set the second audio sink controller.
     * This sets the controller for the audio sink.
     * @param controller controller for this sink
     */

    virtual Result setSecondSinkController(const ISinkController* controller) = 0;
    /**
     * Makes the slave (2nd) the master (1st) controller and vica versa
     */

    virtual void switchControllers() = 0;
    /**
     * Sets a skew between audio0 and audio1 output.
     * The audio1 output is presented later than the audio0 output.
     */

    virtual Result setSkew(const Interval& skew) = 0;
    /**
     * Sets the ISwitch interface of this synchronizer
     */

    virtual void setSwitch(const ISwitch* s) = 0;
    /**
     * Sets the modus for handover: either
     * INSTREAM_SWITCH = ISwitch::switchOutputs()
     * OUTOFBAND_SWITCH = Mute/unmute method calls from synchronizer
     * INSTREAM_FLUSHED_SWITCH = ISwitch::flushOutputs(), ISwitch::switchOutputs()
     */

    virtual void setMode(const IAudioHandoverSynchronizer::Mode& mode) = 0;
    /**
     * Sets the overlapping interval during which master and
     * slave perform playback during handover (only possible
     * for modus 1)
     */

    virtual void setOverlap(const Interval& overlap) = 0;
    /**
     * Blocking method that waits until handover is complete, 
     * must be called after adding slave controller with addAudioSinkController
     */

    virtual void waitForSwitchControllers() = 0;


  private:
    void registerEvents();
  };

}

#endif
