
#include "IMultiAudioVideoSynchronizer.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IMultiAudioVideoSynchronizer::setMasterAudioSink(const int& id)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > id_value(id, false);
    Event event("IMultiAudioVideoSynchronizer::setMasterAudioSink", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMultiAudioVideoSynchronizer::setAudioSkew(const int& id, const Interval& skew)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > id_value(id, false);
    TInValue<Interval > skew_value(skew, false);
    Event event("IMultiAudioVideoSynchronizer::setAudioSkew", &id_value, &skew_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMultiAudioVideoSynchronizer::setVideoSkew(const int& id, const Interval& skew)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > id_value(id, false);
    TInValue<Interval > skew_value(skew, false);
    Event event("IMultiAudioVideoSynchronizer::setVideoSkew", &id_value, &skew_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IMultiAudioVideoSynchronizer::IMultiAudioVideoSynchronizer_type = "IMultiAudioVideoSynchronizer";
  TIFactoryObject<IMultiAudioVideoSynchronizer > __IMultiAudioVideoSynchronizer_prototype("IMultiAudioVideoSynchronizer");


  const char* IMultiAudioVideoSynchronizer::setMasterAudioSink_event = "IMultiAudioVideoSynchronizer::setMasterAudioSink";
  const char* IMultiAudioVideoSynchronizer::setAudioSkew_event = "IMultiAudioVideoSynchronizer::setAudioSkew";
  const char* IMultiAudioVideoSynchronizer::setVideoSkew_event = "IMultiAudioVideoSynchronizer::setVideoSkew";


}


