
#ifndef NMM_ISwitch_HPP
#define NMM_ISwitch_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class ISwitch : virtual public Interface { 
  public:
    ISwitch() {  setTypeInfo(TypeInfo(ISwitch::ISwitch_type, "NMM")); };

    ISwitch(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ISwitch::ISwitch_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISwitch() {};

    static const char* ISwitch_type;

    virtual  string getInterfaceType() const { return ISwitch_type; }

    Interface* copy() const { return new ISwitch(*this); }
    static const char* switchOutputs_event;
    static const char* flushOutputs_event;
    static const char* setSwitchOutputsEnabled_event;
    static const char* getSwitchOutputsEnabled_event;
    /**
     * Perform whatever switch operation is available
     */

    virtual void switchOutputs()
      throw(const Exception&);
    /**
     * Perform whatever flush operation is available
     */

    virtual void flushOutputs(const Timestamp& ts)
      throw(const Exception&);
    /**
     * Sets whether the outgoing streams should be checked
     * for new connections
     */

    virtual void setSwitchOutputsEnabled(const bool& enable)
      throw(const Exception&);
    /**
     * Gets the status of the check for new connections
     */

    virtual bool getSwitchOutputsEnabled()
      throw(const Exception&);


  };

  typedef auto_ptr<ISwitch> ISwitch_var;




}

#endif
