
#ifndef NMM_ISynchronizer_HPP
#define NMM_ISynchronizer_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class ISynchronizer : virtual public Interface { 
  public:
    ISynchronizer() {  setTypeInfo(TypeInfo(ISynchronizer::ISynchronizer_type, "NMM")); };

    ISynchronizer(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ISynchronizer::ISynchronizer_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISynchronizer() {};

    static const char* ISynchronizer_type;

    virtual  string getInterfaceType() const { return ISynchronizer_type; }

    Interface* copy() const { return new ISynchronizer(*this); }
    static const char* pause_event;
    static const char* wakeup_event;
    static const char* reset_event;

    virtual Result pause()
      throw(const Exception&);

    virtual Result wakeup()
      throw(const Exception&);
   /**
     * Resets the Synchronizer.
     * This method is used by the sink controllers and should not be called 
     * directly.
     * Only internal variables are reset, you don't need to set the
     * audio and video sink after a reset call.
     * Switches the state from WORKING to RESET.
     * @param sink sink ID
     */

    virtual Result reset(const int& sink)
      throw(const Exception&);


  };

  typedef auto_ptr<ISynchronizer> ISynchronizer_var;




}

#endif
