
#include "IAudioDevice.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IAudioDevice::setMixerDevice(const string& value) 
      throw(const Exception&, const DeviceNotFoundException&)
  {
    TInValue<string > value_value(value, false);
    Event event("IAudioDevice::setMixerDevice", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(DeviceNotFoundException* ex = dynamic_cast<DeviceNotFoundException*>(event.getException())) {
          throw DeviceNotFoundException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  string IAudioDevice::getMixerDevice() const
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IAudioDevice::getMixerDevice");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IAudioDevice::increaseVolume()
      throw(const Exception&)
  {
    Event event("IAudioDevice::increaseVolume");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IAudioDevice::decreaseVolume()
      throw(const Exception&)
  {
    Event event("IAudioDevice::decreaseVolume");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IAudioDevice::setVolume(const int& volume)
      throw(const Exception&)
  {
    TInValue<int > volume_value(volume, false);
    Event event("IAudioDevice::setVolume", &volume_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int IAudioDevice::getVolume()
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IAudioDevice::getVolume");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IAudioDevice::IAudioDevice_type = "IAudioDevice";
  TIFactoryObject<IAudioDevice > __IAudioDevice_prototype("IAudioDevice");


  const char* IAudioDevice::setMixerDevice_event = "IAudioDevice::setMixerDevice";
  const char* IAudioDevice::getMixerDevice_event = "IAudioDevice::getMixerDevice";
  const char* IAudioDevice::increaseVolume_event = "IAudioDevice::increaseVolume";
  const char* IAudioDevice::decreaseVolume_event = "IAudioDevice::decreaseVolume";
  const char* IAudioDevice::setVolume_event = "IAudioDevice::setVolume";
  const char* IAudioDevice::getVolume_event = "IAudioDevice::getVolume";


}


