
#include "IEncode.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IEncode::setQuality(const float& quality)
      throw(const Exception&)
  {
    TInValue<float > quality_value(quality, false);
    Event event("IEncode::setQuality", &quality_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEncode::setBitrate(const int& bitrate)
      throw(const Exception&)
  {
    TInValue<int > bitrate_value(bitrate, false);
    Event event("IEncode::setBitrate", &bitrate_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  float IEncode::getQuality()
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    Event event("IEncode::getQuality");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IEncode::getBitrate()
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IEncode::getBitrate");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IEncode::IEncode_type = "IEncode";
  TIFactoryObject<IEncode > __IEncode_prototype("IEncode");


  const char* IEncode::setQuality_event = "IEncode::setQuality";
  const char* IEncode::setBitrate_event = "IEncode::setBitrate";
  const char* IEncode::getQuality_event = "IEncode::getQuality";
  const char* IEncode::getBitrate_event = "IEncode::getBitrate";


}


