
#ifndef NMM_ITrackDurationImpl_HPP
#define NMM_ITrackDurationImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ITrackDuration.hpp"


namespace NMM
{
  /**
   *  @brief TrackDuration events
   *
   *  The trackDuration event is sent by read nodes, decoders or demuxers. It contains the duration of the
   *  audio/video track that is being processed. Applications that need to know the duration of the track
   *  can register a listener for this event.
   */

  class ITrackDurationImpl  : public virtual NMMObject { 
  public:
    ITrackDurationImpl() { addInterface(new ITrackDuration(*this)); registerEvents(); };

    virtual ~ITrackDurationImpl() {};

  protected:
    /**
     *  @brief Duration of the current track
     *  @param duration of the track (in seconds and nanoseconds)
     *
     *  Instream event containing the duration of the current track.
     */

    virtual Result trackDuration(const Interval& i) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#endif
