
#ifndef NMM_ITVGuidePeerObjectImpl_HPP
#define NMM_ITVGuidePeerObjectImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ITVGuidePeerObject.hpp"


namespace NMM
{
  /**
   *Interface to get notification from remote TVGuides of available information
   */

  class ITVGuidePeerObjectImpl  : public virtual NMMObject { 
  public:
    ITVGuidePeerObjectImpl() { addInterface(new ITVGuidePeerObject(*this)); registerEvents(); };

    virtual ~ITVGuidePeerObjectImpl() {};

  protected:
    /**
     *This method is calles from the remote TVGuide to notify the local one of available information. 
     *@param pdlist list of ProgramDescriptions the local TVGuide has rewusted
     *@param id if the search. This is for the application wich started the request to match the resault to a request
     *@param history This list stores the ITVGuidePeerObect-interfaces of TVGuide wich should be notified about the result
     */

    virtual Result peerResult(const ProgramDescriptionList& pdlist, const int& id, const list<Interface* > & history) = 0;

    virtual Result peerChannelResult(const ChannelDescriptionList& cdlist, const list<Interface* > & history) = 0;


  private:
    void registerEvents();
  };
  /**
   *Interface to search remote TVGuides for information 
   */

  class ITVGuidePeerSearchImpl  : public virtual NMMObject { 
  public:
    ITVGuidePeerSearchImpl() { addInterface(new ITVGuidePeerSearch(*this)); registerEvents(); };

    virtual ~ITVGuidePeerSearchImpl() {};

  protected:
    /**
     *method to search a remote TVGuide for information. 
     *@param filter A ProgramFilter containing the request
     *@param history This list stores the ITVGuidePeerObect-interfaces of TVGuide wich should be notified about the result
     */

    virtual Result peerSearch(const ProgramFilter& filter, const list<Interface* > & history) = 0;

    virtual Result peerChannelSearch(const list<Interface* > & history) = 0;


  private:
    void registerEvents();
  };

}

#endif
