
#include "ITVManager.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result ITVManager::setChannel(const unsigned int& channel)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned int > channel_value(channel, false);
    Event event("ITVManager::setChannel", &channel_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManager::channelUp()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVManager::channelUp");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManager::channelDown()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVManager::channelDown");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode* ITVManager::getTVCard()
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    Event event("ITVManager::getTVCard");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManager::startGraph()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVManager::startGraph");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITVManager::setUsageNotify(const IUsageNotify* u)
      throw(const Exception&)
  {
    TInValue<IUsageNotify* > u_value (const_cast<IUsageNotify* >(u), false);
    Event event("ITVManager::setUsageNotify", &u_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITVManager::releaseGraph()
      throw(const Exception&)
  {
    Event event("ITVManager::releaseGraph");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Result ITVManager::switchingSource()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVManager::switchingSource");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManager::sharedSource()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVManager::sharedSource");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManager::skippingChannel()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVManager::skippingChannel");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITVManager::ITVManager_type = "ITVManager";
  TIFactoryObject<ITVManager > __ITVManager_prototype("ITVManager");


  const char* ITVManager::setChannel_event = "ITVManager::setChannel";
  const char* ITVManager::channelUp_event = "ITVManager::channelUp";
  const char* ITVManager::channelDown_event = "ITVManager::channelDown";
  const char* ITVManager::getTVCard_event = "ITVManager::getTVCard";
  const char* ITVManager::startGraph_event = "ITVManager::startGraph";
  const char* ITVManager::setUsageNotify_event = "ITVManager::setUsageNotify";
  const char* ITVManager::releaseGraph_event = "ITVManager::releaseGraph";
  const char* ITVManager::switchingSource_event = "ITVManager::switchingSource";
  const char* ITVManager::sharedSource_event = "ITVManager::sharedSource";
  const char* ITVManager::skippingChannel_event = "ITVManager::skippingChannel";


}


