
#include "ITVTimer.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result ITVTimer::programTimer(const TVTimerEntry& entry) 
      throw(const Exception&, const TimerAlreadyProgrammedException&)
  {
    TOutValue<Result > return_value;
    TInValue<TVTimerEntry > entry_value(entry, false);
    Event event("ITVTimer::programTimer", &entry_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(TimerAlreadyProgrammedException* ex = dynamic_cast<TimerAlreadyProgrammedException*>(event.getException())) {
          throw TimerAlreadyProgrammedException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVTimer::changeTimer(const TVTimerEntry& newEntry) 
      throw(const Exception&, const ActionNotAllowedException&)
  {
    TOutValue<Result > return_value;
    TInValue<TVTimerEntry > newEntry_value(newEntry, false);
    Event event("ITVTimer::changeTimer", &newEntry_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(ActionNotAllowedException* ex = dynamic_cast<ActionNotAllowedException*>(event.getException())) {
          throw ActionNotAllowedException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVTimer::deleteTimer(const int& i, const string& user /* = "" */) 
      throw(const Exception&, const ActionNotAllowedException&)
  {
    TOutValue<Result > return_value;
    TInValue<int > i_value(i, false);
    TInValue<string > user_value(user, false);
    Event event("ITVTimer::deleteTimer", &i_value, &user_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(ActionNotAllowedException* ex = dynamic_cast<ActionNotAllowedException*>(event.getException())) {
          throw ActionNotAllowedException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVTimerEntry >  ITVTimer::getTimers(const string& day /* = "" */, const string& channel /* = "" */)
      throw(const Exception&)
  {
    TOutValue<list<TVTimerEntry >  > return_value;
    TInValue<string > day_value(day, false);
    TInValue<string > channel_value(channel, false);
    Event event("ITVTimer::getTimers", &day_value, &channel_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  TVTimerEntry ITVTimer::getTimer(const unsigned int& id)
      throw(const Exception&)
  {
    TOutValue<TVTimerEntry > return_value;
    TInValue<unsigned int > id_value(id, false);
    Event event("ITVTimer::getTimer", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVTimerEntry >  ITVTimer::getSleepingTimers()
      throw(const Exception&)
  {
    TOutValue<list<TVTimerEntry >  > return_value;
    Event event("ITVTimer::getSleepingTimers");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVTimerEntry >  ITVTimer::getRecordingTimers()
      throw(const Exception&)
  {
    TOutValue<list<TVTimerEntry >  > return_value;
    Event event("ITVTimer::getRecordingTimers");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVTimerEntry >  ITVTimer::getRecordedTimers()
      throw(const Exception&)
  {
    TOutValue<list<TVTimerEntry >  > return_value;
    Event event("ITVTimer::getRecordedTimers");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVTimerEntry >  ITVTimer::getConflictTimers()
      throw(const Exception&)
  {
    TOutValue<list<TVTimerEntry >  > return_value;
    Event event("ITVTimer::getConflictTimers");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVTimerEntry >  ITVTimer::getConflictsWithTimer(const TVTimerEntry& entry)
      throw(const Exception&)
  {
    TOutValue<list<TVTimerEntry >  > return_value;
    TInValue<TVTimerEntry > entry_value(entry, false);
    Event event("ITVTimer::getConflictsWithTimer", &entry_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVTimer::loadTimer(const string& filename)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > filename_value(filename, false);
    Event event("ITVTimer::loadTimer", &filename_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVTimer::saveTimer(const string& filename)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > filename_value(filename, false);
    Event event("ITVTimer::saveTimer", &filename_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITVTimer::setTVGuide(const ITVGuide* itvg)
      throw(const Exception&)
  {
    TInValue<ITVGuide* > itvg_value (const_cast<ITVGuide* >(itvg), false);
    Event event("ITVTimer::setTVGuide", &itvg_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  ITVGuide* ITVTimer::getTVGuide()
      throw(const Exception&)
  {
    TOutValue<ITVGuide* > return_value;
    Event event("ITVTimer::getTVGuide");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITVTimer::setDirectory(const string& dir)
      throw(const Exception&)
  {
    TInValue<string > dir_value(dir, false);
    Event event("ITVTimer::setDirectory", &dir_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  string ITVTimer::getDirectory()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("ITVTimer::getDirectory");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  TVChannelList ITVTimer::getChannelConfig()
      throw(const Exception&)
  {
    TOutValue<TVChannelList > return_value;
    Event event("ITVTimer::getChannelConfig");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITVTimer::ITVTimer_type = "ITVTimer";
  TIFactoryObject<ITVTimer > __ITVTimer_prototype("ITVTimer");


  const char* ITVTimer::programTimer_event = "ITVTimer::programTimer";
  const char* ITVTimer::changeTimer_event = "ITVTimer::changeTimer";
  const char* ITVTimer::deleteTimer_event = "ITVTimer::deleteTimer";
  const char* ITVTimer::getTimers_event = "ITVTimer::getTimers";
  const char* ITVTimer::getTimer_event = "ITVTimer::getTimer";
  const char* ITVTimer::getSleepingTimers_event = "ITVTimer::getSleepingTimers";
  const char* ITVTimer::getRecordingTimers_event = "ITVTimer::getRecordingTimers";
  const char* ITVTimer::getRecordedTimers_event = "ITVTimer::getRecordedTimers";
  const char* ITVTimer::getConflictTimers_event = "ITVTimer::getConflictTimers";
  const char* ITVTimer::getConflictsWithTimer_event = "ITVTimer::getConflictsWithTimer";
  const char* ITVTimer::loadTimer_event = "ITVTimer::loadTimer";
  const char* ITVTimer::saveTimer_event = "ITVTimer::saveTimer";
  const char* ITVTimer::setTVGuide_event = "ITVTimer::setTVGuide";
  const char* ITVTimer::getTVGuide_event = "ITVTimer::getTVGuide";
  const char* ITVTimer::setDirectory_event = "ITVTimer::setDirectory";
  const char* ITVTimer::getDirectory_event = "ITVTimer::getDirectory";
  const char* ITVTimer::getChannelConfig_event = "ITVTimer::getChannelConfig";


}


