
#include "INetClient.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void INetClient::setHostname(const string& value)
      throw(const Exception&)
  {
    TInValue<string > value_value(value, false);
    Event event("INetClient::setHostname", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void INetClient::setPort(const int& port)
      throw(const Exception&)
  {
    TInValue<int > port_value(port, false);
    Event event("INetClient::setPort", &port_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void INetClient::setProtocol(const string& protocol)
      throw(const Exception&)
  {
    TInValue<string > protocol_value(protocol, false);
    Event event("INetClient::setProtocol", &protocol_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  string INetClient::getHostname() const
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("INetClient::getHostname");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int INetClient::getPort() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("INetClient::getPort");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string INetClient::getProtocol() const
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("INetClient::getProtocol");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INetClient::connectionLost() /* in-stream */ 
      throw(const Exception&)
  {
    Event event("INetClient::connectionLost");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* INetClient::INetClient_type = "INetClient";
  TIFactoryObject<INetClient > __INetClient_prototype("INetClient");


  const char* INetClient::setHostname_event = "INetClient::setHostname";
  const char* INetClient::setPort_event = "INetClient::setPort";
  const char* INetClient::setProtocol_event = "INetClient::setProtocol";
  const char* INetClient::getHostname_event = "INetClient::getHostname";
  const char* INetClient::getPort_event = "INetClient::getPort";
  const char* INetClient::getProtocol_event = "INetClient::getProtocol";
  const char* INetClient::connectionLost_event = "INetClient::connectionLost";
  Event* INetClient::create_connectionLost()
  {

    return new Event("INetClient::connectionLost");
  }



}


