
#ifndef NMM_ISleep_HPP
#define NMM_ISleep_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   * Interfae for setting the sleep time after receiving a buffer.
   * This functionality is for testing purposes and implemented, for example, in DevNullNode.
   */

  class ISleep : virtual public Interface { 
  public:
    ISleep() {  setTypeInfo(TypeInfo(ISleep::ISleep_type, "NMM")); };

    ISleep(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ISleep::ISleep_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISleep() {};

    static const char* ISleep_type;

    virtual  string getInterfaceType() const { return ISleep_type; }

    Interface* copy() const { return new ISleep(*this); }
    static const char* setSleepTime_event;
    static const char* getSleepTime_event;
    /**
     * Set the sleep time.
     * Set the time the node will wait after receiving a buffer, in seconds.
     */

    virtual void setSleepTime(const int& value)
      throw(const Exception&);
    /**
     * Get the sleep time.
     * Get the time the node will wait after receiving a buffer, in seconds.
     */

    virtual int getSleepTime() const
      throw(const Exception&);


  };

  typedef auto_ptr<ISleep> ISleep_var;




}

#endif
