
#ifndef NMM_ISVGEncode_HPP
#define NMM_ISVGEncode_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class ISVGEncode : virtual public Interface { 
  public:
  enum SVGDrawMode { 
    line , 
    rectangle , 
    freehand , 
    pointer  };

    ISVGEncode() {  setTypeInfo(TypeInfo(ISVGEncode::ISVGEncode_type, "NMM")); };

    ISVGEncode(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ISVGEncode::ISVGEncode_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISVGEncode() {};

    static const char* ISVGEncode_type;

    virtual  string getInterfaceType() const { return ISVGEncode_type; }

    Interface* copy() const { return new ISVGEncode(*this); }
    static const char* setColor_event;
    static const char* setDimension_event;
    static const char* getPixelTaken_event;
    static const char* setPixelTaken_event;
    static const char* setWithFile_event;
    static const char* setStrokeWidth_event;
    static const char* drawSVGLine_event;
    static const char* drawSVGRectangle_event;
    static const char* drawSVGFreehand_event;
    static const char* drawSVGPointer_event;
    static const char* newAnnotation_event;
    static const char* clearAnnotation_event;
    static const char* undo_event;
    static const char* redo_event;
    static const char* repaint_event;
      /**
       * Sets the painting color
       * @param red red-value in RGB-Color
       * @param green green-value in RGB-Color
       * @param blue blue-value in RGB-Color
       */

    virtual void setColor(const int& red, const int& green, const int& blue)
      throw(const Exception&);
      /**
       * Sets the dimension for the SVG-graphic
       * @param width the width of the graphic
       * @param height the height of the graphic
       */

    virtual void setDimension(const int& width, const int& height)
      throw(const Exception&);
      /**
       * Returns the accuracy for the SVGFreehand object. Every nth pixel is taken for the bezier-curve.
       * @return the accuracy of a SVGFreehand object.
       */

    virtual int getPixelTaken()
      throw(const Exception&);
      /**
       * Sets the accuracy for the SVGFreehand object. Every nth pixel is taken for the bezier-curve.
       * @param pixel_taken the accuracy of a SVGFreehand object.
       */

    virtual void setPixelTaken(const int& pixel_taken)
      throw(const Exception&);
      /**
       * Sets a flag wether a SVG file should be written or not
       * @param withFile a flag wether a SVG file should be written or not
       */

    virtual void setWithFile(const bool& withFile)
      throw(const Exception&);
      /**
       * Sets the stroke width of the SVG objects.
       * @param stroke_width the stroke width of the SVG objects
       */

    virtual void setStrokeWidth(const int& stroke_width)
      throw(const Exception&);
      /**
       * Adds the point (x|y) to a SVGLine. If mouse button just pressed a new object will be created. If mouse button is released the object is finished.
       * @param x x-coordinate of the point
       * @param y y-coordinate of the point
       * @param buttonmask the buttonmask of the mousebuttons
       */

    virtual void drawSVGLine(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&);
      /**
       * Adds the point (x|y) to a SVGRectangle. If mouse button just pressed a new object will be created. If mouse button is released the object is finished.
       * @param x x-coordinate of the point
       * @param y y-coordinate of the point
       * @param buttonmask the buttonmask of the mousebuttons
       */ 

    virtual void drawSVGRectangle(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&);
      /**
       * Adds the point (x|y) to a SVGFreehand. If mouse button just pressed a new object will be created. If mouse is moved the point will be added to the object. If mouse button is released the object is finished.
       * @param x x-coordinate of the point
       * @param y y-coordinate of the point
       * @param buttonmask the buttonmask of the mousebuttons
       */

    virtual void drawSVGFreehand(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&);
      /**
       * Adds the point (x|y) to a SVGPointer. If mouse button just pressed a new object will be created. If mouse is moved the point will be added to the object. If mouse button is released the object is finished.
       * @param x x-coordinate of the point
       * @param y y-coordinate of the point
       * @param buttonmask the buttonmask of the mousebuttons
       */

    virtual void drawSVGPointer(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&);
      /**
       * Creates a new annotation and write the data to the SVG file wether the flag writeFile is set and there are objects in history. The objects in the history will then be deleted.
       */

    virtual void newAnnotation()
      throw(const Exception&);
      /**
       * Empty the actual history and have a new empty annotation.
       */

    virtual void clearAnnotation()
      throw(const Exception&);
      /**
       * Deletes the last created object in history
       */

    virtual void undo()
      throw(const Exception&);
      /**
       * Restores the last deleted object in history
       */

    virtual void redo()
      throw(const Exception&);
      /**
       * Repaints the current picture
       */

    virtual void repaint()
      throw(const Exception&);


  };

  typedef auto_ptr<ISVGEncode> ISVGEncode_var;

  OValueStream& operator<<(OValueStream& ov, const ISVGEncode::SVGDrawMode& t);
  IValueStream& operator>>(IValueStream& iv, ISVGEncode::SVGDrawMode& t);



}

#endif
