
#ifndef NMM_IXDisplayEventSender_HPP
#define NMM_IXDisplayEventSender_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/video/display/IDisplayEventSender.hpp"

namespace NMM
{
  /**
   * XDisplay event sender.
   * This Node extends IDisplayEventSender with methods to determine the kind mouseevents and keyevents are treated resp. to test,
   * which kind is choosen.
   */

  class IXDisplayEventSender : virtual public IDisplayEventSender { 
  public:
    IXDisplayEventSender() {  setTypeInfo(TypeInfo(IXDisplayEventSender::IXDisplayEventSender_type, "NMM")); };

    IXDisplayEventSender(NMMObject& parent) : IDisplayEventSender(parent) {  setTypeInfo(TypeInfo(IXDisplayEventSender::IXDisplayEventSender_type, "NMM")); setEventReceiver(parent); };

    virtual ~IXDisplayEventSender() {};

    static const char* IXDisplayEventSender_type;

    virtual  string getInterfaceType() const { return IXDisplayEventSender_type; }

    Interface* copy() const { return new IXDisplayEventSender(*this); }
    static const char* setSendingMouseEventsDirect_event;
    static const char* isSendingMouseEventsDirect_event;
    static const char* setUsingKeySymKeyEvents_event;
    static const char* isUsingKeySymKeyEvents_event;
    static const char* setXAutoRepeat_event;
    static const char* isXAutoRepeat_event;
    /**
     * Switches between processing only Mouseclicks, which are in predefined Screenrectangles or sending all MouseEvents.
     * @param smed boolean Value
     */

    virtual void setSendingMouseEventsDirect(const bool& smed) /* limited(INode::STARTED) */
      throw(const Exception&, const IllegalStateException&);
    /**
     * Returns Value of SendingMouseEventsDirect.
     */

    virtual bool isSendingMouseEventsDirect()
      throw(const Exception&);
    /**
     * Switches between sending KeyEvents with strings or with KeySym representing the key triggering the event.
     * @param ukke boolean Value
     */

    virtual void setUsingKeySymKeyEvents(const bool& ukke) /* limited(INode::STARTED) */
      throw(const Exception&, const IllegalStateException&);
    /**
     * Returns Value of boolean Variable UsingKeySymKeyEvents.
     */

    virtual bool isUsingKeySymKeyEvents()
      throw(const Exception&);
    /**
     * Turn XAutoRepeat on or off
     */

    virtual void setXAutoRepeat(const bool& on)
      throw(const Exception&);
    /**
     * Is XAutoRepeat on or off?
     */

    virtual bool isXAutoRepeat()
      throw(const Exception&);


  };

  typedef auto_ptr<IXDisplayEventSender> IXDisplayEventSender_var;




}

#endif
