
#include "IOverlay.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IOverlay::setAlpha(const float& alpha)
      throw(const Exception&)
  {
    TInValue<float > alpha_value(alpha, false);
    Event event("IOverlay::setAlpha", &alpha_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  float IOverlay::getAlpha() const
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    Event event("IOverlay::getAlpha");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOverlay::setForeground(const int& x, const int& y, const int& w, const int& h)
      throw(const Exception&)
  {
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > w_value(w, false);
    TInValue<int > h_value(h, false);
    Event event("IOverlay::setForeground", &x_value, &y_value, &w_value, &h_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IOverlay::setForegroundSegment(const int& x, const int& y, const int& w, const int& h)
      throw(const Exception&)
  {
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > w_value(w, false);
    TInValue<int > h_value(h, false);
    Event event("IOverlay::setForegroundSegment", &x_value, &y_value, &w_value, &h_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Result IOverlay::disableOverlay() /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOverlay::disableOverlay");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IOverlay::IOverlay_type = "IOverlay";
  TIFactoryObject<IOverlay > __IOverlay_prototype("IOverlay");


  const char* IOverlay::setAlpha_event = "IOverlay::setAlpha";
  const char* IOverlay::getAlpha_event = "IOverlay::getAlpha";
  const char* IOverlay::setForeground_event = "IOverlay::setForeground";
  const char* IOverlay::setForegroundSegment_event = "IOverlay::setForegroundSegment";
  const char* IOverlay::disableOverlay_event = "IOverlay::disableOverlay";
  Event* IOverlay::create_disableOverlay()
  {

    return new Event("IOverlay::disableOverlay");
  }



}


