
#ifndef NMM_IMagickManipulationNode_HPP
#define NMM_IMagickManipulationNode_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IMagickManipulationNode : virtual public Interface { 
  public:
    IMagickManipulationNode() {  setTypeInfo(TypeInfo(IMagickManipulationNode::IMagickManipulationNode_type, "NMM")); };

    IMagickManipulationNode(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IMagickManipulationNode::IMagickManipulationNode_type, "NMM")); setEventReceiver(parent); };

    virtual ~IMagickManipulationNode() {};

    static const char* IMagickManipulationNode_type;

    virtual  string getInterfaceType() const { return IMagickManipulationNode_type; }

    Interface* copy() const { return new IMagickManipulationNode(*this); }
    static const char* setBorderColor_event;
    static const char* clearManipulators_event;
    static const char* adaptiveThreshold_event;
    static const char* addNoise_event;
    static const char* annotate_event;
    static const char* blur_event;
    static const char* border_event;
    static const char* channel_event;
    static const char* charcoal_event;
    static const char* chop_event;
    static const char* colorize_event;
    static const char* comment_event;
    static const char* contrast_event;
    static const char* crop_event;
    static const char* cycleColormap_event;
    static const char* despeckle_event;
    static const char* edge_event;
    static const char* emboss_event;
    static const char* enhance_event;
    static const char* equalize_event;
    static const char* erase_event;
    static const char* flip_event;
    static const char* floodFillColor_event;
    static const char* floodFillOpacity_event;
    static const char* flop_event;
    static const char* frame_event;
    static const char* gamma_event;
    static const char* gaussianBlur_event;
    static const char* implode_event;
    static const char* label_event;
    static const char* magnify_event;
    static const char* matteFloodfill_event;
    static const char* medianFilter_event;
    static const char* minify_event;
    static const char* modulate_event;
    static const char* negate_event;
    static const char* normalize_event;
    static const char* oilPaint_event;
    static const char* opacity_event;
    static const char* opaque_event;
    static const char* quantize_event;
    static const char* raise_event;
    static const char* reduceNoise_event;
    static const char* roll_event;
    static const char* rotate_event;
    static const char* sample_event;
    static const char* scale_event;
    static const char* segment_event;
    static const char* shade_event;
    static const char* sharpen_event;
    static const char* shave_event;
    static const char* shear_event;
    static const char* solarize_event;
    static const char* spread_event;
    static const char* swirl_event;
    static const char* threshold_event;
    static const char* transform_event;
    static const char* transparent_event;
    static const char* trim_event;
    static const char* unsharpmask_event;
    static const char* wave_event;
    static const char* zoom_event;

    virtual void setBorderColor(const double& r, const double& g, const double& b)
      throw(const Exception&);

    virtual void clearManipulators()
      throw(const Exception&);

    virtual void adaptiveThreshold(const unsigned int& width, const unsigned int& height, const unsigned int& offset = 0)
      throw(const Exception&);

    virtual void addNoise(const string& noise_type)
      throw(const Exception&);

    virtual void annotate(const string& text, const string& bounding_area, const string& gravity, const double& degrees)
      throw(const Exception&);

    virtual void blur(const double& radius = 1, const double& sigma = 0.5)
      throw(const Exception&);

    virtual void border(const string& geometry = "6x6+0+0")
      throw(const Exception&);

    virtual void channel(const string& layer)
      throw(const Exception&);

    virtual void charcoal(const double& radius = 1, const double& sigma = 0.5)
      throw(const Exception&);

    virtual void chop(const string& geometry)
      throw(const Exception&);

    virtual void colorize(const unsigned int& opacity_red, const unsigned int& opacity_green, const unsigned int& opacity_blue, const double& pen_color_r, const double& pen_color_g, const double& pen_color_b)
      throw(const Exception&);

    virtual void comment(const string& comment)
      throw(const Exception&);

    virtual void contrast(const unsigned int& sharpen)
      throw(const Exception&);

    virtual void crop(const string& geometry)
      throw(const Exception&);

    virtual void cycleColormap(const int& amount)
      throw(const Exception&);

    virtual void despeckle()
      throw(const Exception&);

    virtual void edge(const unsigned int& radius = 0.0)
      throw(const Exception&);

    virtual void emboss(const double& radius = 1, const double& sigma = 0.5)
      throw(const Exception&);

    virtual void enhance()
      throw(const Exception&);

    virtual void equalize()
      throw(const Exception&);

    virtual void erase()
      throw(const Exception&);

    virtual void flip()
      throw(const Exception&);

    virtual void floodFillColor(const unsigned int& x, const unsigned int& y, const double& fill_color_r, const double& fill_color_g, const double& fill_color_b)
      throw(const Exception&);

    virtual void floodFillOpacity(const long& x, const long& y, const unsigned int& opacity, const string& method)
      throw(const Exception&);

    virtual void flop()
      throw(const Exception&);

    virtual void frame(const unsigned int& width, const unsigned int& height, const int& x, const int& y, const int& inner_bevel = 0, const int& outer_bevel = 0)
      throw(const Exception&);

    virtual void gamma(const double& gamma_red, const double& gamma_green, const double& gamma_blue)
      throw(const Exception&);

    virtual void gaussianBlur(const double& width, const double& sigma)
      throw(const Exception&);

    virtual void implode(const double& factor)
      throw(const Exception&);

    virtual void label(const string& name)
      throw(const Exception&);

    virtual void magnify()
      throw(const Exception&);

    virtual void matteFloodfill(const double& target_r, const double& target_g, const double& target_b, const unsigned int& opacity, const int& x, const int& y, const string& method)
      throw(const Exception&);

    virtual void medianFilter(const double& radius = 0.0)
      throw(const Exception&);

    virtual void minify()
      throw(const Exception&);

    virtual void modulate(const double& brightness, const double& saturation, const double& hue)
      throw(const Exception&);

    virtual void negate()
      throw(const Exception&);

    virtual void normalize()
      throw(const Exception&);

    virtual void oilPaint(const unsigned int& radius = 3)
      throw(const Exception&);

    virtual void opacity(const unsigned int& opacity)
      throw(const Exception&);

    virtual void opaque(const double& opaque_color_r, const double& opaque_color_g, const double& opaque_color_b, const double& pen_color_r, const double& pen_color_g, const double& pen_color_b)
      throw(const Exception&);

    virtual void quantize(const bool& measureError = false)
      throw(const Exception&);

    virtual void raise(const string& geometry, const bool& raisedFlag = false)
      throw(const Exception&);

    virtual void reduceNoise(const unsigned int& order)
      throw(const Exception&);

    virtual void roll(const int& columns, const int& rows)
      throw(const Exception&);

    virtual void rotate(const double& degrees)
      throw(const Exception&);

    virtual void sample(const string& geometry)
      throw(const Exception&);

    virtual void scale(const string& geometry)
      throw(const Exception&);

    virtual void segment(const double& cluster_threshold = 1.0, const double& smoothing_threshold = 1.5)
      throw(const Exception&);

    virtual void shade(const double& azimuth = 30, const double& elevation = 30, const bool& color_shading = false)
      throw(const Exception&);

    virtual void sharpen(const double& radius = 1, const double& sigma = 0.5)
      throw(const Exception&);

    virtual void shave(const string& geometry)
      throw(const Exception&);

    virtual void shear(const double& x_shear_angle, const double& y_shear_angle)
      throw(const Exception&);

    virtual void solarize(const double& factor = 50.0)
      throw(const Exception&);

    virtual void spread(const unsigned int& amount = 3)
      throw(const Exception&);

    virtual void swirl(const double& degrees)
      throw(const Exception&);

    virtual void threshold(const double& threshold)
      throw(const Exception&);

    virtual void transform(const string& image_geometry, const string& crop_geometry)
      throw(const Exception&);

    virtual void transparent(const double& color_r, const double& color_g, const double& color_b)
      throw(const Exception&);

    virtual void trim()
      throw(const Exception&);

    virtual void unsharpmask(const double& radius, const double& sigma, const double& amount, const double& threshold)
      throw(const Exception&);

    virtual void wave(const double& amplitude = 25.0, const double& wavelength = 150.0)
      throw(const Exception&);

    virtual void zoom(const string& geometry)
      throw(const Exception&);


  };

  typedef auto_ptr<IMagickManipulationNode> IMagickManipulationNode_var;




}

#endif
