
#ifndef NMM_IBitmapReader_HPP
#define NMM_IBitmapReader_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IBitmapReader : virtual public Interface { 
  public:    /** Bitmap format. */

  enum Format { 
    RGBA = 0 , 
    YV12 = 1 , 
    YV12A = 2  };

    IBitmapReader() {  setTypeInfo(TypeInfo(IBitmapReader::IBitmapReader_type, "NMM")); };

    IBitmapReader(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IBitmapReader::IBitmapReader_type, "NMM")); setEventReceiver(parent); };

    virtual ~IBitmapReader() {};

    static const char* IBitmapReader_type;

    virtual  string getInterfaceType() const { return IBitmapReader_type; }

    Interface* copy() const { return new IBitmapReader(*this); }
    static const char* loadImage_event;
    static const char* getImage_event;
    static const char* createImage_event;
    static const char* freeImage_event;
    static const char* freeAll_event;
    /**
     * loads a (PNG) image that can be accessed by the given identifier
     * @param id identifier for accessing image later
     * @param filename filename of the (png) image
     * @param f color format that should be used (RGBA, YV12, YV12A)
     * @returns true, if bitmap could be read
     * Note: an image that is already saved under the given id, will be deleted before automatically
     */

    virtual bool loadImage(const string& id, const string& filename, const IBitmapReader::Format& f = IBitmapReader::RGBA)
      throw(const Exception&);
    /**
     * Get width and height of an image.
     * @param id identifier of the image
     * @param w width of the image
     * @param h width of the image
     */

    virtual void getImage(const string& id, unsigned int& w, unsigned int& h)
      throw(const Exception&);
    /**
     * Creates an image with the given color.
     * @param id identifier for accessing image later
     * @param width width of the bitmap
     * @param height height of the bitmap
     * @param color color of the image (RGBA)
     * @param f color format that should be used (RGBA, YV12, YV12A)
     */

    virtual bool createImage(const string& id, const unsigned int& width, const unsigned int& height, const unsigned int& color1, const unsigned int& color2, const unsigned int& color3, const IBitmapReader::Format& f = IBitmapReader::RGBA)
      throw(const Exception&);
    /** Deletes image with the given id */

    virtual void freeImage(const string& id)
      throw(const Exception&);
    /** Deletes all read images */

    virtual void freeAll()
      throw(const Exception&);


  };

  typedef auto_ptr<IBitmapReader> IBitmapReader_var;

  OValueStream& operator<<(OValueStream& ov, const IBitmapReader::Format& t);
  IValueStream& operator>>(IValueStream& iv, IBitmapReader::Format& t);



}

#endif
