
#include "IButtons.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IButtons::addButton(const int& buttonid, const string& label, const int& x, const int& y, const string& inactive_id, const string& active_id, const string& select_id)
      throw(const Exception&)
  {
    TInValue<int > buttonid_value(buttonid, false);
    TInValue<string > label_value(label, false);
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<string > inactive_id_value(inactive_id, false);
    TInValue<string > active_id_value(active_id, false);
    TInValue<string > select_id_value(select_id, false);
    Event event("IButtons::addButton", &buttonid_value, &label_value, &x_value, &y_value, &inactive_id_value, &active_id_value, &select_id_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  string IButtons::getButtonLabel(const int& button_id)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<int > button_id_value(button_id, false);
    Event event("IButtons::getButtonLabel", &button_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IButtons::getSelectedButtonLabel()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IButtons::getSelectedButtonLabel");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool IButtons::getButtonState(const int& button_id)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<int > button_id_value(button_id, false);
    Event event("IButtons::getButtonState", &button_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IButtons::deleteButton(const int& button_id)
      throw(const Exception&)
  {
    TInValue<int > button_id_value(button_id, false);
    Event event("IButtons::deleteButton", &button_id_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IButtons::deleteAllButtons()
      throw(const Exception&)
  {
    Event event("IButtons::deleteAllButtons");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int IButtons::selectButton(const int& buttonid)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<int > buttonid_value(buttonid, false);
    Event event("IButtons::selectButton", &buttonid_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IButtons::setStep(const unsigned int& value)
      throw(const Exception&)
  {
    TInValue<unsigned int > value_value(value, false);
    Event event("IButtons::setStep", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IButtons::deselectButton()
      throw(const Exception&)
  {
    Event event("IButtons::deselectButton");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IButtons::activateButton(const int& button_id)
      throw(const Exception&)
  {
    TInValue<int > button_id_value(button_id, false);
    Event event("IButtons::activateButton", &button_id_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IButtons::deactivateButton(const int& button_id)
      throw(const Exception&)
  {
    TInValue<int > button_id_value(button_id, false);
    Event event("IButtons::deactivateButton", &button_id_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IButtons::deactivateAllButtons()
      throw(const Exception&)
  {
    Event event("IButtons::deactivateAllButtons");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IButtons::IButtons_type = "IButtons";
  TIFactoryObject<IButtons > __IButtons_prototype("IButtons");


  const char* IButtons::addButton_event = "IButtons::addButton";
  const char* IButtons::getButtonLabel_event = "IButtons::getButtonLabel";
  const char* IButtons::getSelectedButtonLabel_event = "IButtons::getSelectedButtonLabel";
  const char* IButtons::getButtonState_event = "IButtons::getButtonState";
  const char* IButtons::deleteButton_event = "IButtons::deleteButton";
  const char* IButtons::deleteAllButtons_event = "IButtons::deleteAllButtons";
  const char* IButtons::selectButton_event = "IButtons::selectButton";
  const char* IButtons::setStep_event = "IButtons::setStep";
  const char* IButtons::deselectButton_event = "IButtons::deselectButton";
  const char* IButtons::activateButton_event = "IButtons::activateButton";
  const char* IButtons::deactivateButton_event = "IButtons::deactivateButton";
  const char* IButtons::deactivateAllButtons_event = "IButtons::deactivateAllButtons";


}


