
#ifndef NMM_IScrollable_HPP
#define NMM_IScrollable_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   * Scrollable item.
   * If an item (need not be a widget) has support for scrollbars, it should
   * implement this interface.
   */

  class IScrollable : virtual public Interface { 
  public:
    IScrollable() {  setTypeInfo(TypeInfo(IScrollable::IScrollable_type, "NMM")); };

    IScrollable(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IScrollable::IScrollable_type, "NMM")); setEventReceiver(parent); };

    virtual ~IScrollable() {};

    static const char* IScrollable_type;

    virtual  string getInterfaceType() const { return IScrollable_type; }

    Interface* copy() const { return new IScrollable(*this); }
    static const char* getScrollBarHeight_event;
    static const char* getScrollBarWidth_event;
    static const char* getScrollBarYPosition_event;
    static const char* getScrollBarXPosition_event;
    /**
     * Get height of vertical scrollbar.
     * Returns the actual scrollbar height (per 10000).
     * That means a value of 10000 indicates a full scrollbar height.
     */

    virtual int getScrollBarHeight() const
      throw(const Exception&);
    /**
     * Get width of horizontal scrollbar.
     * Returns the actual scrollbar width (per 10000).
     * That means a value of 10000 indicates a full scrollbar width.
     */

    virtual int getScrollBarWidth() const
      throw(const Exception&);
    /**
     * Get vertical scrollbar position.
     * Returns the actual scrollbar position (per 10000).
     */

    virtual int getScrollBarYPosition() const
      throw(const Exception&);
    /**
     * Get horizontal scrollbar position.
     * Returns the actual scrollbar position (per 10000).
     */

    virtual int getScrollBarXPosition() const
      throw(const Exception&);


  };

  typedef auto_ptr<IScrollable> IScrollable_var;




}

#endif
