
#include "ITextView.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void ITextView::setFont(const string& value)
      throw(const Exception&)
  {
    TInValue<string > value_value(value, false);
    Event event("ITextView::setFont", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int ITextView::selectLine(const unsigned int& line)
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<unsigned int > line_value(line, false);
    Event event("ITextView::selectLine", &line_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int ITextView::selectLine2(const unsigned int& line)
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<unsigned int > line_value(line, false);
    Event event("ITextView::selectLine2", &line_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::deSelectLine()
      throw(const Exception&)
  {
    Event event("ITextView::deSelectLine");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::deSelectLine2()
      throw(const Exception&)
  {
    Event event("ITextView::deSelectLine2");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int ITextView::getSelectedLine() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("ITextView::getSelectedLine");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int ITextView::getSelectedLine2() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("ITextView::getSelectedLine2");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ITextView::getLineContent(const unsigned int& line) const
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<unsigned int > line_value(line, false);
    Event event("ITextView::getLineContent", &line_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::changeLineContent(const unsigned int& line, const string& text)
      throw(const Exception&)
  {
    TInValue<unsigned int > line_value(line, false);
    TInValue<string > text_value(text, false);
    Event event("ITextView::changeLineContent", &line_value, &text_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  string ITextView::getLineURL(const unsigned int& line) const
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<unsigned int > line_value(line, false);
    Event event("ITextView::getLineURL", &line_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IWidget* ITextView::getLineWidget(const unsigned int& line) const
      throw(const Exception&)
  {
    TOutValue<IWidget* > return_value;
    TInValue<unsigned int > line_value(line, false);
    Event event("ITextView::getLineWidget", &line_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::setAutoFocus(const bool& value)
      throw(const Exception&)
  {
    TInValue<bool > value_value(value, false);
    Event event("ITextView::setAutoFocus", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::setSelectionColor(const u_int32_t& color)
      throw(const Exception&)
  {
    TInValue<u_int32_t > color_value(color, false);
    Event event("ITextView::setSelectionColor", &color_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::setSelectionColor2(const u_int32_t& color)
      throw(const Exception&)
  {
    TInValue<u_int32_t > color_value(color, false);
    Event event("ITextView::setSelectionColor2", &color_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::reset()
      throw(const Exception&)
  {
    Event event("ITextView::reset");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::setMaxLines(const unsigned int& value)
      throw(const Exception&)
  {
    TInValue<unsigned int > value_value(value, false);
    Event event("ITextView::setMaxLines", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int ITextView::getMaxLines() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("ITextView::getMaxLines");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int ITextView::setStartLine(const unsigned int& start)
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<unsigned int > start_value(start, false);
    Event event("ITextView::setStartLine", &start_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int ITextView::getStartLine() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("ITextView::getStartLine");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int ITextView::getNumLines() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("ITextView::getNumLines");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::focusSelection()
      throw(const Exception&)
  {
    Event event("ITextView::focusSelection");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::insertLine(const int& pos, const string& text, const string& url /* = "" */)
      throw(const Exception&)
  {
    TInValue<int > pos_value(pos, false);
    TInValue<string > text_value(text, false);
    TInValue<string > url_value(url, false);
    Event event("ITextView::insertLine", &pos_value, &text_value, &url_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int ITextView::addLine(const string& text, const string& url /* = "" */)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<string > text_value(text, false);
    TInValue<string > url_value(url, false);
    Event event("ITextView::addLine", &text_value, &url_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::addLineWidget(const unsigned int& line, const IWidget& widget)
      throw(const Exception&)
  {
    TInValue<unsigned int > line_value(line, false);
    TInValue<IWidget > widget_value(widget, false);
    Event event("ITextView::addLineWidget", &line_value, &widget_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::deleteLine(const unsigned int& line)
      throw(const Exception&)
  {
    TInValue<unsigned int > line_value(line, false);
    Event event("ITextView::deleteLine", &line_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::deleteAllLines()
      throw(const Exception&)
  {
    Event event("ITextView::deleteAllLines");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::markLine(const unsigned int& line)
      throw(const Exception&)
  {
    TInValue<unsigned int > line_value(line, false);
    Event event("ITextView::markLine", &line_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITextView::unmarkLine(const unsigned int& line)
      throw(const Exception&)
  {
    TInValue<unsigned int > line_value(line, false);
    Event event("ITextView::unmarkLine", &line_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  list<unsigned int >  ITextView::getMarkedLines()
      throw(const Exception&)
  {
    TOutValue<list<unsigned int >  > return_value;
    Event event("ITextView::getMarkedLines");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITextView::ITextView_type = "ITextView";
  TIFactoryObject<ITextView > __ITextView_prototype("ITextView");


  const char* ITextView::setFont_event = "ITextView::setFont";
  const char* ITextView::selectLine_event = "ITextView::selectLine";
  const char* ITextView::selectLine2_event = "ITextView::selectLine2";
  const char* ITextView::deSelectLine_event = "ITextView::deSelectLine";
  const char* ITextView::deSelectLine2_event = "ITextView::deSelectLine2";
  const char* ITextView::getSelectedLine_event = "ITextView::getSelectedLine";
  const char* ITextView::getSelectedLine2_event = "ITextView::getSelectedLine2";
  const char* ITextView::getLineContent_event = "ITextView::getLineContent";
  const char* ITextView::changeLineContent_event = "ITextView::changeLineContent";
  const char* ITextView::getLineURL_event = "ITextView::getLineURL";
  const char* ITextView::getLineWidget_event = "ITextView::getLineWidget";
  const char* ITextView::setAutoFocus_event = "ITextView::setAutoFocus";
  const char* ITextView::setSelectionColor_event = "ITextView::setSelectionColor";
  const char* ITextView::setSelectionColor2_event = "ITextView::setSelectionColor2";
  const char* ITextView::reset_event = "ITextView::reset";
  const char* ITextView::setMaxLines_event = "ITextView::setMaxLines";
  const char* ITextView::getMaxLines_event = "ITextView::getMaxLines";
  const char* ITextView::setStartLine_event = "ITextView::setStartLine";
  const char* ITextView::getStartLine_event = "ITextView::getStartLine";
  const char* ITextView::getNumLines_event = "ITextView::getNumLines";
  const char* ITextView::focusSelection_event = "ITextView::focusSelection";
  const char* ITextView::insertLine_event = "ITextView::insertLine";
  const char* ITextView::addLine_event = "ITextView::addLine";
  const char* ITextView::addLineWidget_event = "ITextView::addLineWidget";
  const char* ITextView::deleteLine_event = "ITextView::deleteLine";
  const char* ITextView::deleteAllLines_event = "ITextView::deleteAllLines";
  const char* ITextView::markLine_event = "ITextView::markLine";
  const char* ITextView::unmarkLine_event = "ITextView::unmarkLine";
  const char* ITextView::getMarkedLines_event = "ITextView::getMarkedLines";


}


