
#ifndef NMM_IIVTVImpl_HPP
#define NMM_IIVTVImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IIVTV.hpp"

#include "ITVImpl.hpp"

namespace NMM
{
  /** Interface for IVTV driver.
   */

  class IIVTVImpl  : virtual public ITVImpl { 
  public:
    IIVTVImpl() { addInterface(new IIVTV(*this)); registerEvents(); };

    virtual ~IIVTVImpl() {};

  protected:
    /** @name Codec.
     */
//@{
    /** This method returns the codec parameters.
     */

    virtual IVTVCodec getCodecParams() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the codec parameters.
     */

    virtual void setCodecParams(const IVTVCodec& codec) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the GOP size
     */

    virtual void setGOPSize(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the number of B-Frames
     */

    virtual void setBFrames(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the GOP closure mode of the codec (1=close GOP, 0=open GOP)
     */

    virtual void setGOPClosure(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets inverse telecine (1=ON, 0=OFF)
     */

    virtual void setPulldown(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets bitrate
     */

    virtual void setBitrate(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets peak bitrate (must be greater than bitrate)
     */

    virtual void setPeakBitrate(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the Streamtype
     *   IVTV_STREAM_PS     = 0
     *   IVTV_STREAM_TS     = 1
     *   IVTV_STREAM_MPEG1  = 2
     *   IVTV_STREAM_PES_AV = 3
     *   IVTV_STREAM_PES_V  = 5
     *   IVTV_STREAM_PES_A  = 7
     *   IVTV_STREAM_DVD    = 10
    */

    virtual void setStreamType(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets bitrate mode
     */

    virtual void setBitrateMode(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets DNRMode
     */

    virtual void setDNRMode(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets DNRType
     */

    virtual void setDNRType(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets DNRSpatial
     */

    virtual void setDNRSpatial(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets DNRTemporal
     */

    virtual void setDNRTemporal(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets aspect
     */

    virtual void setAspect(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;

//@}

  private:
    void registerEvents();
  };

}

#endif
