
#include "IVISCACamera.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IVISCACamera::IF_Clear(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::IF_Clear", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AddressSet()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IVISCACamera::AddressSet");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::getNumberOfCameras(unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TOutValue<unsigned char > value_value(value);
    Event event("IVISCACamera::getNumberOfCameras", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value = value_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::CommandCancel(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    Event event("IVISCACamera::CommandCancel", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PowerOn(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PowerOn", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PowerOff(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PowerOff", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutoPowerOff(const unsigned char& value1, const int& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    Event event("IVISCACamera::AutoPowerOff", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ZoomStop(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ZoomStop", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ZoomTeleStd(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ZoomTeleStd", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ZoomWideStd(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ZoomWideStd", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ZoomTeleVar(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    Event event("IVISCACamera::ZoomTeleVar", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ZoomWideVar(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    Event event("IVISCACamera::ZoomWideVar", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ZoomDirect(const unsigned char& value1, const int& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    Event event("IVISCACamera::ZoomDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ZoomDigitalZoomOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<bool > value2_value(value2, false);
    Event event("IVISCACamera::ZoomDigitalZoomOnOff", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusStop(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusStop", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusFarStd(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusFarStd", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusNearStd(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusNearStd", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusFarVar(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    Event event("IVISCACamera::FocusFarVar", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusNearVar(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    Event event("IVISCACamera::FocusNearVar", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::FocusDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusAutoFocus(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusAutoFocus", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusManualFocus(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusManualFocus", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusAutoManual(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusAutoManual", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusOnePushTrigger(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusOnePushTrigger", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusInfinity(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusInfinity", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusAFSensHigh(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusAFSensHigh", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusAFSensLow(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::FocusAFSensLow", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusNearLimit(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::FocusNearLimit", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WhiteBalance(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::WhiteBalance", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WhiteBalanceAuto(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WhiteBalanceAuto", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WhiteBalanceIndoor(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WhiteBalanceIndoor", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WhiteBalanceOutdoor(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WhiteBalanceOutdoor", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WhiteBalanceOnePush(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WhiteBalanceOnePush", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WhiteBalanceAT(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WhiteBalanceAT", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WhiteBalanceManual(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WhiteBalanceManual", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WhiteBalanceOnePushTrigger(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WhiteBalanceOnePushTrigger", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::RGainReset(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::RGainReset", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::RGainUp(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::RGainUp", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::RGainDown(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::RGainDown", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::RGainDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::RGainDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BGainReset(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::BGainReset", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BGainUp(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::BGainUp", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BGainDown(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::BGainDown", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BGainDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::BGainDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutomaticExposureFull(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::AutomaticExposureFull", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutomaticExposureManual(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::AutomaticExposureManual", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutomaticExposureShutterPriority(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::AutomaticExposureShutterPriority", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutomaticExposureIrisPriority(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::AutomaticExposureIrisPriority", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutomaticExposureGainPriority(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::AutomaticExposureGainPriority", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutomaticExposureBright(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::AutomaticExposureBright", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutomaticExposureShutterAuto(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::AutomaticExposureShutterAuto", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutomaticExposureIrisAuto(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::AutomaticExposureIrisAuto", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutomaticExposureGainAuto(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::AutomaticExposureGainAuto", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::SlowShutter(const unsigned char& value1, const bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<bool > value2_value(value2, false);
    Event event("IVISCACamera::SlowShutter", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ShutterReset(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ShutterReset", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ShutterUp(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ShutterUp", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ShutterDown(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ShutterDown", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ShutterDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::ShutterDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::IrisReset(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::IrisReset", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::IrisUp(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::IrisUp", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::IrisDown(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::IrisDown", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::IrisDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::IrisDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::GainReset(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::GainReset", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::GainUp(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::GainUp", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::GainDown(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::GainDown", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::GainDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::GainDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BrightReset(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::BrightReset", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BrightUp(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::BrightUp", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BrightDown(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::BrightDown", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BrightDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::BrightDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ExposureCompensationOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<bool > value2_value(value2, false);
    Event event("IVISCACamera::ExposureCompensationOnOff", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ExposureCompensationReset(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ExposureCompensationReset", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ExposureCompensationUp(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ExposureCompensationUp", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ExposureCompensationDown(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ExposureCompensationDown", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ExposureCompensationDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::ExposureCompensationDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BackLightOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<bool > value2_value(value2, false);
    Event event("IVISCACamera::BackLightOnOff", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ApertureReset(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ApertureReset", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ApertureUp(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ApertureUp", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ApertureDown(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::ApertureDown", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ApertureDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned short > value2_value(value2, false);
    Event event("IVISCACamera::ApertureDirect", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WideModeOff(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WideModeOff", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WideModeCinema(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WideModeCinema", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WideMode16to9Full(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::WideMode16to9Full", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::LR_Reverse(const unsigned char& value1, const bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<bool > value2_value(value2, false);
    Event event("IVISCACamera::LR_Reverse", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::Freeze(const unsigned char& value1, const bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<bool > value2_value(value2, false);
    Event event("IVISCACamera::Freeze", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectOff(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PictureEffectOff", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectPastel(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PictureEffectPastel", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectNegArt(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PictureEffectNegArt", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectSepia(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PictureEffectSepia", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectBW(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PictureEffectBW", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectSolarize(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PictureEffectSolarize", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectMosaic(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PictureEffectMosaic", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectSlim(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PictureEffectSlim", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectStretch(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PictureEffectStretch", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DigitalEffectOff(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::DigitalEffectOff", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DigitalEffectStill(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::DigitalEffectStill", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DigitalEffectFlash(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::DigitalEffectFlash", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DigitalEffectLumi(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::DigitalEffectLumi", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DigitalEffectTrail(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::DigitalEffectTrail", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DigitalEffectLevel(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::DigitalEffectLevel", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::Memory(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::Memory", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::MemoryReset(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    Event event("IVISCACamera::MemoryReset", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::MemorySet(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    Event event("IVISCACamera::MemorySet", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::MemoryRecall(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    Event event("IVISCACamera::MemoryRecall", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DataScreenOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<bool > value2_value(value2, false);
    Event event("IVISCACamera::DataScreenOnOff", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::IRReceiveOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<bool > value2_value(value2, false);
    Event event("IVISCACamera::IRReceiveOnOff", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::IRReceiveReturn(const unsigned char& value1, const bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<bool > value2_value(value2, false);
    Event event("IVISCACamera::IRReceiveReturn", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveMotion(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    TInValue<unsigned char > value4_value(value4, false);
    TInValue<unsigned char > value5_value(value5, false);
    Event event("IVISCACamera::PanTiltDriveMotion", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveUp(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::PanTiltDriveUp", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveDown(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::PanTiltDriveDown", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveLeft(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::PanTiltDriveLeft", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveRight(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::PanTiltDriveRight", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveUpLeft(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::PanTiltDriveUpLeft", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveUpRight(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::PanTiltDriveUpRight", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveDownLeft(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::PanTiltDriveDownLeft", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveDownRight(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::PanTiltDriveDownRight", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveStop(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<unsigned char > value3_value(value3, false);
    Event event("IVISCACamera::PanTiltDriveStop", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDrivePosition(const unsigned char& value1, const unsigned char& value2, const int& value3, const int& value4, const unsigned char& value5, const unsigned char& value6)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<int > value3_value(value3, false);
    TInValue<int > value4_value(value4, false);
    TInValue<unsigned char > value5_value(value5, false);
    TInValue<unsigned char > value6_value(value6, false);
    Event event("IVISCACamera::PanTiltDrivePosition", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value, &value6_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveAbsolute(const unsigned char& value1, const int& value2, const int& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    TInValue<int > value3_value(value3, false);
    TInValue<unsigned char > value4_value(value4, false);
    TInValue<unsigned char > value5_value(value5, false);
    Event event("IVISCACamera::PanTiltDriveAbsolute", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveRelative(const unsigned char& value1, const int& value2, const int& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    TInValue<int > value3_value(value3, false);
    TInValue<unsigned char > value4_value(value4, false);
    TInValue<unsigned char > value5_value(value5, false);
    Event event("IVISCACamera::PanTiltDriveRelative", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveAbsoluteDeg(const unsigned char& value1, const int& value2, const int& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    TInValue<int > value3_value(value3, false);
    TInValue<unsigned char > value4_value(value4, false);
    TInValue<unsigned char > value5_value(value5, false);
    Event event("IVISCACamera::PanTiltDriveAbsoluteDeg", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveRelativeDeg(const unsigned char& value1, const int& value2, const int& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    TInValue<int > value3_value(value3, false);
    TInValue<unsigned char > value4_value(value4, false);
    TInValue<unsigned char > value5_value(value5, false);
    Event event("IVISCACamera::PanTiltDriveRelativeDeg", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveHome(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PanTiltDriveHome", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltDriveReset(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::PanTiltDriveReset", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltLimitSet(const unsigned char& value1, const unsigned char& value2, const int& value3, const int& value4)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    TInValue<int > value3_value(value3, false);
    TInValue<int > value4_value(value4, false);
    Event event("IVISCACamera::PanTiltLimitSet", &value1_value, &value2_value, &value3_value, &value4_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltLimitClear(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TInValue<unsigned char > value2_value(value2, false);
    Event event("IVISCACamera::PanTiltLimitClear", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PowerInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::PowerInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutoPowerOffInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::AutoPowerOffInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DZoomModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::DZoomModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ZoomPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::ZoomPosInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::FocusModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::FocusPosInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AFModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::AFModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FocusNearLimitInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::FocusNearLimitInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WhiteBalanceModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    Event event("IVISCACamera::WhiteBalanceModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::RGainInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    Event event("IVISCACamera::RGainInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BGainInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::BGainInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::AutoExposureModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    Event event("IVISCACamera::AutoExposureModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::SlowShutterModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::SlowShutterModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ShutterPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::ShutterPosInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::IrisPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::IrisPosInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::GainPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::GainPosInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BrightPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::BrightPosInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ExposureCompensationModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::ExposureCompensationModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ExposureCompensationPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::ExposureCompensationPosInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::BackLightModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::BackLightModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::ApertureInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::ApertureInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::WideModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    Event event("IVISCACamera::WideModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::LR_ReverseModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::LR_ReverseModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::FreezeModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::FreezeModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PictureEffectModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    Event event("IVISCACamera::PictureEffectModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DigitalEffectModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    Event event("IVISCACamera::DigitalEffectModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DigitalEffectLevelInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    Event event("IVISCACamera::DigitalEffectLevelInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::MemoryInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    Event event("IVISCACamera::MemoryInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DatascreenInq(const unsigned char& value1, bool& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<bool > value2_value(value2);
    Event event("IVISCACamera::DatascreenInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltModeInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    Event event("IVISCACamera::PanTiltModeInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltMaxSpeedInq(const unsigned char& value1, unsigned char& value2, unsigned char& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    TOutValue<unsigned char > value3_value(value3);
    Event event("IVISCACamera::PanTiltMaxSpeedInq", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    value3 = value3_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::PanTiltPosInq(const unsigned char& value1, int& value2, int& value3)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<int > value2_value(value2);
    TOutValue<int > value3_value(value3);
    Event event("IVISCACamera::PanTiltPosInq", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    value3 = value3_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::VideoSystemInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned char > value2_value(value2);
    Event event("IVISCACamera::VideoSystemInq", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IVISCACamera::DeviceTypeInq(const unsigned char& value1, unsigned short& value2, unsigned short& value3, unsigned short& value4, unsigned char& value5)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned char > value1_value(value1, false);
    TOutValue<unsigned short > value2_value(value2);
    TOutValue<unsigned short > value3_value(value3);
    TOutValue<unsigned short > value4_value(value4);
    TOutValue<unsigned char > value5_value(value5);
    Event event("IVISCACamera::DeviceTypeInq", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value2 = value2_value.getRValue();
    value3 = value3_value.getRValue();
    value4 = value4_value.getRValue();
    value5 = value5_value.getRValue();
    return return_value.getRValue();
  }
  
  void IVISCACamera::verbose(const string& value1, const string& value2)
      throw(const Exception&)
  {
    TInValue<string > value1_value(value1, false);
    TInValue<string > value2_value(value2, false);
    Event event("IVISCACamera::verbose", &value1_value, &value2_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool IVISCACamera::islocked(const unsigned char& value)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<unsigned char > value_value(value, false);
    Event event("IVISCACamera::islocked", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IVISCACamera::IVISCACamera_type = "IVISCACamera";
  TIFactoryObject<IVISCACamera > __IVISCACamera_prototype("IVISCACamera");


  const char* IVISCACamera::IF_Clear_event = "IVISCACamera::IF_Clear";
  const char* IVISCACamera::AddressSet_event = "IVISCACamera::AddressSet";
  const char* IVISCACamera::getNumberOfCameras_event = "IVISCACamera::getNumberOfCameras";
  const char* IVISCACamera::CommandCancel_event = "IVISCACamera::CommandCancel";
  const char* IVISCACamera::PowerOn_event = "IVISCACamera::PowerOn";
  const char* IVISCACamera::PowerOff_event = "IVISCACamera::PowerOff";
  const char* IVISCACamera::AutoPowerOff_event = "IVISCACamera::AutoPowerOff";
  const char* IVISCACamera::ZoomStop_event = "IVISCACamera::ZoomStop";
  const char* IVISCACamera::ZoomTeleStd_event = "IVISCACamera::ZoomTeleStd";
  const char* IVISCACamera::ZoomWideStd_event = "IVISCACamera::ZoomWideStd";
  const char* IVISCACamera::ZoomTeleVar_event = "IVISCACamera::ZoomTeleVar";
  const char* IVISCACamera::ZoomWideVar_event = "IVISCACamera::ZoomWideVar";
  const char* IVISCACamera::ZoomDirect_event = "IVISCACamera::ZoomDirect";
  const char* IVISCACamera::ZoomDigitalZoomOnOff_event = "IVISCACamera::ZoomDigitalZoomOnOff";
  const char* IVISCACamera::FocusStop_event = "IVISCACamera::FocusStop";
  const char* IVISCACamera::FocusFarStd_event = "IVISCACamera::FocusFarStd";
  const char* IVISCACamera::FocusNearStd_event = "IVISCACamera::FocusNearStd";
  const char* IVISCACamera::FocusFarVar_event = "IVISCACamera::FocusFarVar";
  const char* IVISCACamera::FocusNearVar_event = "IVISCACamera::FocusNearVar";
  const char* IVISCACamera::FocusDirect_event = "IVISCACamera::FocusDirect";
  const char* IVISCACamera::FocusAutoFocus_event = "IVISCACamera::FocusAutoFocus";
  const char* IVISCACamera::FocusManualFocus_event = "IVISCACamera::FocusManualFocus";
  const char* IVISCACamera::FocusAutoManual_event = "IVISCACamera::FocusAutoManual";
  const char* IVISCACamera::FocusOnePushTrigger_event = "IVISCACamera::FocusOnePushTrigger";
  const char* IVISCACamera::FocusInfinity_event = "IVISCACamera::FocusInfinity";
  const char* IVISCACamera::FocusAFSensHigh_event = "IVISCACamera::FocusAFSensHigh";
  const char* IVISCACamera::FocusAFSensLow_event = "IVISCACamera::FocusAFSensLow";
  const char* IVISCACamera::FocusNearLimit_event = "IVISCACamera::FocusNearLimit";
  const char* IVISCACamera::WhiteBalance_event = "IVISCACamera::WhiteBalance";
  const char* IVISCACamera::WhiteBalanceAuto_event = "IVISCACamera::WhiteBalanceAuto";
  const char* IVISCACamera::WhiteBalanceIndoor_event = "IVISCACamera::WhiteBalanceIndoor";
  const char* IVISCACamera::WhiteBalanceOutdoor_event = "IVISCACamera::WhiteBalanceOutdoor";
  const char* IVISCACamera::WhiteBalanceOnePush_event = "IVISCACamera::WhiteBalanceOnePush";
  const char* IVISCACamera::WhiteBalanceAT_event = "IVISCACamera::WhiteBalanceAT";
  const char* IVISCACamera::WhiteBalanceManual_event = "IVISCACamera::WhiteBalanceManual";
  const char* IVISCACamera::WhiteBalanceOnePushTrigger_event = "IVISCACamera::WhiteBalanceOnePushTrigger";
  const char* IVISCACamera::RGainReset_event = "IVISCACamera::RGainReset";
  const char* IVISCACamera::RGainUp_event = "IVISCACamera::RGainUp";
  const char* IVISCACamera::RGainDown_event = "IVISCACamera::RGainDown";
  const char* IVISCACamera::RGainDirect_event = "IVISCACamera::RGainDirect";
  const char* IVISCACamera::BGainReset_event = "IVISCACamera::BGainReset";
  const char* IVISCACamera::BGainUp_event = "IVISCACamera::BGainUp";
  const char* IVISCACamera::BGainDown_event = "IVISCACamera::BGainDown";
  const char* IVISCACamera::BGainDirect_event = "IVISCACamera::BGainDirect";
  const char* IVISCACamera::AutomaticExposureFull_event = "IVISCACamera::AutomaticExposureFull";
  const char* IVISCACamera::AutomaticExposureManual_event = "IVISCACamera::AutomaticExposureManual";
  const char* IVISCACamera::AutomaticExposureShutterPriority_event = "IVISCACamera::AutomaticExposureShutterPriority";
  const char* IVISCACamera::AutomaticExposureIrisPriority_event = "IVISCACamera::AutomaticExposureIrisPriority";
  const char* IVISCACamera::AutomaticExposureGainPriority_event = "IVISCACamera::AutomaticExposureGainPriority";
  const char* IVISCACamera::AutomaticExposureBright_event = "IVISCACamera::AutomaticExposureBright";
  const char* IVISCACamera::AutomaticExposureShutterAuto_event = "IVISCACamera::AutomaticExposureShutterAuto";
  const char* IVISCACamera::AutomaticExposureIrisAuto_event = "IVISCACamera::AutomaticExposureIrisAuto";
  const char* IVISCACamera::AutomaticExposureGainAuto_event = "IVISCACamera::AutomaticExposureGainAuto";
  const char* IVISCACamera::SlowShutter_event = "IVISCACamera::SlowShutter";
  const char* IVISCACamera::ShutterReset_event = "IVISCACamera::ShutterReset";
  const char* IVISCACamera::ShutterUp_event = "IVISCACamera::ShutterUp";
  const char* IVISCACamera::ShutterDown_event = "IVISCACamera::ShutterDown";
  const char* IVISCACamera::ShutterDirect_event = "IVISCACamera::ShutterDirect";
  const char* IVISCACamera::IrisReset_event = "IVISCACamera::IrisReset";
  const char* IVISCACamera::IrisUp_event = "IVISCACamera::IrisUp";
  const char* IVISCACamera::IrisDown_event = "IVISCACamera::IrisDown";
  const char* IVISCACamera::IrisDirect_event = "IVISCACamera::IrisDirect";
  const char* IVISCACamera::GainReset_event = "IVISCACamera::GainReset";
  const char* IVISCACamera::GainUp_event = "IVISCACamera::GainUp";
  const char* IVISCACamera::GainDown_event = "IVISCACamera::GainDown";
  const char* IVISCACamera::GainDirect_event = "IVISCACamera::GainDirect";
  const char* IVISCACamera::BrightReset_event = "IVISCACamera::BrightReset";
  const char* IVISCACamera::BrightUp_event = "IVISCACamera::BrightUp";
  const char* IVISCACamera::BrightDown_event = "IVISCACamera::BrightDown";
  const char* IVISCACamera::BrightDirect_event = "IVISCACamera::BrightDirect";
  const char* IVISCACamera::ExposureCompensationOnOff_event = "IVISCACamera::ExposureCompensationOnOff";
  const char* IVISCACamera::ExposureCompensationReset_event = "IVISCACamera::ExposureCompensationReset";
  const char* IVISCACamera::ExposureCompensationUp_event = "IVISCACamera::ExposureCompensationUp";
  const char* IVISCACamera::ExposureCompensationDown_event = "IVISCACamera::ExposureCompensationDown";
  const char* IVISCACamera::ExposureCompensationDirect_event = "IVISCACamera::ExposureCompensationDirect";
  const char* IVISCACamera::BackLightOnOff_event = "IVISCACamera::BackLightOnOff";
  const char* IVISCACamera::ApertureReset_event = "IVISCACamera::ApertureReset";
  const char* IVISCACamera::ApertureUp_event = "IVISCACamera::ApertureUp";
  const char* IVISCACamera::ApertureDown_event = "IVISCACamera::ApertureDown";
  const char* IVISCACamera::ApertureDirect_event = "IVISCACamera::ApertureDirect";
  const char* IVISCACamera::WideModeOff_event = "IVISCACamera::WideModeOff";
  const char* IVISCACamera::WideModeCinema_event = "IVISCACamera::WideModeCinema";
  const char* IVISCACamera::WideMode16to9Full_event = "IVISCACamera::WideMode16to9Full";
  const char* IVISCACamera::LR_Reverse_event = "IVISCACamera::LR_Reverse";
  const char* IVISCACamera::Freeze_event = "IVISCACamera::Freeze";
  const char* IVISCACamera::PictureEffectOff_event = "IVISCACamera::PictureEffectOff";
  const char* IVISCACamera::PictureEffectPastel_event = "IVISCACamera::PictureEffectPastel";
  const char* IVISCACamera::PictureEffectNegArt_event = "IVISCACamera::PictureEffectNegArt";
  const char* IVISCACamera::PictureEffectSepia_event = "IVISCACamera::PictureEffectSepia";
  const char* IVISCACamera::PictureEffectBW_event = "IVISCACamera::PictureEffectBW";
  const char* IVISCACamera::PictureEffectSolarize_event = "IVISCACamera::PictureEffectSolarize";
  const char* IVISCACamera::PictureEffectMosaic_event = "IVISCACamera::PictureEffectMosaic";
  const char* IVISCACamera::PictureEffectSlim_event = "IVISCACamera::PictureEffectSlim";
  const char* IVISCACamera::PictureEffectStretch_event = "IVISCACamera::PictureEffectStretch";
  const char* IVISCACamera::DigitalEffectOff_event = "IVISCACamera::DigitalEffectOff";
  const char* IVISCACamera::DigitalEffectStill_event = "IVISCACamera::DigitalEffectStill";
  const char* IVISCACamera::DigitalEffectFlash_event = "IVISCACamera::DigitalEffectFlash";
  const char* IVISCACamera::DigitalEffectLumi_event = "IVISCACamera::DigitalEffectLumi";
  const char* IVISCACamera::DigitalEffectTrail_event = "IVISCACamera::DigitalEffectTrail";
  const char* IVISCACamera::DigitalEffectLevel_event = "IVISCACamera::DigitalEffectLevel";
  const char* IVISCACamera::Memory_event = "IVISCACamera::Memory";
  const char* IVISCACamera::MemoryReset_event = "IVISCACamera::MemoryReset";
  const char* IVISCACamera::MemorySet_event = "IVISCACamera::MemorySet";
  const char* IVISCACamera::MemoryRecall_event = "IVISCACamera::MemoryRecall";
  const char* IVISCACamera::DataScreenOnOff_event = "IVISCACamera::DataScreenOnOff";
  const char* IVISCACamera::IRReceiveOnOff_event = "IVISCACamera::IRReceiveOnOff";
  const char* IVISCACamera::IRReceiveReturn_event = "IVISCACamera::IRReceiveReturn";
  const char* IVISCACamera::PanTiltDriveMotion_event = "IVISCACamera::PanTiltDriveMotion";
  const char* IVISCACamera::PanTiltDriveUp_event = "IVISCACamera::PanTiltDriveUp";
  const char* IVISCACamera::PanTiltDriveDown_event = "IVISCACamera::PanTiltDriveDown";
  const char* IVISCACamera::PanTiltDriveLeft_event = "IVISCACamera::PanTiltDriveLeft";
  const char* IVISCACamera::PanTiltDriveRight_event = "IVISCACamera::PanTiltDriveRight";
  const char* IVISCACamera::PanTiltDriveUpLeft_event = "IVISCACamera::PanTiltDriveUpLeft";
  const char* IVISCACamera::PanTiltDriveUpRight_event = "IVISCACamera::PanTiltDriveUpRight";
  const char* IVISCACamera::PanTiltDriveDownLeft_event = "IVISCACamera::PanTiltDriveDownLeft";
  const char* IVISCACamera::PanTiltDriveDownRight_event = "IVISCACamera::PanTiltDriveDownRight";
  const char* IVISCACamera::PanTiltDriveStop_event = "IVISCACamera::PanTiltDriveStop";
  const char* IVISCACamera::PanTiltDrivePosition_event = "IVISCACamera::PanTiltDrivePosition";
  const char* IVISCACamera::PanTiltDriveAbsolute_event = "IVISCACamera::PanTiltDriveAbsolute";
  const char* IVISCACamera::PanTiltDriveRelative_event = "IVISCACamera::PanTiltDriveRelative";
  const char* IVISCACamera::PanTiltDriveAbsoluteDeg_event = "IVISCACamera::PanTiltDriveAbsoluteDeg";
  const char* IVISCACamera::PanTiltDriveRelativeDeg_event = "IVISCACamera::PanTiltDriveRelativeDeg";
  const char* IVISCACamera::PanTiltDriveHome_event = "IVISCACamera::PanTiltDriveHome";
  const char* IVISCACamera::PanTiltDriveReset_event = "IVISCACamera::PanTiltDriveReset";
  const char* IVISCACamera::PanTiltLimitSet_event = "IVISCACamera::PanTiltLimitSet";
  const char* IVISCACamera::PanTiltLimitClear_event = "IVISCACamera::PanTiltLimitClear";
  const char* IVISCACamera::PowerInq_event = "IVISCACamera::PowerInq";
  const char* IVISCACamera::AutoPowerOffInq_event = "IVISCACamera::AutoPowerOffInq";
  const char* IVISCACamera::DZoomModeInq_event = "IVISCACamera::DZoomModeInq";
  const char* IVISCACamera::ZoomPosInq_event = "IVISCACamera::ZoomPosInq";
  const char* IVISCACamera::FocusModeInq_event = "IVISCACamera::FocusModeInq";
  const char* IVISCACamera::FocusPosInq_event = "IVISCACamera::FocusPosInq";
  const char* IVISCACamera::AFModeInq_event = "IVISCACamera::AFModeInq";
  const char* IVISCACamera::FocusNearLimitInq_event = "IVISCACamera::FocusNearLimitInq";
  const char* IVISCACamera::WhiteBalanceModeInq_event = "IVISCACamera::WhiteBalanceModeInq";
  const char* IVISCACamera::RGainInq_event = "IVISCACamera::RGainInq";
  const char* IVISCACamera::BGainInq_event = "IVISCACamera::BGainInq";
  const char* IVISCACamera::AutoExposureModeInq_event = "IVISCACamera::AutoExposureModeInq";
  const char* IVISCACamera::SlowShutterModeInq_event = "IVISCACamera::SlowShutterModeInq";
  const char* IVISCACamera::ShutterPosInq_event = "IVISCACamera::ShutterPosInq";
  const char* IVISCACamera::IrisPosInq_event = "IVISCACamera::IrisPosInq";
  const char* IVISCACamera::GainPosInq_event = "IVISCACamera::GainPosInq";
  const char* IVISCACamera::BrightPosInq_event = "IVISCACamera::BrightPosInq";
  const char* IVISCACamera::ExposureCompensationModeInq_event = "IVISCACamera::ExposureCompensationModeInq";
  const char* IVISCACamera::ExposureCompensationPosInq_event = "IVISCACamera::ExposureCompensationPosInq";
  const char* IVISCACamera::BackLightModeInq_event = "IVISCACamera::BackLightModeInq";
  const char* IVISCACamera::ApertureInq_event = "IVISCACamera::ApertureInq";
  const char* IVISCACamera::WideModeInq_event = "IVISCACamera::WideModeInq";
  const char* IVISCACamera::LR_ReverseModeInq_event = "IVISCACamera::LR_ReverseModeInq";
  const char* IVISCACamera::FreezeModeInq_event = "IVISCACamera::FreezeModeInq";
  const char* IVISCACamera::PictureEffectModeInq_event = "IVISCACamera::PictureEffectModeInq";
  const char* IVISCACamera::DigitalEffectModeInq_event = "IVISCACamera::DigitalEffectModeInq";
  const char* IVISCACamera::DigitalEffectLevelInq_event = "IVISCACamera::DigitalEffectLevelInq";
  const char* IVISCACamera::MemoryInq_event = "IVISCACamera::MemoryInq";
  const char* IVISCACamera::DatascreenInq_event = "IVISCACamera::DatascreenInq";
  const char* IVISCACamera::PanTiltModeInq_event = "IVISCACamera::PanTiltModeInq";
  const char* IVISCACamera::PanTiltMaxSpeedInq_event = "IVISCACamera::PanTiltMaxSpeedInq";
  const char* IVISCACamera::PanTiltPosInq_event = "IVISCACamera::PanTiltPosInq";
  const char* IVISCACamera::VideoSystemInq_event = "IVISCACamera::VideoSystemInq";
  const char* IVISCACamera::DeviceTypeInq_event = "IVISCACamera::DeviceTypeInq";
  const char* IVISCACamera::verbose_event = "IVISCACamera::verbose";
  const char* IVISCACamera::islocked_event = "IVISCACamera::islocked";


}


