/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "DVBEPGPlugin.hpp"

#include "nmm/base/ProxyObject.hpp"

#include "nmm/base/registry/ReferenceManager.hpp"
#include "nmm/base/registry/PluginManager.hpp"

#include "nmm/interfaces/misc/tvguide/ITVGuideUpdateFinished.hpp"
#include "nmm/interfaces/base/IReferenceCount.hpp"
#include "nmm/interfaces/base/IUsage.hpp"

#include "nmm/utils/utils.hpp"

namespace NMM{

  static DVBEPGPlugin * dvbEpgPlugin = new DVBEPGPlugin();

  DVBEPGPlugin::DVBEPGPlugin() :
    TVGuidePlugin("DVBEPGPlugin"),
    tvmanager(NULL),
    termUpdate(false),
    switchPeriod(60),
    switch_timer(this, &DVBEPGPlugin::switchChannel)
  {
    configFile = getNMMResource("tvguide") + "/DVBEPGPluginConf.xml";
  }

  DVBEPGPlugin::~DVBEPGPlugin(){}

  Result DVBEPGPlugin::doInit(){
    timer = new TimerProducer();
    timer->connectTo(this); 
    timer->start();

    registerEvent("SWITCHCHANNEL", &switch_timer);

    if(tvmanager == NULL) return FAILURE;
    
    tvmanager->setUsageNotify(this->getInterface<IUsageNotify>());

    return SUCCESS;
  }
  Result DVBEPGPlugin::doDeinit(){

    if(tvmanager){
      delete tvmanager;
      tvmanager = NULL;
    }

    return SUCCESS;
  }

  Result  DVBEPGPlugin::doStart(){
    cerr << "starting dvbepg" << endl;
    if(tvmanager) {
      if(tvmanager->setChannel(0) == SUCCESS){
        tvmanager->startGraph();
      }else{
        if(tvmanager->channelUp() == SUCCESS){
          tvmanager->startGraph();
        }
      }
      
      if(tvmanager->getTVCard()){
        ITVGuideProducer_var iepgdvb(tvmanager->getTVCard()->getParentObject()->getInterface<ITVGuideProducer>());
        if(iepgdvb.get()){
          iepgdvb->registerTVGuideServer(tvgobject.get());
        }else{
          GLOBAL_ERROR_STREAM("DVBEPGPLUGIN: could not get ITVGuideProducer-interface" << endl);
          cerr << "DVBEPGPLUGIN: could not get ITVGuideProducer-interface" << endl;
          return FAILURE;
        }
      }else{
        GLOBAL_ERROR_STREAM("DVBEPGPLUGIN: don't have ITVCard-Interface" << endl);
        cerr << "DVBEPGPLUGIN: don't have ITVCard-Interface" << endl;
        return FAILURE;
      }
    }else{
      cerr << "don't have TVManager" << endl;
    }

    //register timer for 1. channel-switch
    Event* e = new Event("SWITCHCHANNEL");
    timer->registerTimer(time(NULL)+switchPeriod, e);

    return SUCCESS;
  }
  
  Result  DVBEPGPlugin::doStop(){
    termUpdate = true;
        
    if(tvmanager){
      cerr << "STOOOOOOOOOOOOOOOP" << endl;
      tvmanager->releaseGraph();
    }

    return SUCCESS;
  }

  Result DVBEPGPlugin::switchChannel(){

    cerr << "DVBEPG: switching channel" << endl;

    Event* e = new Event("SWITCHCHANNEL");
    timer->registerTimer(time(NULL)+switchPeriod, e);

    Result res = tvmanager->channelUp();

    
    if(res == SUCCESS){
      
      //for safety. When the TVCard was changed, the old interface is useless, so we reset it.
      if(tvmanager->getTVCard()){
        ITVGuideProducer_var iepgdvb(tvmanager->getTVCard()->getParentObject()->getInterface<ITVGuideProducer>());
        if(iepgdvb.get()){
          iepgdvb->registerTVGuideServer(tvgobject.get());
        }else{
          GLOBAL_ERROR_STREAM("DVBEPGPLUGIN: could not get ITVGuideProducer-interface" << endl);
          cerr << "DVBEPGPLUGIN: could not get ITVGuideProducer-interface" << endl;
          return FAILURE;
        }
      }
      
      cerr << "restart graph()" << endl;
      tvmanager->startGraph();
    }
    return res;
  }
}
