/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef TVTIMEREXCEPTIONS_HPP
#define TVTIMEREXCEPTIONS_HPP TVTIMEREXCEPTIONS_HPP

#include "nmm/NMMTypes.hpp"

namespace NMM{

  class NoCardFoundException : public Exception{
  public:
    NoCardFoundException() : Exception("No new TVCard found"){}
    NoCardFoundException(string channel) : Exception("No new TVCard found for channel " + channel){}

    NoCardFoundException* copy() const { return new NoCardFoundException(*this); }
  };

  class ActionNotAllowedException : public Exception{
  private:
    NMM::TValue<string> forbiddenAction;
  public:
    ActionNotAllowedException() 
      : Exception("You do'nt have the permisson to perfom this task"){ setTypeName("ActionNotAllowedException"); registerAttributes();}
    ActionNotAllowedException(string action) 
      : Exception("You do'nt have the permisson for " + action){setTypeName("ActionNotAllowedException"); registerAttributes();}

    ActionNotAllowedException* copy() const { return new ActionNotAllowedException(*this); }
    const string& getForbiddenAction() const { return forbiddenAction.getRValue();}
  private:
    void registerAttributes();
  };

  class NoTimerFoundException : public Exception{
  public:
    NoTimerFoundException() : Exception("No Timer found"){}
    NoTimerFoundException* copy() { return new NoTimerFoundException();}
  };

  class TimerAlreadyProgrammedException : public Exception{
  private: 
    NMM::TValue<unsigned int> timerID;
  public:
    TimerAlreadyProgrammedException() : Exception("Timer already programmed"){
      setTypeName("TimerAlreadyProgrammedException"); 
      registerAttributes(); 
      timerID = 0;}
    TimerAlreadyProgrammedException(unsigned int id) : Exception("Timer already programmed"){
      setTypeName("TimerAlreadyProgrammedException"); 
      registerAttributes(); 
      timerID = id;
    }

    TimerAlreadyProgrammedException* copy() const { return new TimerAlreadyProgrammedException(*this);}

    const unsigned int& getTimerID() const {return timerID.getRValue();}
  private:
    void registerAttributes();
  };

  class TimerRecordedException : public Exception{
  private: 
    NMM::TValue<unsigned int> timerID;
  public:
    TimerRecordedException() : Exception("Timer already recorded"){setTypeName("TimerRecordedException"); registerAttributes(); timerID = 0;}
    TimerRecordedException(unsigned int id) : Exception("Timer already programmed"){setTypeName("TimerRecordedException"); registerAttributes(); timerID = id;}

    TimerRecordedException* copy() const { return new TimerRecordedException(*this);}

    const unsigned int& getTimerID() {return timerID.getRValue();}
  private:
    void registerAttributes();
  };
}//end namespace NMM

#endif
