/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainers:       NMM work group 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_ICECASTNODE_HPP
#define NMM_ICECASTNODE_HPP

#include "nmm/base/GenericSinkNode.hpp"
//#include "nmm/base/EventDispatcher.hpp"
#include "nmm/interfaces/audio/icecast/IIcecastImpl.hpp"
#include "nmm/interfaces/file/ITrackImpl.hpp"

#include <shout/shout.h>

namespace NMM {

  /**
   * IcecastNode
   */
  class IcecastNode :
    public GenericSinkNode,
    public IIcecastImpl,
    public ITrackImpl
  {
  private:
    string title;                     // title send as metadata
    bool send_title;                  // if <-this flag is set
    
    bool ignore_incoming;             // if set, the node will ignore all incoming data (id3 tags for example)
    
  protected:
    shout_t   *shout_connection; // connection data structure
    shout_metadata_t *shout_metadata;
    
    virtual Result doInit();
    virtual Result doActivate();
    virtual Result doDeactivate();
    virtual Buffer *processBuffer(Buffer *buffer);
    
    // ITrack interface
    virtual Result startTrack(const string& track);
    virtual Result endTrack();
    virtual Result finished();
    
  public:
    IcecastNode(const char *name = "IcecastNode", 
		const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND, 
		const int queue_size = 1);
    ~IcecastNode();
    
    // IIcecast interface
    virtual Result setServerIP(const string& value);
    virtual Result setServerPort(const int& value);
    virtual Result setServerPassword(const string& value);
    virtual Result setUser(const string& value);
    virtual Result setMountpoint(const string& value);
    virtual Result setTitle(const string& value);
    virtual Result setStreamName(const string& value);
    virtual Result setGenre(const string& value);
    virtual Result setDescription(const string& value);
    virtual Result setURL(const string& value);
    virtual Result setPublic(const int& value);
    virtual Result setSendTitle(const bool& value);
    
  private:
    void sendMetadata(const string& name, const string& value);
  };
  
}

#endif
