/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

#ifndef NMM_INT2LADSPADATANODE_HPP
#define NMM_INT2LADSPADATANODE_HPP NMM_INT2LADSPADATANODE_HPP

#include "nmm/base/GenericProcessorNode.hpp"

namespace NMM {

    /**
     *This node converts an signed short based NMM-buffer into a LADSPA_Data based buffer.
     *LADSPA_Data is a floating point format (for exact type look at ladspa.h).
     *Additionally, this node sorts the sample by channel and produces a kind of 'planar' 
     *audio format in the way a LADSPA-plugin awaits the audio samples
     *
     *The Int2LADSPADataNode can only be used int the context of the LADSPANode
     */
    class Int2LADSPADataNode : public GenericProcessorNode {
    private:
	int channels; /* number of channels in the input stream */

    protected:
	// INode-Interface
	virtual Result doInit(void);
	virtual Result doDeinit(void);
	virtual Result doInitOutput(void);
	virtual Result doDeinitOutput(void);
	virtual Result doActivate(void);
	virtual Result doDeactivate(void);
	virtual Message* processBuffer(Buffer* in_buffer);

    public:
	Int2LADSPADataNode(const char* = "Int2LADSPADataNode",
			   const StreamQueue::Mode qmode= StreamQueue::DEFAULT_MODE,
			   const int qsize= StreamQueue::DEFAULT_SIZE);
	~Int2LADSPADataNode(void);

    }; // class Int2LADSPADataNode

}; // namespace NMM

#endif // NMM_INT2LADSPADATANODE_HPP
