/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "ImageNode.hpp"
#include "nmm/base/format/Property.hpp"
#include "nmm/base/registry/Plugin.hpp"
#include <png.h>

#define WAIT_SECS 0
#define WAIT_USECS 200000
#define WAIT_MILLIS 100

NMM_REGISTER_PLUGIN(NMM::ImageNode, "ImageNode")
 
namespace NMM {

  ImageNode::ImageNode(const char* name /* = "Node" */,
		   StreamQueue::Mode mode /* = StreamQueue::MODE_SUSPEND */,
		   int queue_size /* = StreamQueue::DEFAULT_SIZE */)
    : GenericSourceNode(name, mode, queue_size) {

    image_filename = 0;
    timer = createLoopTimer();

//     NMM::Format *myOutputFormat;
//     myOutputFormat=getOwnOutputProperty()->addNewFormat("video/raw");
//     myOutputFormat->addWildcard(Format::x_resolution_param);
//     myOutputFormat->addWildcard(Format::y_resolution_param);
//     myOutputFormat->addStringParamValue(Format::colorspace_param,"rgb");
//     myOutputFormat->addStringParamValue(Format::channelorder_param,"bgra");
//     myOutputFormat->addIntParamValue(Format::bitperpixel_param,32);
//     myOutputFormat->addIntParamValue(Format::bitperpixel_param,24);
//     myOutputFormat->addStringParamValue(Format::endian_param, "little");
//     myOutputFormat->addRFloatParamValue(Format::framerate_param, 0.0, 100.0);

//     Format *myOutputFormat2 = new Format();
//     myOutputFormat2=getOwnOutputProperty()->addNewFormat("video/raw");
//     myOutputFormat2->addWildcard(Format::x_resolution_param);
//     myOutputFormat2->addWildcard(Format::y_resolution_param);
//     myOutputFormat2->addStringParamValue(Format::colorspace_param,"yuv422");
//     myOutputFormat2->addStringParamValue(Format::channelorder_param,"uyvy");
//     myOutputFormat2->addIntParamValue(Format::bitperpixel_param,16);
//     myOutputFormat2->addStringParamValue(Format::endian_param, "little");
//     myOutputFormat2->addStringParamValue(Format::format_param, "packet");
//     myOutputFormat2->addRFloatParamValue(Format::framerate_param, 0.0, 100.0);
    registerEvent("image_path", new TEDObject1<ImageNode, string>(this, &ImageNode::setImage));
  }

  ImageNode::~ImageNode() {
    // not yet finished...
    destroyLoopTimer(timer);
  }

  Result ImageNode::doInit() {
    Result res;
    res = preparePNGImage(image_filename);
    if (res == SUCCESS) {
      NMM::Format *myOutputFormat;
      myOutputFormat=getOwnOutputProperty()->addNewFormat("video/raw");
      myOutputFormat->addIntParamValue(Format::x_resolution_param, image_width);
      myOutputFormat->addIntParamValue(Format::y_resolution_param, image_height);
      myOutputFormat->addStringParamValue(Format::colorspace_param,"rgb");
      myOutputFormat->addStringParamValue(Format::channelorder_param,"rgba");
      myOutputFormat->addIntParamValue(Format::bitperpixel_param,32);
      myOutputFormat->addStringParamValue(Format::endian_param, "little");
      myOutputFormat->addRFloatParamValue(Format::framerate_param, 0.0, 100.0);
      DEBUG_STREAM("Image loaded ok!" << endl);
      return SUCCESS;
    } else {
      DEBUG_STREAM("Image load failed!" << endl);
      return FAILURE;
    }
  }

  Result ImageNode::doActivate() {
    image_fps = getOutputFormat()->getFloatValue(Format::framerate_param);
    return SUCCESS;
  }

  Buffer *ImageNode::processBuffer(Buffer *) {
    Buffer *out;

    startLoopTimer(timer);

    if ((out = getNewBuffer(image_width*image_height*4))!=0) {
      paintPNGImage(out->getData());
      out->setUsed(image_width*image_height*4);
    } else {
      ERROR_STREAM( " Could not get out buffer." << endl);
    }

    // wait for next frame tick
    delayLoopTimer(timer, (long)(1000.0 * (1.0/image_fps)));
    return out;
  }


  Result ImageNode::setImage(string& value) {
    const char *str = value.c_str();
    if (image_filename != 0) delete [] image_filename;
    image_filename = new char[strlen(str)+1];
    strcpy(image_filename, str);
    return SUCCESS;
  }



  /* ==== Internal methods follow ==== */




  void ImageNode::setPixel(char *image, int x, int y,
			  unsigned char r, unsigned char g, unsigned char b) {
    unsigned char *ptr;
    ptr = (unsigned char *)image;
    ptr += 4 * (image_width * y + x);
    ptr[0] = b;
    ptr[1] = g;
    ptr[2] = r;
  }

  Result ImageNode::preparePNGImage(char *filename) {

    // SEE THE LIBPNG-EXAMPLE.c FOR DETAILS!

    png_bytep *png_rows;               
    png_uint_32 png_width, png_height;  
    png_structp png_ptr;
    png_infop info_ptr;
    //unsigned int sig_read = 0;
    //png_uint_32 width, height;
    int bit_depth, color_type, interlace_type;
    FILE *fp;
    
    png_rows = NULL;
    
    if ((fp = fopen(filename, "rb")) == NULL) return FAILURE;
    
    png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
				     NULL, NULL, NULL);
    
    if (png_ptr == NULL) {
      fclose(fp);
      return FAILURE;
    }
    
    info_ptr = png_create_info_struct(png_ptr);
    if (info_ptr == NULL) {
      fclose(fp);
      png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
      return FAILURE;
    }

    if (setjmp(png_ptr->jmpbuf)) {
      png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
      fclose(fp);
      return FAILURE;
    }

    png_init_io(png_ptr, fp);

    png_read_info(png_ptr, info_ptr);
    
    png_get_IHDR(png_ptr, info_ptr, &png_width, &png_height,
		 &bit_depth, &color_type,
		 &interlace_type, NULL, NULL);
    
    png_rows = new png_bytep[png_height];

    for (unsigned int row = 0; row < png_height; row++) {
      png_rows[row] = new png_byte[png_get_rowbytes(png_ptr, info_ptr)];
    }
    
    png_read_image(png_ptr, png_rows);
    
    png_read_end(png_ptr, info_ptr);
    
    png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
    
    fclose(fp);

    image_width = (int) png_width;
    image_height = (int) png_height;
    image_data = new unsigned char[image_width * image_height * 4];

    for (int y = 0; y < image_height; y++) {
      unsigned char *pngimg;
      unsigned char *img; 
      pngimg = (unsigned char*) png_rows[y];
      img = (unsigned char*)image_data + y * image_width * 4;
      memcpy(img, pngimg, image_width*4);
      delete [] png_rows[y];
    }
    delete [] png_rows;
        
    return SUCCESS;
  }

  void ImageNode::paintPNGImage(char *image){
      memcpy(image, image_data, image_width * image_height * 4);
  }

}
