/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_PNGWRITENODE_HPP_
#define _NMM_PNGWRITENODE_HPP_ _NMM_PNGWRITENODE_HPP_

#include "nmm/base/GenericSinkNode.hpp"
#include "nmm/interfaces/file/ISinkFileHandlerImpl.hpp"
#include "nmm/interfaces/file/IFileModeHandlerImpl.hpp"
#include "nmm/interfaces/file/IPNGWriteImpl.hpp"
#include "nmm/misc/png/PNGUtils.hpp"

namespace NMM {

  class PNGWriteNode : public GenericSinkNode,
		       public ISinkFileHandlerImpl,
		       public IFileModeHandlerImpl,
		       public IPNGWriteImpl{

  private:
    PNGUtils png_utils;
    unsigned int image_width;  ///< width of the image in the input stream
    unsigned int image_height; ///< height of the image in the input stream
    unsigned int buf_count;
    string filename;
    string workdir;
    string fileext;
    const string deffileext; ///< Default file extension
    IFileModeHandler::FilenameMode filename_mode;
    Timestamp last_buf_timestamp;
    string last_filename;
    bool discardImages;
    
  protected:
    Result doInit();
    Result doActivate();
    Message* processBuffer(Buffer *);
    
    // IPNGWriteNode interface
    virtual void setDiscardImages(const bool&);

    // IFileHandler interface
    virtual void setFilename(const string&);

    // ISinkFileHandler interface
    virtual void setWorkingDirectory(const string& dir);
    virtual string getWorkingDirectory();
    virtual void setFileExtension(const string& ext);
    virtual string getFileExtension();
    
    // IFileModeHandler interface
    virtual void setMode(const IFileModeHandler::FileMode& mode);
    virtual IFileModeHandler::FileMode getMode();
    virtual void setFormat(const IFileModeHandler::FileFormat& format);
    virtual IFileModeHandler::FileFormat getFormat();
    virtual void setFilenameMode(const IFileModeHandler::FilenameMode& mode);
    virtual IFileModeHandler::FilenameMode getFilenameMode();
    virtual void setLoop(const bool& d);
    
  public:

    PNGWriteNode(const char *name = "PNGWriteNode",
		StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		int = StreamQueue::DEFAULT_SIZE);
    ~PNGWriteNode();

  };

}

#endif //_NMM_PNGWRITENODE_HPP_
