/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_WAVWRITENODE_HPP_
#define _NMM_WAVWRITENODE_HPP_ _NMM_WAVWRITENODE_HPP_


#include "nmm/base/GenericSinkNode.hpp"
#include "nmm/interfaces/file/ISinkFileHandlerImpl.hpp"
#include "nmm/interfaces/file/ITrackImpl.hpp"

#include <fstream>

namespace NMM {

  class WavWriteNode :
    public GenericSinkNode,
    public ISinkFileHandlerImpl,
    public ITrackImpl
  {
  private:
    enum localstates { START, DATA };
    localstates state;
    
    ofstream out_fstream; /* File object */
    long int bytes_receive;
    string filename;
    bool ready;
    
    string file_extension;
    string working_directory;
    bool override_default_filename;
    
    //! misc functions 
    long int getBuffers();
    bool isReady();
    
  protected:
    virtual Result doInit();
    virtual Result doActivate();
    virtual Result doStop();
    
    virtual Message* processBuffer(Buffer *in_buffer);
    
    // ITrack interface
    virtual Result startTrack(const string& filename);
    virtual Result endTrack();
    virtual Result finished();
    
  public:
    WavWriteNode(const char* name = "WavWriteNode" , 
                 const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND,
                 const int size = StreamQueue::DEFAULT_SIZE);
    virtual ~WavWriteNode();
    
    // ISinkFileHandler interface
    virtual void setWorkingDirectory(const string& directory);
    virtual string getWorkingDirectory();
    virtual void setFileExtension(const string& extension);
    virtual string getFileExtension();
    
    // IFileHandler interface
    virtual void setFilename(const string& filename);
  };
}

#endif /* _NMM_WAVWRITENODE_HPP_ */
