/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#include "Camera1394.hpp"
#include "nmm/utils/utils.hpp"
//#include <stream.h>
#include <iostream>
#include <l1394/l1394_session.h>
#include "nmm/base/EventDispatcher.hpp"
#include "nmm/interfaces/video/camera/ICamera.hpp"


namespace NMM{

  Camera1394::Camera1394(const char* name)
  : GenericFireWireNode(name), camera(0)
  {

  }


  Result Camera1394::doActivate() {
    if (GenericFireWireNode::doActivate() == FAILURE)
      return FAILURE;
    return SUCCESS;
  }

  Result Camera1394::doStart()  {
    if (GenericFireWireNode::doStart() == FAILURE)
      return FAILURE;
    return SUCCESS;
  }
  Result Camera1394::doStop() {
    if (GenericFireWireNode::doStop() == FAILURE)
      return FAILURE;
    return SUCCESS;
  }


  Result Camera1394::doInit() {
    if (GenericFireWireNode::doInit() == FAILURE)
      return FAILURE;
    camera = dynamic_cast<L1394::Camera*>(getInternalDevice());
    if (camera) {
//      camera->whiteBalance(1)->automatic(true);
//      automatic = true;
      return SUCCESS;
    }
    return FAILURE;
  }

  void Camera1394::setBrightness(const int& value){
    camera->brightness()->setValue(value);
  }

  void Camera1394::setAutoExposure(const int& value){
    camera->autoExposure()->setValue(value);
  }
  void Camera1394::setSharpness(const int& value){
    camera->sharpness()->setValue(value) ;
  }
  void Camera1394::setWhiteBalance(const int& value){
    camera->whiteBalance()->setValue(value) ;
  }

  void Camera1394::setBlueBalance(const int& value) {
    camera->whiteBalance(RED)->setValue(value);
  }
  void Camera1394::setRedBalance(const int& value) {
    camera->whiteBalance(BLUE)->setValue(value);
  }

  void Camera1394::setHue(const int& value) {
    camera->hue()->setValue(value) ;
  }
  void Camera1394::setSaturation(const int& value){
    camera->saturation()->setValue(value) ;
  }
  void Camera1394::setGamma(const int& value)    {
    camera->gamma()->setValue(value) ;
  }
  void Camera1394::setShutter(const int& value){
    camera->shutter()->setValue(value) ;
  }
  void Camera1394::setGain(const int& value)    {
    camera->gain()->setValue(value) ;
  }
  void Camera1394::setIris(const int& value)    {
    camera->iris()->setValue(value) ;
  }  ;
  void Camera1394::setFocus(const int& value)    {
    camera->focus()->setValue(value) ;
  };
  void Camera1394::setZoom(const int& value)     {
    camera->zoom()->setValue(value);
  };
  void Camera1394::onePush()              {
    camera->whiteBalance()->onePush();
  };



  int Camera1394::getBrightness()        const{
    return camera->brightness()->getValue();
  };
  int Camera1394::getAutoExposure()      const{
    return camera->autoExposure()->getValue();
  };
  int Camera1394::getSharpness()         const{
    return camera->sharpness()->getValue();
  };
  int Camera1394::getWhiteBalance()      const{
    return camera->whiteBalance()->getValue();
  };
  int Camera1394::getBlueBalance()       const {
    return camera->whiteBalance(BLUE)->getValue();
  }
  int Camera1394::getRedBalance()        const {
    return camera->whiteBalance(RED)->getValue();
  }
  int Camera1394::getHue()               const{
    return camera->hue()->getValue();
  };
  int Camera1394::getSaturation()        const{
    return camera->saturation()->getValue();
  };
  int Camera1394::getGamma()             const{
    return camera->gamma()->getValue();
  };
  int Camera1394::getShutter()           const{
    return camera->shutter()->getValue();
  };
  int Camera1394::getGain()              const{
    return camera->gain()->getValue();
  };
  int Camera1394::getIris()              const{
    return camera->iris()->getValue();
  };
  int Camera1394::getFocus()             const{
    return camera->focus()->getValue();
  };
  int Camera1394::getZoom()              const{
    return camera->zoom()->getValue();
  };


  pair<int, int> Camera1394::getBrightnessRange()    const{
    return pair<int, int>(camera->brightness()->getMinValue(),camera->brightness()->getMaxValue());
  };
  pair<int, int> Camera1394::getAutoExposureRange()  const{
    return pair<int, int>(camera->autoExposure()->getMinValue(),camera->autoExposure()->getMaxValue());
  }
  pair<int, int> Camera1394::getSharpnessRange()     const{
    return pair<int, int>(camera->sharpness()->getMinValue(),camera->sharpness()->getMaxValue());
  };
  pair<int, int> Camera1394::getWhiteBalanceRange()  const{
    return pair<int, int>(camera->whiteBalance()->getMinValue(),camera->whiteBalance()->getMaxValue());
  }
  pair<int, int> Camera1394::getHueRange()           const{
    return pair<int, int>(camera->hue()->getMinValue(),camera->hue()->getMaxValue());
  };
  pair<int, int> Camera1394::getSaturationRange()    const{
    return pair<int, int>(camera->saturation()->getMinValue(),camera->saturation()->getMaxValue());
  }
  pair<int, int> Camera1394::getGammaRange()         const{
    return pair<int, int>(camera->gamma()->getMinValue(),camera->gamma()->getMaxValue());
  };
  pair<int, int> Camera1394::getShutterRange()       const{
    return pair<int, int>(camera->shutter()->getMinValue(),camera->shutter()->getMaxValue());
  };
  pair<int, int> Camera1394::getGainRange()          const{
    return pair<int, int>(camera->gain()->getMinValue(),camera->gain()->getMaxValue());
  }
  pair<int, int> Camera1394::getIrisRange()          const{
    return pair<int, int>(camera->iris()->getMinValue(),camera->iris()->getMaxValue());
  };
  pair<int, int> Camera1394::getFocusRange()         const{
    return pair<int, int>(camera->focus()->getMinValue(),camera->focus()->getMaxValue());
  };
  pair<int, int> Camera1394::getZoomRange()          const{
    return pair<int, int>(camera->zoom()->getMinValue(),camera->zoom()->getMaxValue());
  };


  bool Camera1394::hasBrightness()     const{
    cout<< camera->brightness()->hasFeature()<<endl;
    return camera->brightness()->hasFeature();
  };
  bool Camera1394::hasAutoExposure()   const{
    cout<< camera->autoExposure()->hasFeature()<<endl;
    return camera->autoExposure()->hasFeature();
  };
  bool Camera1394::hasSharpness()      const{
    cout << camera->sharpness()->hasFeature() << endl;
    return camera->sharpness()->hasFeature();
  };
  bool Camera1394::hasWhiteBalance()   const{
    cout << camera->whiteBalance()->hasFeature() << endl;
    return camera->whiteBalance()->hasFeature();
  };
  bool Camera1394::hasHue()            const{
    return camera->hue()->hasFeature();
  };
  bool Camera1394::hasSaturation()     const{
    return camera->saturation()->hasFeature();
  };
  bool Camera1394::hasGamma()          const{
    cout << camera->gamma()->hasFeature() << endl;
    return camera->gamma()->hasFeature();
  };
  bool Camera1394::hasShutter()        const{
    cout << camera->shutter()->hasFeature() << endl;
    return camera->shutter()->hasFeature();
  };
  bool Camera1394::hasGain()           const{
    cout << camera->gain()->hasFeature() << endl;
    return camera->gain()->hasFeature();
  };
  bool Camera1394::hasIris()           const{
    cout << camera->iris()->hasFeature() << endl;
    return camera->iris()->hasFeature();
  };
  bool Camera1394::hasFocus()          const{
    cout << camera->focus()->hasFeature() << endl;
    return camera->focus()->hasFeature();
  };
  bool Camera1394::hasZoom()           const{
    cout << camera->zoom()->hasFeature() << endl;
    return camera->zoom()->hasFeature();
  };

    


} //end namespace

