/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christian Gerstner <gerstner@graphics.cs.uni-sb.de>
 *                    Eric Peters <epeters@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef NMM_VNCSOURCENODE_HPP
#define NMM_VNCSOURCENODE_HPP NMM_VNCSOURCENODE_HPP

#include "nmm/base/sync/GenericSyncSourceNode.hpp"

#include "nmm/interfaces/net/INetClientImpl.hpp"
#include "nmm/interfaces/net/IVNCImpl.hpp"

#include "RFBProtoController.hpp"


namespace NMM {
  class StreamTimer;

  class VNCSourceNode : public GenericSyncSourceNode,
			public INetClientImpl,
			public IVNCImpl {

  public: 
    
    /** Constructor
     */
    VNCSourceNode(const char* = "VNCSourceNode");
    
    /** Destructor
     */
    ~VNCSourceNode();
    
    void setArgsAndResources(int argc, char **argv);
    

  protected:
    
    virtual Message* produceBuffer(Buffer*);
    Result doInit();
    Result doInitOutput();
    virtual Result doDeactivate();

    // virtual functions inherited from GenericSyncSourceNode
    virtual Time getLocalTime(u_int64_t);

    // INetClientImpl
    virtual void setHostname(const string& value);
    virtual void setPort(const int& port);
    virtual void setProtocol(const string& protocol);
    virtual string getHostname() const;
    virtual int getPort() const;
    virtual string getProtocol() const;
    virtual void connectionLost();

    // IVNC
    virtual void setTolerance(const float& tolerance);
    virtual void setInterval(const float& interval);
    virtual void setShared(const bool& unshared);
    virtual float getTolerance() const;
    virtual float getInterval() const;
    virtual bool isShared() const;
    virtual void setPasswd(const string& pw);
    virtual void setPasswdPath(const string& passwd);
    virtual void sendKeyEvent(const unsigned long& key, const int& down);
    virtual void sendPointerEvent(const int& x, const int& y, const int& buttonmask);
    virtual void sendFrameBufferUpdateRequest();
    virtual int  getServerHeight() const;
    virtual int  getServerWidth() const;
    virtual void setMode(const IVNC::Mode& mode);
	
  private:
	
    StreamTimer*             timer;
    RFBProtoController*      proto;
      
    IVNC::Mode               mode;
    long                     buf_count;
    int                      buffer_size;
    int                      width;
    int                      height;
    int                      colordepth;
    int                      bits_per_pixel;
    string                   hostname;
    int                      port;
    bool                     mode_changed;
    
  };
}//end namespace
#endif
