/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_IDNODE_HPP
#define NMM_IDNODE_HPP

#include "nmm/base/GenericFilterNode.hpp"

#include "nmm/interfaces/stats/IBufferCounterImpl.hpp"
#include "nmm/interfaces/stats/ISleepImpl.hpp"
#include "nmm/interfaces/stats/IBuffersPassedEventImpl.hpp"

namespace NMM {
  
  /**
   * IdNode is a Node which performs the identity function, i.e.
   * passes on any buffers unaltered. Its only use is for
   * statistical purposes.
   */
  class IdNode
    : public GenericFilterNode
    , public IBufferCounterImpl
    , public ISleepImpl
    , public IBuffersPassedEventReceiverImpl
  {
  private:
    unsigned int sleep_time;
    long buffers_passed;
    long upstream_passed;
    
  protected:
    virtual void setSleepTime(const int& value);
    virtual int getSleepTime() const;
    
    virtual int getBuffersPassed() const;
    virtual void resetBufferCounter();
    
    virtual Result buffersPassed(const int& value);
    virtual int getUpstreamBuffersPassed() const;
    virtual void resetUpstreamBufferCounter();
    
    virtual Result doInit();
    virtual Result doInitOutput();
    
    virtual Message *processBuffer(Buffer *in_buffer);
    
  public:
    IdNode(const char* name = "IdNode",
           const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
           const int size = StreamQueue::DEFAULT_SIZE);
    virtual ~IdNode() {};
  };
}

#endif //NMM_IDNODE_HPP
