/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christian Gerstner <gerstner@graphics.cs.uni-sb.de>
 *                    Eric Peters <epeters@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_SVGPaintNode_HPP
#define NMM_SVGPaintNode_HPP NMM_SVGPaintNode_HPP

#include "nmm/base/GenericMultiplexerNode.hpp"
#include "nmm/plugins/text/svg/SVGEncodeNode.hpp"
#include "nmm/interfaces/text/ISVGPaintImpl.hpp"
#include "gd.h"
#include <vector>

namespace NMM{

  class SVGPaintNode : public GenericMultiplexerNode,
		       public ISVGPaintImpl
  {
    
  public:
	
    /** Constructor
     */
    SVGPaintNode(const char* = "SVGPaintNode", const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE, const int queue_size = 1);
	
    /** Destructor
     */
    ~SVGPaintNode();
	
    //Interface ISVGPaint
    void setImageWidth(const int&);
    void setImageHeight(const int&);
    void setShowSVGAnnotation(const bool&);   

  protected:
    virtual Message* processBuffer(Buffer*);
    Result doInit();
    Result doInitOutput();
    virtual Result doDeactivate();
      
      
  private:
	
    Buffer* working_background;
    Buffer* unchanged_background;
    Buffer* last_annotationbuffer;
    int buffer_size;
    int bits_per_pixel;
	
    gdImagePtr im;
    int imagewidth;
    int imageheight;
    bool has_background;
    bool has_annotationbuffer;
    bool showSVGAnnotation;
    void initgdImage(gdImagePtr,int*);
    void processSVGString(Buffer*);
    int parseStroke(const string&,const int&) const;
    int parseColor(const string&,const int&) const;
    void drawLine(const int&,const int&,const int&,const int&,const int&);
    void drawRectangle(const int&, const int&, const int&, const int&,const int&);
    void drawBezier(const vector<Point>&,const int&);
    void drawCurve(const Point&, const Point&, const Point&, const Point&, const int&);
  };
    
}
#endif
