/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_WHITENOISE_HPP_
#define _NMM_WHITENOISE_HPP_

#include "nmm/base/GenericSourceNode.hpp"
#include "nmm/base/sync/StreamTimer.hpp"
#include "nmm/utils/utils.hpp"

namespace NMM {

  class WhiteNoiseNode : public GenericSourceNode {
  private:
    StreamTimer* stimer;
    
    Buffer* produceRGBBuffer();
    Buffer* produceYUVBuffer();
    Buffer* produceYV12Buffer();
    
    int image_width;
    int image_height;
    int image_bpp;
    double image_fps;
    
    void addSupportedFormats(Property* property);

    looptimer       __last_timer;     ///< stores the latest timespamp
    ThreadMutex     __cond_mutex;     ///< ThreadMutex, required for the ThreadCondition
    ThreadCondition __condition;      ///< used to wait until next buffer must be send
  protected:
    virtual Message* processBuffer(Buffer*);
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    virtual Result doStop();
    void waitToSendBuffer();
    
  public:
    WhiteNoiseNode(const char* name = "WhiteNoiseNode",
                   StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
                   int = StreamQueue::DEFAULT_SIZE);
    
    virtual ~WhiteNoiseNode();
  };
}

#endif //_NMM_WHITENOISE_HPP_

