/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "VISCACameraReadNode.hpp"
#include "nmm/utils/SecurityCheck.hpp"
#include "nmm/base/registry/DevicePlugin.hpp"

#include <sstream>

static NMM::TDevicePlugin<NMM::VISCACameraReadNode> plugin("/dev/video", false, "VISCACameraReadNode");
NMM_REGISTER_SPECIAL_PLUGIN(plugin)

namespace NMM {
 /*
  * todo: some functions are call by value others call by reference 
  */

  VISCACameraReadNode::VISCACameraReadNode(const char *name /* = "VISCACameraReadNode" */,
				 StreamQueue::Mode mode /* = StreamQueue::MODE_SUSPEND */,
				 int queue_size /* = StreamQueue::DEFAULT_SIZE */) : IVTVReadNode(name, mode, queue_size) {
    serial_device = "/dev/ttyS1";
    old_termios   = NULL;
    new_termios   = NULL;
    spd           = 0;

    /* clear the response arrays */
    for(unsigned int i=0; i<MAX_VISCA_COMMAND_LENGTH; i++) {
      visca_response1.command[i] = 0x00;
      visca_response2.command[i] = 0x00;
    }
    visca_response1.length = 0;
    visca_response2.length = 0;
    
    
    /* clear the slot array */ 
    for(unsigned int i=0; i<MAX_VISCA_DEVICES; i++) {
      slots[i].locked = false;
      for(unsigned int j=0; j<MAX_SLOTS_PER_CAMERA; j++) {
        slots[i].slots[j].active = false;
      }
    } 

  } /* constructor */


  VISCACameraReadNode::~VISCACameraReadNode() {
    if (NULL != old_termios) { 
      delete old_termios; 
      old_termios = NULL;
    }
    if (NULL != new_termios) { 
      delete new_termios; 
      new_termios = NULL;
    }
  } /* destructor */
 

  Result VISCACameraReadNode::setDevice(string& _video_device, string& _serial_device) {
    IVTVReadNode::setDevice(_video_device);
    serial_device = _serial_device;
    return SUCCESS;
  } /* setDevice */


  Result VISCACameraReadNode::getDevice(string& _video_device, string& _serial_device) {
    _video_device = IVTVReadNode::getDevice();
    _serial_device = serial_device;
    return SUCCESS;
  } /* getDevice */
 

  Result VISCACameraReadNode::doInit() {
    /* call parent's doInit() */
    IVTVReadNode::doInit();
   
    
    DEBUG_STREAM("Open device " << serial_device << endl);
    if (FAILURE == init_serialdevice()) {
      return FAILURE;
    }
   
    number_of_cameras = 0;
    unsigned char bc = 8;
    DEBUG_STREAM("Init camera(s) " << endl);
    if ((FAILURE == IF_Clear(bc)) || ( FAILURE == AddressSet())) return FAILURE;
    DEBUG_STREAM("  " << number_of_cameras << " Camera found" << endl);

    number_of_tries = 3;


    return SUCCESS;
  } /* doInit */


  Result VISCACameraReadNode::doDeinit() {
    IVTVReadNode::doDeinit();

    if (spd >= 0) {
      ::close(spd);
      spd = -1;
    }

    return SUCCESS;
  }
  //////////////////////////////////////////////////////////////////////////////
  ///////////////////// TELL WULFMEN TO FIX THIS ///////////////////////////////
  //////////////////////////////////////////////////////////////////////////////
//   void VISCACameraReadNode::verbose(string what, struct_visca_command command) {
//     //   stringstream o();
//     //o << what;
//     //for (int i=0;i<command.length;i++)
//     //  o << "[" << hex << setw(2) << setfill('0') << int(command.command[i]) << "]";
//     //o << endl;
//     DEBUG_STREAM("Fix me: verbose call with (" << what << "," << command.length << ")" << endl);
//   } /* verbose */
  void VISCACameraReadNode::verbose(const string&, const string&) {
  }


  bool VISCACameraReadNode::islocked(const unsigned char& camera_id) {
    if (slots[camera_id].locked) {
      read_from_camera(&visca_response1);
      if (0 == check_response(&visca_response1,false)) {
        process_response(&visca_response1);
      }
    }
    return slots[camera_id].locked;
  } /* islocked */
  
  
  bool VISCACameraReadNode::compare_commands( struct_visca_command *cmd1, struct_visca_command *cmd2 ) {
    int i;
    bool rv = false;
    
    if (cmd1->length == cmd2->length) {
      i=0;
      while ( (cmd1->command[i] == cmd2->command[i]) && (i < cmd1->length) ) {
        i++;
      }
      
      if (i == cmd1->length) {
        rv = true;
      }
      
    }
    return rv;
  } /* compare_commands */
  
  
  Result VISCACameraReadNode::init_serialdevice() {
    int rc;

    /* allocate memory for serial device parameter structure */
    if (NULL == old_termios) { old_termios = new struct termios; }
    if (NULL == new_termios) { new_termios = new struct termios; }
    
    /* Open modem device for reading and writing and not as controlling tty
       because we don't want to get killed if linenoise send CTRL-C
    */
    if(!SSecurityCheck::getInstance().checkFilenameForReading(serial_device)) {
      ERROR_STREAM("Security check not passed for file " << serial_device << ", check .nmmrc for options" << endl);
      return FAILURE;
    }
    spd = open(serial_device.c_str(), O_RDWR | O_NOCTTY);

    if (0 < spd) {

      /* get and save current parameter from serial device */ 
      rc = tcgetattr(spd, old_termios);
      if (-1 == rc ) {
        ERROR_STREAM("Error getting(1) attr,tcgetattr error \n" << endl);
        return FAILURE;
      }

      /* get current parameter from serial device to modify it*/ 
      rc = tcgetattr(spd, new_termios);
      if (-1 == rc ) {
        ERROR_STREAM("Error getting(2) attr,tcgetattr error \n" << endl);
        return FAILURE;
      } else {

        /* setting port parameters */
        cfsetispeed(new_termios,B9600);      /* 9600 Bds   */
        new_termios->c_cflag &= ~PARENB;     /* No parity  */
        new_termios->c_cflag &= ~CSTOPB;     /*            */
        new_termios->c_cflag &= ~CSIZE;      /* 8bit       */
        new_termios->c_cflag |= CS8;         /*            */
        new_termios->c_cflag &= ~CRTSCTS;    /* No hdw ctl */
 
        /* local flags */
        new_termios->c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); /* raw input */
 
        /* input flags */
        new_termios->c_iflag &= ~(INPCK | ISTRIP); /* no parity */
        new_termios->c_iflag &= ~(IXON | IXOFF | IXANY); /* no soft ctl */
        new_termios->c_cc[VMIN] = 0;
        new_termios->c_cc[VTIME] = 10;  /* equals 1 second */
 
        /* output flags */
        new_termios->c_oflag &= ~OPOST; /* raw output */

        /* enable new parameter */
        rc = tcflush(spd, TCIFLUSH);
        rc = tcsetattr(spd, TCSANOW, new_termios);
        if (-1 == rc) {
          ERROR_STREAM("Error setting attr, tcsetattr error\n" << endl);
          return FAILURE;
        }
      }
    } else {
      ERROR_STREAM("Can't get serial port [" << serial_device << "] !" << endl);
    } /* if(0 < spd) */

    /* free memory */
    if (NULL != new_termios) { 
      delete new_termios; 
      new_termios = NULL;
    }

    return SUCCESS;
  } /* VISCACameraReadNode::init_serial_device */
  

  Result VISCACameraReadNode::restore_serialdevice() {
    int rc;

    /* restore to old parameters */
    rc = tcflush(spd, TCIFLUSH);
    rc = tcsetattr(spd,TCSANOW, new_termios);
    if (-1 == rc) {
      ERROR_STREAM("Error setting attr, tcsetattr error\n");
      return FAILURE;
    }

    if (NULL != old_termios) { 
      delete old_termios; 
      old_termios = NULL;
    }
    if (NULL != new_termios) {
      delete new_termios; 
      new_termios = NULL;
    }

    return SUCCESS;
  } /*VISCACameraReadNode::restore_serialdevice */



  Result VISCACameraReadNode::send_to_camera(struct_visca_command *cmdsend){
    int     rc;
    Result retval = SUCCESS;
 
//     verbose( "Send : ", *cmdsend );
  
    rc = write(spd, cmdsend->command, cmdsend->length);   /* Send Visca message */
 
    if (-1 == rc) {
      ERROR_STREAM("Error writing to serial port" << endl);
      retval = FAILURE;         

      /* Flush the link */
      rc = tcflush(spd, TCOFLUSH);
      if (-1 == rc ) {
        ERROR_STREAM("Error flushing link, tcsetattr error \n" << endl);
      }
    }
    return retval;

  } /* send_to_camera */


  Result VISCACameraReadNode::read_from_camera(struct_visca_command *cmdread){
    int rc  = 0;
    int pos = 0;
    bool working_flag = true;
    Result retval = SUCCESS;

    fd_set rfds;
    timeval tv;
    int retsel;

    FD_ZERO(&rfds);
    FD_SET(spd,&rfds);

    
    /* read complete VISCA command character
       by character up to 0xFF whitch should appear
       only at the end !!
    */
    while (working_flag){
      // wait up to 1 second
      tv.tv_sec = 1;
      tv.tv_usec = 0;

      // should avoid a deadlock if there are NO cameras connected
      retsel = select(spd+1,&rfds,NULL,NULL,&tv);

      if (retsel) {
        rc = read(spd, &cmdread->command[pos], 1);

        if (rc > 0) {
 
          if (0xff == cmdread->command[pos]) {
            working_flag = false;
          }
          pos++;
        
        } else if (rc < 0) {
          pos = -1;
          working_flag = false;
          ERROR_STREAM("Error reading from serial port" << endl);
          retval = FAILURE;
        }
      } else {
        working_flag = false;
        retval = FAILURE;
        ERROR_STREAM("timeout while reading from serial port" << endl);
      }
    }
    
    cmdread->length = pos;
    
//     verbose( "Read : " , *cmdread );
    
    return retval;
  } /* read_from_camera */
  
  
  Result VISCACameraReadNode::wait_for_ack(const unsigned char& camera_id,const unsigned char& slot_id){
    enum enum_status { WAIT, ERROR, ARRIVED };
    
    enum_status status = WAIT;
    Result retval = SUCCESS;
    
    /* read from serial port, until the ACK arrived
       all other packages are processed to free the slots
    */
    
    DEBUG_STREAM("Wait for ACK" << endl);
    while(WAIT == status){
      
      if (read_from_camera(&visca_response1)){
        
        if (0 == check_response(&visca_response1,true)){

          if ( (3 == visca_response1.length) && 
               (camera_id == ((visca_response1.command[0] & 0x70) >> 4)) &&
               (0x40 == (visca_response1.command[1] & 0xf0)) && 
               (0xff == (visca_response1.command[2]))
               ) {
            DEBUG_STREAM("ACK arrived" << endl);
            status = ARRIVED;
          } else{
            DEBUG_STREAM("Received command is NOT an ACK" << endl);
            process_response(&visca_response1);
          }
          
        } else {
          /* the camera send an errror */
          ERROR_STREAM("Error occured, maybe the camera is power-off" << endl);
          status = ERROR;
        }
      }
      
    }

    if (ARRIVED == status) {
      DEBUG_STREAM("Camera is " << int(camera_id) << ", put data in slot " << int(slot_id) << endl);

      /* put data in the free slot */
      slots[camera_id].slots[slot_id].active=true;
      slots[camera_id].slots[slot_id].socket=visca_response1.command[1] & 0x0f;

      slots[camera_id].slots[slot_id].expected_answer.command[0] = 0x80 | (camera_id << 4);
      slots[camera_id].slots[slot_id].expected_answer.command[1] = 0x50 | (visca_response1.command[1] & 0x0f);
      slots[camera_id].slots[slot_id].expected_answer.command[2] = 0xff;
      slots[camera_id].slots[slot_id].expected_answer.length     = 3;

      //    slot[camera_id].slots[slot_id].comparison_mask.command[0] = 0x80;
      //    slot[camera_id].slots[slot_id].comparison_mask.command[1] = 0x50;
      //    slot[camera_id].slots[slot_id].comparison_mask.command[2] = 0xff;
      //    slot[camera_id].slots[slot_id].comparison_mask.length     = 3;

      gettimeofday (&(slots[camera_id].slots[slot_id].expiredate), NULL);
      //      cout << "gettimeofday = " << slots[camera_id].slots[slot_id].expiredate.tv_sec << endl;
      slots[camera_id].slots[slot_id].expiredate.tv_sec +=30;
    }
    
    if (ERROR == status) {
      retval = FAILURE;
    }

    return retval;
  } /* wait_for_ack */


  Result VISCACameraReadNode::send_command_and_wait_for_ack(struct_visca_command *cmdsend, bool lockcam) {
    MutexGuard mg(mutex_command_send);

    int camera_id = 0x07 & cmdsend->command[0];
    int slot_id = -1;
    Result retval = SUCCESS;
        
    check_slots_for_timeout(camera_id);

    /* send a new command to camera camera_id only if
       the camera is not locked */
    if (!slots[camera_id].locked) {
      slot_id = find_free_slot(camera_id);
      
      if (-1 < slot_id) {
        DEBUG_STREAM("Send command to camera " << camera_id << endl);
        send_to_camera(&visca_command);
        wait_for_ack(camera_id, slot_id);
        
        if (lockcam) {
          slots[camera_id].locked     = true;
          slots[camera_id].lockedslot = slot_id;
        }
        
      } else {
        ERROR_STREAM("Can't find free slot for camera " << camera_id << endl);
        retval = FAILURE;
      }
    } else {
      ERROR_STREAM("camera " << camera_id << " is LOCKED!" << endl);
      retval = FAILURE;
    }

    return retval;
  } /* send_command_and_wait_for_ack */;


  Result VISCACameraReadNode::send_inquiry_command(const unsigned char& camera_id, const unsigned char& byte1, const unsigned char& byte2, struct_visca_command *cmd_out) {
    MutexGuard mg(mutex_command_send);

    DEBUG_STREAM("enter : send_inquiry_command" << endl);

    // #warning FIXME: must do some debugging  !!!!
    bool done = false;
    int  wtry=0;

    /* build command */
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x09;
    visca_command.command[2] = byte1;
    visca_command.command[3] = byte2;
    visca_command.command[4] = 0xff;
    visca_command.length = 5;
    
    while ((wtry < number_of_tries) && (!send_to_camera(&visca_command))) {
      wtry++;
    }
    
    if (wtry > number_of_tries) {
      ERROR_STREAM("Error: Send command for " << wtry  << " times to the camera, but no success." << endl);
      return FAILURE;
    }
    
    /* read from camera and check the response
       for errors and messages from other camera(s)
       until a message for the waiting camera is arrived
    */
    while (!done){
      if (!read_from_camera(cmd_out)){
        ERROR_STREAM("send_inquiry_command : could not read from camera" << endl);
        return FAILURE;
      }
      
      if ( camera_id == ((cmd_out->command[0] & 0x70) >> 4) ){
        /* response-camera is send-camera */
        DEBUG_STREAM("send_inquiry_command : catch something" << endl);
        if (3 == cmd_out->length) {
          if (!process_response(cmd_out)){
            // and here?
          } 
        } else {
          done = true;
        }
      } else {
        /* response-camera is NOT send-camera */
        if (0 != check_response(cmd_out,true)){
          // what's here?
        }
        
        if (!process_response(cmd_out)){
          // and here?
        }
      }
    }
    
    DEBUG_STREAM("leave : send_inquiry_command" << endl);    

    return SUCCESS;
    //  return(read_from_camera(cmd_out));

  } /* send_inquiry_command */
  
  
  void VISCACameraReadNode::compose_command6(const unsigned char& camera_id,const unsigned char& byte1,const unsigned char& byte2,const unsigned char& byte3,
                                        struct_visca_command *command) {
    command->command[0] = 0x80 | camera_id;
    command->command[1] = 0x01;
    command->command[2] = byte1;
    command->command[3] = byte2;
    command->command[4] = byte3;
    command->command[5] = 0xff;
    command->length = 6;
    
    return;
  } /* compose_command */
  
  
  int VISCACameraReadNode::check_response(struct_visca_command *response, bool display=false) {
    int retval  = 0;
    int socknum = 0;
    
    if (0x60 == (response->command[1] & 0xf0)){
      
      retval = response->command[2];
      
      if (display) {
        socknum = response->command[1] & 0x0f;
        

        
        switch (retval) {
        case 0x02 : 
          ERROR_STREAM("Camera " << ((response->command[0] & 0x70) >> 4) << " : "
                       << "Syntax Error" << endl 
                       << "  Returned when the command format is different "
                       << "  or when a command with illegal command parameters"
                       << "  is accepted." << endl);
          break;
          
        case 0x03:
          ERROR_STREAM("Camera " << ((response->command[0] & 0x70) >> 4) << " : "
                       << "Command Buffer Full" << endl
                       << "  Indicates that two sockets are already being used"
                       << "  (executing two commands) and the command could not"
                       << "  be accepted when received." << endl);
          break;
          
        case 0x04:
          ERROR_STREAM("Camera " << ((response->command[0] & 0x70) >> 4) << " : "
                       << "Command canceled on socket " << socknum << endl
                       << "  Returned when a command which is beeing executed"
                       << "  in a socket specified by the cancel command is"
                       << "  canceled. The completion messgae for the command"
                       << "  is not returned." << endl);
          break;
          
        case 0x05:
          ERROR_STREAM("Camera " << ((response->command[0] & 0x70) >> 4) << " : "
                       << "No socket" << socknum << endl
                       << "  Returned when no command is executed in a socket"
                       << "  specified by the cancel command, or when an"
                       << "  invalid socket number is specified" << endl);
          break;
          
        case 0x41:
          ERROR_STREAM("Camera " << ((response->command[0] & 0x70) >> 4) << " : "
                       << "Command Not Executable on socket " << socknum << endl
                       << "  Returned when a command cannot be executed due"
                       << "  to currend conditions. For example, when"
                       << "  commands controlling the focus manually are"
                       << "  received during auto focus." << endl);
          
        default:
          ERROR_STREAM("Unkown error received." << endl);
        } /* switch (retval) */
      } /* if(display) */
    } /* if (0x60) */
    
    return retval;
  } /* check_response */
  
  
  Result VISCACameraReadNode::process_response(struct_visca_command *command) {
    DEBUG_STREAM("enter process response" << endl);
    int i;
    unsigned char camera_id;
    Result retval = FAILURE;

    if (0x50 == (command->command[1] & 0xf0)) {
      camera_id = (command->command[0] & 0x70) >> 4;
      for (i=0; i<MAX_SLOTS_PER_CAMERA; i++) {
        
        // cerr << "Slot " << i << " : "; verbose ( "", slots[camera_id].slots[i].expected_answer );
        
        if (slots[camera_id].slots[i].active) {
          if (compare_commands(&(slots[camera_id].slots[i].expected_answer), command)) {
            slots[camera_id].slots[i].active = false;
            DEBUG_STREAM("Free Slot " << i << " of camera " << int(camera_id) << endl);
            
            if (slots[camera_id].locked && ( i == slots[camera_id].lockedslot))
              slots[camera_id].locked = false;
            
            retval = SUCCESS;
            break;
          }
        } 
      } /* for (i=0;...) */
    } else if (0x70 == (command->command[1] & 0xf0)) {
      /* the IR-remote control has send a message */
      /* what to do if we get one ? */
    }

    DEBUG_STREAM("leave process response" << endl);    
    return retval;
  } /* process_response */
  
  
  int VISCACameraReadNode::check_slots_for_timeout(const unsigned char& camera_id){
    int loop;
    int outtimed=0;
    struct timeval timenow;
    
    /* if the camera is locked check if there is a command */
    if (slots[camera_id].locked) {
      if (read_from_camera(&visca_response1) &&
          (0 == check_response(&visca_response1,false))) {
        process_response(&visca_response1);
      }
    }
    
    gettimeofday(&timenow,NULL);
    
    /* clear all timeout slots for reuse */
    for (loop=0;loop <MAX_SLOTS_PER_CAMERA;loop++){
      if ( slots[camera_id].slots[loop].active ) {
        if (timercmp ((&timenow), (&(slots[camera_id].slots[loop].expiredate)), > )) {
          //          cout << "timeout : camera = " << int(camera_id) 
          //    << ", slot = " << loop 
          //    << ", socket = " << int(slots[camera_id].slots[loop].socket) << endl;
          slots[camera_id].slots[loop].active = false;
          if (slots[camera_id].locked && (loop == slots[camera_id].lockedslot)) {
            slots[camera_id].locked = false;
          }
          outtimed++;
        }
      }
    }
    return outtimed;
  } /* check_slots_for_timeout */ 
  

  int VISCACameraReadNode::find_free_slot(const unsigned char& camera_id){
    int found = -1;
    int loop  = 0;
    
    while (MAX_SLOTS_PER_CAMERA > loop){
      if (!slots[camera_id].slots[loop].active) {
        found = loop;
        break;
      }
      loop++;
    }
    
    return found;
  } /* find_free_slot */
  
  
  /***********************************************************************/
  /* high level camera stuff */
  /***********************************************************************/
  
  Result VISCACameraReadNode::IF_Clear(const unsigned char& camera_id){
    Result retval = SUCCESS;
    int i;
    
    /* build the command */
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x00;
    visca_command.command[3] = 0x01;
    visca_command.command[4] = 0xff;
    visca_command.length     = 5;
    
    /* send and wait for answer */
    if (send_to_camera(&visca_command) &&
        read_from_camera(&visca_response1)) {
      /* comparision request <-> response, must be equal */
      if (visca_command.length == visca_response1.length) {
        for(i=0; i<visca_command.length; i++) {
          if (visca_command.command[i] != visca_response1.command[i]) {
            ERROR_STREAM("Can't send IF_Clear command to camera " << int(camera_id) << "." << endl);
            retval = FAILURE;
            break;
          }
        }
      }
    } else {
      ERROR_STREAM("Failure by sending 'IF_Clear' command to camera " << camera_id); 
      retval = FAILURE;
    }
    return retval;
  } /* IF_Clear */
  
  
  Result VISCACameraReadNode::AddressSet() {
    Result retval = SUCCESS;
    
    /* build the command */
    visca_command.command[0] = 0x88;
    visca_command.command[1] = 0x30;
    visca_command.command[2] = 0x01;
    visca_command.command[3] = 0xff;
    visca_command.length     = 4;  

    /* send and wait for answer */
    if (send_to_camera(&visca_command) && 
        read_from_camera(&visca_response1) ) {
      number_of_cameras = visca_response1.command[2]-1;
    } else {
      retval = FAILURE;
      ERROR_STREAM("Failure by sending 'AddressSet' command to camera "); 
    }
    return retval;
  } /* AddressSet */
  

  Result VISCACameraReadNode::getNumberOfCameras(unsigned char& numofcams){
    numofcams = number_of_cameras;
    return SUCCESS;
  } /* getNumberOfCameras */
  

  Result VISCACameraReadNode::CommandCancel( const unsigned char& camera_id, const unsigned char& socket_number_ ) {
    Result retval = SUCCESS;
    
    int socket_number = socket_number_;
    /* check parameter limits */
    if ( 1 > socket_number ) { socket_number = 1; }
    if ( 3 < socket_number ) { socket_number = 2; }
    
    /* build the command */
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x20 | socket_number;
    visca_command.command[2] = 0xff;
    visca_command.length     = 3;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send CommandCancel command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    
    return retval;
  } /* CommandCancel */
  
  
  Result VISCACameraReadNode::PowerOn( const unsigned char& camera_id ) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x00,0x02,&visca_command);
    
    if (!send_command_and_wait_for_ack( &visca_command )) {
      ERROR_STREAM("Can't send PowerOn command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PowerOn */


  Result VISCACameraReadNode::PowerOff( const unsigned char& camera_id ) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x00,0x03,&visca_command);
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
     ERROR_STREAM("Can't send PowerOff command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PowerOn */


  Result VISCACameraReadNode::AutoPowerOff( const unsigned char& camera_id, const int& time ) {
    Result retval = SUCCESS;
    
    /* build the command */
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x40;
    visca_command.command[4] = (time & 0xf000) >> 12;
    visca_command.command[5] = (time & 0x0f00) >>  8;
    visca_command.command[6] = (time & 0x00f0) >>  4;
    visca_command.command[7] = (time & 0x000f) >>  0;
    visca_command.command[8] = 0xff;
    visca_command.length     = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutoPowerOff command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE; 
    }
    return retval;
  } /* AutoPowerOff */
  
  
  Result VISCACameraReadNode::ZoomStop(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x07,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't sending ZoomStop command to the camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ZoomStop */
  
  
  Result VISCACameraReadNode::ZoomTeleStd(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x07,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ZoomTeleStd command to the camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ZoomTeleStd */
  
  Result VISCACameraReadNode::ZoomWideStd(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x07,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ZoomWideStd command to the camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ZoomWideStd */
  
  
  Result VISCACameraReadNode::ZoomTeleVar(const unsigned char& camera_id, const unsigned char& speed_) {
    Result retval = SUCCESS;
    
    int speed = speed_;
    if (7 < speed) speed = 7;
    
    compose_command6(camera_id,0x04,0x07,0x20 | speed,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ZoomTeleVar command to the camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ZoomTeleVar */
  

  Result VISCACameraReadNode::ZoomWideVar(const unsigned char& camera_id, const unsigned char& speed_) {
    Result retval = SUCCESS;
    
    int speed = speed_;
    if (7 < speed) speed = 7;
    compose_command6(camera_id,0x04,0x07,0x30 | speed,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ZoomWideVar command to the camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ZoomWideVar */


  Result VISCACameraReadNode::ZoomDirect( const unsigned char& camera_id, const int& factor_ ) {
    Result retval = SUCCESS;

    int factor = factor_;
    if (0x0000 > factor) { factor=0x0000; }
    if (0x7000 < factor) { factor=0x7000; }
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x47;
    visca_command.command[4] = (factor & 0xf000) >> 12;
    visca_command.command[5] = (factor & 0x0f00) >>  8;
    visca_command.command[6] = (factor & 0x00f0) >>  4;
    visca_command.command[7] = (factor & 0x000f) >>  0;
    visca_command.command[8] = 0xff;
    visca_command.length     = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      MESSAGE_STREAM("Can't send ZoomDirect command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }

    return retval;
  } /* ZoomDirect */


  Result VISCACameraReadNode::ZoomDigitalZoomOnOff( const unsigned char& camera_id, const bool& enable){
    Result retval = SUCCESS;
    
    if (enable) {
      compose_command6(camera_id,0x04,0x06,0x02,&visca_command);
    } else {
      compose_command6(camera_id,0x04,0x06,0x03,&visca_command);
    }
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ZoomDigitalZoom command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    
    return retval;
  } /* ZoomDigitalZoom */


  Result VISCACameraReadNode::FocusStop(const unsigned char& camera_id){
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x08,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusStop command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusStop */
  
  
  Result VISCACameraReadNode::FocusFarStd(const unsigned char& camera_id){
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x08,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusFarStd command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusFarStd */


  Result VISCACameraReadNode::FocusNearStd(const unsigned char& camera_id){
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x08,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusNearStd command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusNearStd */
  
  
  Result VISCACameraReadNode::FocusFarVar(const unsigned char& camera_id, const unsigned char& speed_){
    Result retval = SUCCESS;
    
    int speed = speed_;
    if (7 < speed) speed = 7;
    
    compose_command6(camera_id,0x04,0x08,0x20 | speed,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusFarVar command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusFarVar */
  

  Result VISCACameraReadNode::FocusNearVar(const unsigned char& camera_id, const unsigned char& speed_){
    Result retval = SUCCESS;
    
    int speed = speed_;
    if (7 < speed) speed = 7;
    
    compose_command6(camera_id,0x04,0x08,0x30 | speed,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusNearVar command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusNearVar */


  Result VISCACameraReadNode::FocusDirect(const unsigned char& camera_id, const short unsigned int& position){
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x48;
    visca_command.command[4] = (position & 0xf000) >> 12;
    visca_command.command[5] = (position & 0x0f00) >>  8;
    visca_command.command[6] = (position & 0x00f0) >>  4;
    visca_command.command[7] = (position & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      retval = FAILURE;
    }
    return retval;
  } /* FocusDirect */
  

  Result VISCACameraReadNode::FocusAutoFocus(const unsigned char& camera_id){ 
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x38,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusAutoFocus command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusAutoFocus */
  

  Result VISCACameraReadNode::FocusManualFocus(const unsigned char& camera_id){ 
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x38,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusManualFocus command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusManualFocus */
  
  
  Result VISCACameraReadNode::FocusAutoManual(const unsigned char& camera_id){ 
    Result retval = SUCCESS;

    compose_command6(camera_id,0x04,0x38,0x10,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusAutoManual command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusAutoManual */

  
  Result VISCACameraReadNode::FocusOnePushTrigger(const unsigned char& camera_id){ 
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x18,0x01,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusOnePushTrigger command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusOnePushTrigger */


  Result VISCACameraReadNode::FocusInfinity(const unsigned char& camera_id){ 
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x18,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusInfinity command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusInfinity */


  Result VISCACameraReadNode::FocusAFSensHigh(const unsigned char& camera_id){ 
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x18,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusAFSensHigh command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusAFSensHigh */


  Result VISCACameraReadNode::FocusAFSensLow(const unsigned char& camera_id){ 
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x58,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusAFSensLow command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* FocusAFSensLow */
  

  Result VISCACameraReadNode::FocusNearLimit(const unsigned char& camera_id, const short unsigned int& limit) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x28;
    visca_command.command[4] = (limit & 0xf000) >> 12;
    visca_command.command[5] = (limit & 0x0f00) >>  8;
    visca_command.command[6] = (limit & 0x00f0) >>  4;
    visca_command.command[7] = (limit & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send FocusNearLimit command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* FocusNearLimit */
  
  Result VISCACameraReadNode::WhiteBalance(const unsigned char&,const unsigned char&,const unsigned char&) {
    Result retval = SUCCESS;
    return retval;
  } /* WhiteBalance */
  
  Result VISCACameraReadNode::WhiteBalanceAuto(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x35,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send WhiteBalanceAuto command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    
    return retval;
  } /* WhiteBalanceAuto */
  

  Result VISCACameraReadNode::WhiteBalanceIndoor(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x35,0x01,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send WhiteBalanceIndoor command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* WhiteBalanceIndoor */
  

  Result VISCACameraReadNode::WhiteBalanceOutdoor(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x35,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send WhiteBalanceOutdoor command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* WhiteBalanceOutdoor */
  
  
  Result VISCACameraReadNode::WhiteBalanceOnePush(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x35,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send WhiteBalanceOnePush command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* WhiteBalanceOnePush */
  

  Result VISCACameraReadNode::WhiteBalanceAT(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x35,0x04,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send WhiteBalanceAT command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* WhiteBalanceAT */
  

  Result VISCACameraReadNode::WhiteBalanceManual(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x35,0x05,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send WhiteBalanceManual command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* WhiteBalanceManual */
  
  
  Result VISCACameraReadNode::WhiteBalanceOnePushTrigger(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x10,0x05,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send WhiteBalanceOnePushTrigger command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* WhiteBalanceOnePushTrigger */
  

  Result VISCACameraReadNode::RGainReset(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x03,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send RGainReset command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* RGainReset */
  
  
  Result VISCACameraReadNode::RGainUp(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x03,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send RGainUp command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* RGainUp */
  
  
  Result VISCACameraReadNode::RGainDown(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x03,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send RGainDown command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* RGainDown */
  
  
  Result VISCACameraReadNode::RGainDirect(const unsigned char& camera_id, const short unsigned int& gain) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x43;
    visca_command.command[4] = (gain & 0xf000) >> 12;
    visca_command.command[5] = (gain & 0x0f00) >>  8;
    visca_command.command[6] = (gain & 0x00f0) >>  4;
    visca_command.command[7] = (gain & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send RGainDirect command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* RGainDirect */
  
  
  Result VISCACameraReadNode::BGainReset(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x04,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send BGainReset command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* BGainReset */
  
  
  Result VISCACameraReadNode::BGainUp(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x04,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send BGainUp command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* BGainUp */
  
  
  Result VISCACameraReadNode::BGainDown(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    bool lock = false;

    compose_command6(camera_id,0x04,0x04,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command, lock)) {
      ERROR_STREAM("Can't send BGainDown command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* BGainDown */
  
  
  Result VISCACameraReadNode::BGainDirect(const unsigned char& camera_id, const short unsigned int& gain) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x44;
    visca_command.command[4] = (gain & 0xf000) >> 12;
    visca_command.command[5] = (gain & 0x0f00) >>  8;
    visca_command.command[6] = (gain & 0x00f0) >>  4;
    visca_command.command[7] = (gain & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send BGainDirect command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* BGainDirect */
  
  
  Result VISCACameraReadNode::AutomaticExposureFull(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x39,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutomaticExposureFull command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* AutomaticExposureFull */
  

  Result VISCACameraReadNode::AutomaticExposureManual(const unsigned char& camera_id) {
    Result retval = SUCCESS;

    compose_command6(camera_id,0x04,0x39,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutomaticExposureManual command to camera "  << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* AutomaticExposureManual */
  

  Result VISCACameraReadNode::AutomaticExposureShutterPriority(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x39,0x0a,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutomaticExposureShutterPriority command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* AutomaticExposureShutterPriority */
  
  
  Result VISCACameraReadNode::AutomaticExposureIrisPriority(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x39,0x0b,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutomaticExposureIrisPriority command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* AutomaticExposureIrisPriority */
  
  
  Result VISCACameraReadNode::AutomaticExposureGainPriority(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x39,0x0c,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutomaticExposureGainPriority command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* AutomaticExposureGainPriority */
  
  
  Result VISCACameraReadNode::AutomaticExposureBright(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x39,0x0d,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutomaticExposureBright command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* AutomaticExposureBright */
  
  
  Result VISCACameraReadNode::AutomaticExposureShutterAuto(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x39,0x1a,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutomaticExposureShutterAuto command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* AutomaticExposureShutterAuto */
  
  
  Result VISCACameraReadNode::AutomaticExposureIrisAuto(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x39,0x1b,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutomaticExposureIrisAuto command to camera "  << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* AutomaticExposureIrisAuto */
  
  
  Result VISCACameraReadNode::AutomaticExposureGainAuto(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x39,0x1c,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send AutomaticExposureGainAuto command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* AutomaticExposureGainAuto */
  
  
  Result VISCACameraReadNode::SlowShutter(const unsigned char& camera_id, const bool& manual) {
    Result retval = SUCCESS;
    
    if (!manual) {
      compose_command6(camera_id,0x04,0x5a,0x02,&visca_command);
    } else {
      compose_command6(camera_id,0x04,0x5a,0x03,&visca_command);
    }
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send SlowShutter command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* SlowShutter */
  
  
  Result VISCACameraReadNode::ShutterReset(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0a,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ShutterReset command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ShutterReset */
  

  Result VISCACameraReadNode::ShutterUp(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0a,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ShutterUp command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ShutterUp */
  
  
  Result VISCACameraReadNode::ShutterDown(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0a,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ShutterDown command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ShutterDown */
  
  
  Result VISCACameraReadNode::ShutterDirect(const unsigned char& camera_id,const short unsigned int& position) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x4a;
    visca_command.command[4] = (position & 0xf000) >> 12;
    visca_command.command[5] = (position & 0x0f00) >>  8;
    visca_command.command[6] = (position & 0x00f0) >>  4;
    visca_command.command[7] = (position & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ShutterDirect command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ShutterDirect */
  
  
  Result VISCACameraReadNode::IrisReset(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0b,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send IrisReset command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* IrisReset */
  
  
  Result VISCACameraReadNode::IrisUp(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0b,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send IrisUp command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* IrisUp */
  
  
  Result VISCACameraReadNode::IrisDown(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0b,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send IrisDown command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* IrisDown */


  Result VISCACameraReadNode::IrisDirect(const unsigned char& camera_id, const short unsigned int& position) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x4b;
    visca_command.command[4] = (position & 0xf000) >> 12;
    visca_command.command[5] = (position & 0x0f00) >>  8;
    visca_command.command[6] = (position & 0x00f0) >>  4;
    visca_command.command[7] = (position & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send IrisDirect command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* IrisDirect */
  

  Result VISCACameraReadNode::GainReset(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0c,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send GainReset command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* GainReset */
  
  
  Result VISCACameraReadNode::GainUp(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0c,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send GainUp command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* GainUp */
  
  
  Result VISCACameraReadNode::GainDown(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0c,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send GainDown command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* GainDown */
  
  
  Result VISCACameraReadNode::GainDirect(const unsigned char& camera_id,const short unsigned int& position) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x4c;
    visca_command.command[4] = (position & 0xf000) >> 12;
    visca_command.command[5] = (position & 0x0f00) >>  8;
    visca_command.command[6] = (position & 0x00f0) >>  4;
    visca_command.command[7] = (position & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;

    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send GainDirect command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* GainDirect */
  
  
  Result VISCACameraReadNode::BrightReset(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0d,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send BrightReset command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* BrightReset */
  
  
  Result VISCACameraReadNode::BrightUp(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0d,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send BrightUp command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* BrightUp */
  
  
  Result VISCACameraReadNode::BrightDown(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0d,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send BrightDown command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* BrightDown */
  
  
  Result VISCACameraReadNode::BrightDirect(const unsigned char& camera_id, const short unsigned int& position) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x4d;
    visca_command.command[4] = (position & 0xf000) >> 12;
    visca_command.command[5] = (position & 0x0f00) >>  8;
    visca_command.command[6] = (position & 0x00f0) >>  4;
    visca_command.command[7] = (position & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;

    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send BrightDirect command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* BrightDirect */


  Result VISCACameraReadNode::ExposureCompensationOnOff(const unsigned char& camera_id,const bool& on) {
    Result retval = SUCCESS;
    
    if (on) {
      compose_command6(camera_id,0x04,0x3e,0x02,&visca_command);
    } else {
      compose_command6(camera_id,0x04,0x3e,0x03,&visca_command);
    }
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't sned ExposurecompensationOnOff command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ExposureCompensationOnOff */


  Result VISCACameraReadNode::ExposureCompensationReset(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0e,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ExposureCompensationReset command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ExposureCompensationReset */
  
  
  Result VISCACameraReadNode::ExposureCompensationUp(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0e,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ExposurecompensationUp command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ExposureCompensationUp */
  
  
  Result VISCACameraReadNode::ExposureCompensationDown(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x0e,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ExposureCompensationDown command to camera "  << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ExposureCompensationDown */
  
  
  Result VISCACameraReadNode::ExposureCompensationDirect(const unsigned char& camera_id, const short unsigned int& position) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x4e;
    visca_command.command[4] = (position & 0xf000) >> 12;
    visca_command.command[5] = (position & 0x0f00) >>  8;
    visca_command.command[6] = (position & 0x00f0) >>  4;
    visca_command.command[7] = (position & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ExposureCompensationDirect command to camera "  << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ExposureCompensationDirect */
  
  
  Result VISCACameraReadNode::BackLightOnOff(const unsigned char& camera_id, const bool& on) {
    Result retval = SUCCESS;
    
    if (on) {
      compose_command6(camera_id,0x04,0x33,0x02,&visca_command);
    } else {
      compose_command6(camera_id,0x04,0x33,0x03,&visca_command);
    }
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send BackLightOnOff command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* BackLightOnOff */
  
  
  Result VISCACameraReadNode::ApertureReset(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x02,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ApertureReset command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ApertureReset */
  

  Result VISCACameraReadNode::ApertureUp(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x02,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ApertureUp command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ApertureUp */
  

  Result VISCACameraReadNode::ApertureDown(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x02,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ApertureDown command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ApertureDown */
  
  
  Result VISCACameraReadNode::ApertureDirect(const unsigned char& camera_id, const short unsigned int& gain) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x42;
    visca_command.command[4] = (gain & 0xf000) >> 12;
    visca_command.command[5] = (gain & 0x0f00) >>  8;
    visca_command.command[6] = (gain & 0x00f0) >>  4;
    visca_command.command[7] = (gain & 0x000f);
    visca_command.command[8] = 0xff;
    visca_command.length = 9;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send ApertureDirect command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* ApertureDirect */
  

  Result VISCACameraReadNode::WideModeOff(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x60,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)){
      ERROR_STREAM("Can't send WideModeOff command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* WideModeOff */
  
  
  Result VISCACameraReadNode::WideModeCinema(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x60,0x01,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)){
      ERROR_STREAM("Can't send WideModeCinema command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* WideModeCinema */
  
  
  Result VISCACameraReadNode::WideMode16to9Full(const unsigned char& camera_id) { 
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x60,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)){
      ERROR_STREAM("Can't send WideMode16to9Full command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* WideMode16to9Full */
  
  
  Result VISCACameraReadNode::LR_Reverse(const unsigned char& camera_id, const bool& activate){
    Result retval = SUCCESS;
    
    if (activate) {
      compose_command6(camera_id,0x04,0x61,0x02,&visca_command);
    } else {
      compose_command6(camera_id,0x04,0x61,0x03,&visca_command);
    }
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send LR_Reverse to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* LR_Reverse */
  
  
  Result VISCACameraReadNode::Freeze(const unsigned char& camera_id, const bool& activate){
    Result retval = SUCCESS;
    
    if (activate) {
      compose_command6(camera_id,0x04,0x62,0x02,&visca_command);
    } else {
      compose_command6(camera_id,0x04,0x62,0x03,&visca_command);
    }
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send Freeze to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* Freeze */
  
  
  Result VISCACameraReadNode::PictureEffectOff(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x63,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send PictureEffectOff command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PictureEffectOff */
  
  
  Result VISCACameraReadNode::PictureEffectPastel(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x63,0x01,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send PictureEffectPastel command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PictureEffectPastel */
  
  
  Result VISCACameraReadNode::PictureEffectNegArt(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x63,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send PictureNegArt command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PictureEffectNegArt */
  
  
  Result VISCACameraReadNode::PictureEffectSepia(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    compose_command6(camera_id,0x04,0x63,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send PictureEffectSepia command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PictureEffectSepia */
  
  
  Result VISCACameraReadNode::PictureEffectBW(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x63,0x04,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send PictureEffectDW command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PictureEffectBW */
  
  
  Result VISCACameraReadNode::PictureEffectSolarize(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x63,0x05,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send PictureEffectSolarize command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PictureEffectSolarize */
  
  
  Result VISCACameraReadNode::PictureEffectMosaic(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x63,0x06,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send PictureEffectSolaize command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PictureEffectMosaic */
  
  
  Result VISCACameraReadNode::PictureEffectSlim(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x63,0x07,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send PictureEffectSlim command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PictureEffectSlim */
  
  
  Result VISCACameraReadNode::PictureEffectStretch(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x63,0x08,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send PictureEffectStretch command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PictureEffectStretch */
  
  
  Result VISCACameraReadNode::DigitalEffectOff(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x64,0x00,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send DigitalEffectOff command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* DigitalEffectOff */
  
  
  Result VISCACameraReadNode::DigitalEffectStill(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x64,0x01,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send DigitalEffectStill command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* DigitalEffectStill */
  

  Result VISCACameraReadNode::DigitalEffectFlash(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x64,0x02,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send DigitalEffectFlash command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* DigitalEffectFlash */
  
  
  Result VISCACameraReadNode::DigitalEffectLumi(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x64,0x03,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send DigitalEffectLumi command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* DigitalEffectLumi */
  
  
  Result VISCACameraReadNode::DigitalEffectTrail(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    compose_command6(camera_id,0x04,0x64,0x04,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send DigitalEffectTrail command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* DigitalEffectTrail */
  
  
  Result VISCACameraReadNode::DigitalEffectLevel(const unsigned char& camera_id, const unsigned char& effect, const unsigned char& level_) {
    Result retval = SUCCESS;
    
    int level = level_;
    // Idee: Inquiry Command senden, sehen welcher Filter gerade aktiv und dann
    // Bereiche checken
    
    /* check parameter limits */
    if ( (0x02 == effect) || (0x04 == effect) ){
      if (0x20 < level) level = 0x20;
    } else {
      if (0x18 < level) level = 0x18;
    }
    
    compose_command6(camera_id,0x04,0x65,level,&visca_command);
    if (!send_command_and_wait_for_ack(&visca_command)){
      ERROR_STREAM("Can't send DigitalEffectLevel command for effect " << int(effect) << " to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* DigitalEffectLevel */
  
  
  Result VISCACameraReadNode::Memory(const unsigned char& camera_id, const unsigned char& mode, const unsigned char& memory_slot_){
    int memory_slot = memory_slot_;
    /* check parameter limits */
    if (5 < memory_slot) memory_slot = 5;
    
    /* build the command to send */
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x04;
    visca_command.command[3] = 0x3f;  
    visca_command.command[4] = mode;  
    visca_command.command[5] = memory_slot;  
    visca_command.command[6] = 0xff;  
    visca_command.length     = 7;
    
    return send_command_and_wait_for_ack(&visca_command);
  }
  
  
  Result VISCACameraReadNode::MemoryReset(const unsigned char& camera_id, const unsigned char& memory_slot) {
    Result retval = SUCCESS;
    const unsigned char& modus = 0x00;

    if (!Memory(camera_id, modus, memory_slot)) {
      ERROR_STREAM("Can't send MemoryReset command to camera " << int(camera_id) << "." << endl);
      retval =  FAILURE;
    }
    return retval;
  } /* MemoryReset */
  
  
  Result VISCACameraReadNode::MemorySet(const unsigned char& camera_id, const unsigned char& memory_slot) {
    Result retval = SUCCESS;
    unsigned char modus = 0x01;

    if (!Memory(camera_id, modus, memory_slot)) {
      ERROR_STREAM("Can't send MemorySet command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* MemorySet */
  
  
  Result VISCACameraReadNode::MemoryRecall(const unsigned char& camera_id, const unsigned char& memory_slot) {
    Result retval = SUCCESS;
    unsigned char modus = 0x02;

    if (!Memory(camera_id, modus, memory_slot)) {
      ERROR_STREAM("Can't send MemoryRecall command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* MemoryRecall */
  
  
  Result VISCACameraReadNode::DataScreenOnOff(const unsigned char& camera_id, const bool& on) {
    Result retval = SUCCESS;
    
    if (on) {
      compose_command6(camera_id,0x06,0x06,0x02,&visca_command);
    } else {
      compose_command6(camera_id,0x06,0x06,0x03,&visca_command);
    }
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send DataScreenOnOff command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* DataScreenOnOff */
  
  
  Result VISCACameraReadNode::IRReceiveOnOff(const unsigned char& camera_id, const bool& on) {
    Result retval = SUCCESS;
    
    if (on) {
      compose_command6(camera_id,0x06,0x08,0x02,&visca_command);
    } else {
      compose_command6(camera_id,0x06,0x08,0x03,&visca_command);
    }
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send IRReceiveOnOff command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* IRReceiveOnOff */
  
  
  Result VISCACameraReadNode::IRReceiveReturn(const unsigned char& camera_id, const bool& on) {
    Result retval = SUCCESS;
    
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x7d;
    visca_command.command[3] = 0x01;
    if (on) {
      visca_command.command[4] = 0x03;
    } else {
      visca_command.command[4] = 0x13;
    }
    visca_command.command[5] = 0x00;
    visca_command.command[6] = 0x00;
    visca_command.command[7] = 0xff;
    visca_command.length = 8;
    
    if (!send_command_and_wait_for_ack(&visca_command)) {
      ERROR_STREAM("Can't send IRReceiveReturn command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* IRReceiveReturn */
  
  
  Result VISCACameraReadNode::PanTiltDriveMotion(const unsigned char& camera_id, const unsigned char& mode1, const unsigned char& mode2, const unsigned char& panspeed_, const unsigned char& tiltspeed_){
    int panspeed = panspeed_;
    int tiltspeed = tiltspeed_;
    /* check parameter limits */
    if (  0 == panspeed) panspeed = 1;
    if (0x18 < panspeed) panspeed = 0x18;
    if (  0 == tiltspeed) tiltspeed = 1;
    if (0x14 < tiltspeed) tiltspeed = 0x14;
    
    /* build the command */
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x06;
    visca_command.command[3] = 0x01;
    visca_command.command[4] = panspeed;
    visca_command.command[5] = tiltspeed;
    visca_command.command[6] = mode1;
    visca_command.command[7] = mode2;
    visca_command.command[8] = 0xff;
    visca_command.length     = 9;
    
    return send_command_and_wait_for_ack(&visca_command);
  } /* PantTiltDriveMotion */
  
  
  Result VISCACameraReadNode::PanTiltDriveUp(const unsigned char& camera_id, const unsigned char& panspeed, const unsigned char& tiltspeed){
    Result retval = SUCCESS;
    unsigned char modus1 = 0x03;
    unsigned char modus2 = 0x01;

    
    if (!PanTiltDriveMotion(camera_id, modus1, modus2, panspeed, tiltspeed)){
      ERROR_STREAM("Can;t send pan/tilt Up command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* PantTiltDriveUp */
  
  
  Result VISCACameraReadNode::PanTiltDriveDown(const unsigned char& camera_id, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    unsigned char modus1 = 0x03;
    unsigned char modus2 = 0x02;

    if (!PanTiltDriveMotion(camera_id, modus1, modus2, panspeed, tiltspeed)){
      ERROR_STREAM("Can't send pan/tilt Down command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* PanTiltDriveDown */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveLeft(const unsigned char& camera_id, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    unsigned char modus1 = 0x01;
    unsigned char modus2 = 0x03;

    if (!PanTiltDriveMotion(camera_id, modus1, modus2, panspeed, tiltspeed)){
      ERROR_STREAM("Can't send pan/tilt Left command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* PanTiltDriveLeft */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveRight(const unsigned char& camera_id, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    unsigned char modus1 = 0x02;
    unsigned char modus2 = 0x03;

    if (!PanTiltDriveMotion(camera_id, modus1, modus2, panspeed, tiltspeed)){
      ERROR_STREAM("Can't send pan/tilt Right command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* PanTiltDriveRight */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveUpLeft(const unsigned char& camera_id, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    unsigned char modus1 = 0x01;
    unsigned char modus2 = 0x01;

    if (!PanTiltDriveMotion(camera_id, modus1, modus2, panspeed, tiltspeed)){
      ERROR_STREAM("Can't send pan/tilt UpLeft command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* PanTiltDriveUpLeft */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveUpRight(const unsigned char& camera_id, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    unsigned char modus1 = 0x02;
    unsigned char modus2 = 0x01;

    if (!PanTiltDriveMotion(camera_id, modus1, modus2, panspeed, tiltspeed)){
      ERROR_STREAM("Can't send pan/tilt UpRight command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* PanTiltDriveUpRight */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveDownLeft(const unsigned char& camera_id, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    unsigned char modus1 = 0x01;
    unsigned char modus2 = 0x02;

    if (!PanTiltDriveMotion(camera_id, modus1, modus2, panspeed, tiltspeed)){
      ERROR_STREAM("Can't send pan/tilt DownLeft command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* PanTiltDriveDownLeft */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveDownRight(const unsigned char& camera_id, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    unsigned char modus1 = 0x02;
    unsigned char modus2 = 0x02;

    if (!PanTiltDriveMotion(camera_id, modus1, modus2, panspeed, tiltspeed)){
      ERROR_STREAM("Can't send pan/tilt DownRight command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* PanTiltDriveDownRight */ 
  
  
  
  Result VISCACameraReadNode::PanTiltDriveStop(const unsigned char& camera_id, const unsigned char& panspeed, const unsigned char& tiltspeed ) {
    Result retval = SUCCESS;
    unsigned char modus1 = 0x03;
    unsigned char modus2 = 0x03;

    if (!PanTiltDriveMotion(camera_id, modus1, modus2, panspeed, tiltspeed)){
      ERROR_STREAM("Can't send pan/tilt Stop command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;  
  } /* PanTiltDriveStop */ 
  
  
  Result VISCACameraReadNode::PanTiltDrivePosition(const unsigned char& camera_id, const unsigned char& mode, const int& panpos_, const int& tiltpos_, const unsigned char& panspeed_, const unsigned char& tiltspeed_) {
    int panspeed = panspeed_;
    int tiltspeed = tiltspeed_;
    int panpos = panpos_;
    int tiltpos = tiltpos_;
    /* check parameter limits */
    if ( 0 == panspeed)  panspeed = 1;
    if (24 < panspeed)   panspeed = 24;
    if ( 0 == tiltspeed) tiltspeed = 1;
    if (20 < tiltspeed)  tiltspeed = 20;
    
    if (-1440 > panpos  ) panpos = -1440;
    if ( 1440 < panpos  ) panpos =  1440;
    if ( -360 > tiltpos ) panpos =  -360;
    if (  360 < tiltpos ) panpos =   360;
    
    /* build the command */
    visca_command.command[ 0] = 0x80 | camera_id;
    visca_command.command[ 1] = 0x01;
    visca_command.command[ 2] = 0x06;
    visca_command.command[ 3] = mode;
    visca_command.command[ 4] = panspeed;
    visca_command.command[ 5] = tiltspeed;
    visca_command.command[ 6] = (panpos  & 0xf000) >> 12;
    visca_command.command[ 7] = (panpos  & 0x0f00) >>  8;
    visca_command.command[ 8] = (panpos  & 0x00f0) >>  4;
    visca_command.command[ 9] = (panpos  & 0x000f);
    visca_command.command[10] = (tiltpos & 0xf000) >> 12;
    visca_command.command[11] = (tiltpos & 0x0f00) >>  8;
    visca_command.command[12] = (tiltpos & 0x00f0) >>  4;
    visca_command.command[13] = (tiltpos & 0x000f);
    visca_command.command[14] = 0xff;
    visca_command.length      = 15;
    
    return send_command_and_wait_for_ack(&visca_command,true);
    
  } /* PanTiltDrivePosition */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveAbsolute(const unsigned char& camera_id, const int& panpos, const int& tiltpos, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    if (!PanTiltDrivePosition(camera_id,0x02,panpos,tiltpos,panspeed,tiltspeed )){
      ERROR_STREAM("Can't send pan/tilt Absolute command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PanTiltDriveAbsolute */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveRelative(const unsigned char& camera_id, const int& panpos, const int& tiltpos, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    if (!PanTiltDrivePosition(camera_id,0x03,panpos,tiltpos,panspeed,tiltspeed )){
      ERROR_STREAM("Can't send pan/tilt Relative command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PanTiltDriveRelative */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveAbsoluteDeg(const unsigned char& camera_id, const int& pandeg_, const int& tiltdeg_, const unsigned char& panspeed_, const unsigned char& tiltspeed_) {
    Result retval = SUCCESS;
    int pansteps = 0, tiltsteps = 0;
    
    int panspeed = panspeed_;
    int tiltspeed = tiltspeed_;
    int pandeg = pandeg_;
    int tiltdeg = tiltdeg_;
    /* convert degree-values to coordinate-values */
    /* a) check ranges */
    if (-100 > pandeg )  pandeg  = -100;
    if ( 100 < pandeg )  pandeg  =  100;
    if ( -25 > tiltdeg ) tiltdeg = -25;
    if (  25 < tiltdeg ) tiltdeg =  25;
    
    /* b) convert */
    if ( 0 > pandeg) pansteps = int(pandeg * 14.4f - 0.5f);
    else pansteps = int(pandeg * 14.4f + 0.5f);
    
    if ( 0 > tiltdeg) tiltsteps = int(tiltdeg * 14.4f - 0.5f);
    else tiltsteps = int(tiltdeg * 14.4f + 0.5f);
    
    if (!PanTiltDrivePosition(camera_id,0x02,pansteps,tiltsteps,panspeed,tiltspeed )){
      ERROR_STREAM("Can't send pan/tilt AbsoluteDeg command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PanTiltDriveAbsoluteDeg */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveRelativeDeg(const unsigned char& camera_id, const int& pandeg_, const int& tiltdeg_, const unsigned char& panspeed, const unsigned char& tiltspeed) {
    Result retval = SUCCESS;
    int  pansteps = 0, tiltsteps = 0;
    
    int pandeg = pandeg_;
    int tiltdeg = tiltdeg_;
    /* convert degree-values to coordinate-values */
    /* a) check ranges */
    if (-100 > pandeg )  pandeg  = -100;
    if ( 100 < pandeg )  pandeg  =  100;
    if ( -25 > tiltdeg ) tiltdeg = -25;
    if (  25 < tiltdeg ) tiltdeg =  25;
    
    /* b) convert */
    if ( 0 > pandeg) pansteps = int(pandeg * 14.4f - 0.5f);
    else pansteps = int(pandeg * 14.4f + 0.5f);
    
    if ( 0 > tiltdeg) tiltsteps = int(tiltdeg * 14.4f - 0.5f);
    else tiltsteps = int(tiltdeg * 14.4f + 0.5f);
    
    if (!PanTiltDrivePosition(camera_id,0x03,pansteps,tiltsteps,panspeed,tiltspeed )){
      ERROR_STREAM("Can't send pan/tilt RelativeDeg command to camera " << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PanTiltDriveRelativeDeg */ 
  
  
  Result VISCACameraReadNode::PanTiltDriveHome(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    /* build the command */
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x06;
    visca_command.command[3] = 0x04;
    visca_command.command[4] = 0xff;
    visca_command.length     = 5;
    
    if(!send_command_and_wait_for_ack(&visca_command,true)) {
      ERROR_STREAM("Can't send pan/tilt Home command to camera" << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PanTiltDriveHome */
  
  
  Result VISCACameraReadNode::PanTiltDriveReset(const unsigned char& camera_id) {
    Result retval = SUCCESS;
    
    /* build the command */
    visca_command.command[0] = 0x80 | camera_id;
    visca_command.command[1] = 0x01;
    visca_command.command[2] = 0x06;
    visca_command.command[3] = 0x05;
    visca_command.command[4] = 0xff;
    visca_command.length     = 5;
    
    if(!send_command_and_wait_for_ack(&visca_command)){
      ERROR_STREAM("cant send pan/tilt Reset command to camera" << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PanTiltDriveReset */
  
  
  Result VISCACameraReadNode::PanTiltLimitSet(const unsigned char& camera_id, const unsigned char& mode_, const int& panpos_, const int& tiltpos_) {
    Result retval = SUCCESS;
    
    int mode = mode_;
    int panpos = panpos_;
    int tiltpos = tiltpos_;

    /* check parameter limits */
    if ( 1 < mode ) mode = 1;
    
    if (-1440 > panpos  ) panpos = -1440;
    if ( 1440 < panpos  ) panpos =  1440;
    if ( -360 > tiltpos ) panpos =  -360;
    if (  360 < tiltpos ) panpos =   360;
    
    /* build the command */
    visca_command.command[ 0] = 0x80 | camera_id;
    visca_command.command[ 1] = 0x01;
    visca_command.command[ 2] = 0x06;
    visca_command.command[ 3] = 0x07;
    visca_command.command[ 4] = 0x00;
    visca_command.command[ 5] = mode;
    visca_command.command[ 6] = (panpos  & 0xf000) >> 12;
    visca_command.command[ 7] = (panpos  & 0x0f00) >>  8;
    visca_command.command[ 8] = (panpos  & 0x00f0) >>  4;
    visca_command.command[ 9] = (panpos  & 0x000f);
    visca_command.command[10] = (tiltpos & 0xf000) >> 12;
    visca_command.command[11] = (tiltpos & 0x0f00) >>  8;
    visca_command.command[12] = (tiltpos & 0x00f0) >>  4;
    visca_command.command[13] = (tiltpos & 0x000f);
    visca_command.command[14] = 0xff;
    visca_command.length      = 15;
    
    if(!send_command_and_wait_for_ack(&visca_command)){
      ERROR_STREAM("Can't send pan/tilt LimitSet command to camera" << int(camera_id) << "." << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PanTiltLimitSet */
  
  
  Result VISCACameraReadNode::PanTiltLimitClear(const unsigned char& camera_id, const unsigned char& mode){
    Result retval = SUCCESS;
    
    /* build the command */
    visca_command.command[ 0] = 0x80 | camera_id;
    visca_command.command[ 1] = 0x01;
    visca_command.command[ 2] = 0x06;
    visca_command.command[ 3] = 0x07;
    visca_command.command[ 4] = 0x01;
    visca_command.command[ 5] = mode;
    visca_command.command[ 6] = 0x0f;
    visca_command.command[ 7] = 0x0f;
    visca_command.command[ 8] = 0x0f;
    visca_command.command[ 9] = 0x0f;
    visca_command.command[10] = 0x0f;
    visca_command.command[11] = 0x0f;
    visca_command.command[12] = 0x0f;
    visca_command.command[13] = 0x0f;
    visca_command.command[14] = 0xff;
    visca_command.length      = 15;
    
    if(!send_command_and_wait_for_ack(&visca_command)){
      ERROR_STREAM("cant send pan/tilt LimitClear command to camera" << endl);
      retval = FAILURE;
    }
    return retval;
  } /* PanTiltLimitClear */
  
  
  /**
   * Inquiry Command List
   * see hpp file for the command documentation 
   */
  
  Result VISCACameraReadNode::PowerInq(const unsigned char& camera_id, bool& activate) {
    Result retval=FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x00, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        
        if ( (0x50 == visca_response1.command[1]) && 
             (0xff == visca_response1.command[3]) ) {
          if (0x02 == visca_response1.command[2]) {
            activate = true;
          } else if (0x03 == visca_response1.command[2]) {
            activate = false;
          }
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send PowerInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* PowerInq */
  
  
  Result VISCACameraReadNode::AutoPowerOffInq( const unsigned char& camera_id,short unsigned int& timer){
    Result retval=FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x40,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   7 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          timer = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send AutoPowerOffInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* AutoPowerOffInq */
  
  
  Result VISCACameraReadNode::DZoomModeInq(const unsigned char& camera_id,bool& activate){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x06, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        if ( 0x50 == visca_response1.command[1]) {
          if (0x02 == visca_response1.command[2]) {
            activate = true;
          } else {
            activate = false;
          }
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send DZoomModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* DZoomModeInq */
  
  
  Result VISCACameraReadNode::ZoomPosInq(const unsigned char& camera_id, short unsigned int& position){
    Result retval=FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x47,&visca_response1)){
      if( 0 == check_response(&visca_response1) && (6 == visca_response1.length)) {
        if (0x50 == visca_response1.command[1]){
          position = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send DZoomPosInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* ZoomPosInq */
  
  
  Result VISCACameraReadNode::FocusModeInq(const unsigned char& camera_id,bool& on) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x38,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   4 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          if (0x02 == visca_response1.command[2]) {
            on = true;
          } else {
            on = false;
          }
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send FocusModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* FocusModeInq */
  
  
  Result VISCACameraReadNode::FocusPosInq(const unsigned char& camera_id,short unsigned int& position){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x48,&visca_response1)) {
      if ( 0 == check_response(&visca_response1) ) {
        if ( (   7 == visca_response1.length) &&
             (0x50 == visca_response1.command[1])) {
          position = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send FocusPosInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* FocusPosInq */
  
  
  Result VISCACameraReadNode::AFModeInq(const unsigned char& camera_id,bool& highsens) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x58,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   4 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          if (0x02 == visca_response1.command[2]) {
            highsens = true;
          } else {
            highsens = false;
          }
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send AFModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* AFModeInq */
  
  
  Result VISCACameraReadNode::FocusNearLimitInq(const unsigned char& camera_id,short unsigned int& limit) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x28,&visca_response1)) {
      if ( 0 == check_response(&visca_response1) ) {
        if ( (   7 == visca_response1.length) &&
             (0x50 == visca_response1.command[1])) {
          limit = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send FocusNearLimitInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  }
  
  
  Result VISCACameraReadNode::WhiteBalanceModeInq(const unsigned char& camera_id,unsigned char& mode){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x35,&visca_response1)) {
      if ( 0 == check_response(&visca_response1) ) {
        if ( (   4 == visca_response1.length) &&
             (0x50 == visca_response1.command[1])) {
          mode = visca_response1.command[2];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send WhiteBalanceModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;  
  } /* WhiteBalanceModeInq */
  
  
  
  //  Result VISCACameraReadNode::RGainInq(const unsigned char& camera_id,short unsigned int& gain) {
  Result VISCACameraReadNode::RGainInq(const unsigned char& camera_id,unsigned char& gain) {
    Result retval = FAILURE;
  
    if (send_inquiry_command(camera_id,0x04,0x43,&visca_response1)) {
      if ( 0 == check_response(&visca_response1) ) {
        if ( (   7 == visca_response1.length) &&
             (0x50 == visca_response1.command[1])) {
          gain = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send RGainInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* RGainInq */
  
  
  Result VISCACameraReadNode::BGainInq(const unsigned char& camera_id,short unsigned int& gain) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x44,&visca_response1)) {
      if ( 0 == check_response(&visca_response1) ) {
        if ( (   7 == visca_response1.length) &&
             (0x50 == visca_response1.command[1])) {
          gain = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send BGainInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* BGainInq */
  
  
  Result VISCACameraReadNode::AutoExposureModeInq(const unsigned char& camera_id,unsigned char& mode) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x39,&visca_response1)) {
      if ( 0 == check_response(&visca_response1) ) {
        if ( (   4 == visca_response1.length) &&
             (0x50 == visca_response1.command[1])) {
          mode = visca_response1.command[2];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send AutoExposureModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;  
  } /* AutoExposureModeInq */
  
  
  Result VISCACameraReadNode::SlowShutterModeInq(const unsigned char& camera_id,bool& automode) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x5a,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   4 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          if (0x02 == visca_response1.command[2]) {
            automode = true;
          } else {
            automode = false;
          }
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send SlowShutterModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* SlowShutterModeInq */
  
  
  Result VISCACameraReadNode::ShutterPosInq(const unsigned char& camera_id,short unsigned int& position) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x4a,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   7 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          position = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send ShutterPosInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* ShutterPosInq */
  
  
  Result VISCACameraReadNode::IrisPosInq(const unsigned char& camera_id,short unsigned int& position) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x4b,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   7 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          position = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send IrisPosInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* IrisPosInq */
  
  
  Result VISCACameraReadNode::GainPosInq(const unsigned char& camera_id,short unsigned int& position) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x4c,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   7 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          position = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send GainPosInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* GainPosInq */
  
  
  
  Result VISCACameraReadNode::BrightPosInq(const unsigned char& camera_id,short unsigned int& position) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x4d,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   7 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          position = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send BrightPosInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* BrightPosInq */
  
  
  Result VISCACameraReadNode::ExposureCompensationModeInq(const unsigned char& camera_id,bool& on){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x3e, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        if ( 0x50 == visca_response1.command[1]) {
          if (0x02 == visca_response1.command[2]) {
            on = true;
          } else {
            on = false;
          }
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send ExposureCompensationModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* ExposureCompensationModeInq */
  
  Result VISCACameraReadNode::ExposureCompensationPosInq(const unsigned char& camera_id,short unsigned int& position){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x4e,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   7 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          position = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send ExposureCompensationPosInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* ExposureCompensationPosInq */
  
  
  Result VISCACameraReadNode::BackLightModeInq(const unsigned char& camera_id,bool& on) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x33, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        if ( 0x50 == visca_response1.command[1]) {
          if (0x02 == visca_response1.command[2]) {
            on = true;
          } else {
            on = false;
          }
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send DZoomModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* BackLightModeInq */
  
  
  Result VISCACameraReadNode::ApertureInq(const unsigned char& camera_id,short unsigned int& gain){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x42,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   7 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          gain = visca_response1.command[2] << 12 |
            visca_response1.command[3] <<  8 |
            visca_response1.command[4] <<  4 |
            visca_response1.command[5];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send ApertureInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* ApertureInq */
  
  
  Result VISCACameraReadNode::WideModeInq(const unsigned char& camera_id,unsigned char& mode) {
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x4e,&visca_response1)){
      if( 0 == check_response(&visca_response1) ){
        if( (   4 == visca_response1.length) &&
            (0x50 == visca_response1.command[1])){
          mode = visca_response1.command[2];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send WideModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* WideModeInq */
  
  
  Result VISCACameraReadNode::LR_ReverseModeInq(const unsigned char& camera_id, bool& activate){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x61, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        
        if ( 0x50 == visca_response1.command[1]) {
          if (0x02 == visca_response1.command[2]) {
            activate = true;
          } else {
            activate = false;
          }
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send LR_ReverseModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* LR_ReverseModeInq */
  
  
  Result VISCACameraReadNode::FreezeModeInq(const unsigned char& camera_id, bool& activate){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x62, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        
        if ( 0x50 == visca_response1.command[1]) {
          if (0x02 == visca_response1.command[2]) {
            activate = true;
          } else {
            activate = false;
          }
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send LR_FreezeModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* FreezeModeInq */

  
  Result VISCACameraReadNode::PictureEffectModeInq(const unsigned char& camera_id, unsigned char& effect){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x63, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        
        if ( 0x50 == visca_response1.command[1]) {
          effect = visca_response1.command[2];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send PictureEffectModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* PictureEffectModeInq */
  
  
  Result VISCACameraReadNode::DigitalEffectModeInq(const unsigned char& camera_id, unsigned char& effect){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x64, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        
        if ( 0x50 == visca_response1.command[1]) {
          effect = visca_response1.command[2];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send DigitalEffectModeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* DigitalEffectModeInq */
  
  
  Result VISCACameraReadNode::DigitalEffectLevelInq(const unsigned char& camera_id, unsigned char& level){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x65, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        
        if ( 0x50 == visca_response1.command[1]) {
          level = visca_response1.command[2];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send DigitalEffectLevelInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* DigitalEffectLevelInq */
  
  
  Result VISCACameraReadNode::MemoryInq(const unsigned char& camera_id,unsigned char& memslot){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x04,0x3f, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        
        if ( 0x50 == visca_response1.command[1]) {
          memslot = visca_response1.command[2];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send MemoryInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* MemoryInq */
  
  
  Result VISCACameraReadNode::DatascreenInq(const unsigned char& camera_id,bool& on) {
    Result retval = FAILURE; 
    
    if (send_inquiry_command(camera_id,0x06,0x06,&visca_response1)) {
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        if (0x02 == visca_response1.command[2]) {
          on = true; 
        } else {
          on = false;
        }
        retval = SUCCESS;
      }
    }
    return retval;
  } /* DatascreenInq */
  
  
  Result VISCACameraReadNode::PanTiltModeInq(const unsigned char& camera_id,short unsigned int& mode){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x06,0x10, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (5 == visca_response1.length)) {
        if ( 0x50 == visca_response1.command[1]) {
          mode = visca_response1.command[2] << 8 |
            visca_response1.command[3];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send PanTiltModeInq inquiry to camera " << int(camera_id) << "." << endl)
    }
    return retval;
  } /* PanTiltModeInq */
  
  
  Result VISCACameraReadNode::PanTiltMaxSpeedInq(const unsigned char& camera_id,unsigned char& pan,unsigned char& tilt){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x06,0x11, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (5 == visca_response1.length)) {
        if ( 0x50 == visca_response1.command[1]) {
          pan  = visca_response1.command[2];
          tilt = visca_response1.command[3];
          retval = SUCCESS;
        }
      }
    } else {
      WARNING_STREAM("Can't send PanTiltMaxSpeedInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* PantTiltMaxSpeedInq */
  
  
  Result VISCACameraReadNode::PanTiltPosInq(const unsigned char& camera_id, int& xpos, int& ypos) {
    Result retval = FAILURE;
    unsigned char camera_in_response;
    int error;
    signed short int tmp;

    send_inquiry_command(camera_id,0x06,0x12, &visca_response1);
    
    error = check_response(&visca_response1,true);
    
  /* we have no error and the reponse length is correct */
    if ((0 == error) && (11 == visca_response1.length)) {
      
      camera_in_response = ((0x70 & visca_response1.command[0]) >> 4);
      
      if (camera_in_response == camera_id) {
      
        tmp =  0 | 
          visca_response1.command[2] << 12 |
          visca_response1.command[3] <<  8 | 
          visca_response1.command[4] <<  4 | 
          visca_response1.command[5] <<  0;
        xpos = tmp;

        tmp =  0 |
          visca_response1.command[6] << 12 |
          visca_response1.command[7] <<  8 | 
          visca_response1.command[8] <<  4 | 
          visca_response1.command[9] <<  0;
        ypos = tmp;

        retval = SUCCESS;
      }
    } else {
      ERROR_STREAM("Can't send PanTiltPosInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* PanTiltPosInq */
  
  
  Result VISCACameraReadNode::VideoSystemInq(const unsigned char& camera_id,unsigned char& system){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x06,0x23, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (4 == visca_response1.length)) {
        
        if ( 0x50 == visca_response1.command[1]) {
          system = visca_response1.command[2];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send VideoSystemInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* VideoSystemInq */


  Result VISCACameraReadNode::DeviceTypeInq(const unsigned char& camera_id,
					    short unsigned int& vender,
					    short unsigned int& modelid,
					    short unsigned int& romversion,
					    unsigned char& socketnum){
    Result retval = FAILURE;
    
    if (send_inquiry_command(camera_id,0x00,0x02, &visca_response1)){
      /* we have no error and the reponse length is correct */
      if ((0 == check_response(&visca_response1,true)) && (10 == visca_response1.length)) {
        
        if ( 0x50 == visca_response1.command[1]) {
          vender     = visca_response1.command[2] << 8 |
            visca_response1.command[3];
          modelid    = visca_response1.command[4] << 8 |
            visca_response1.command[5];
          romversion = visca_response1.command[6] << 8 |
            visca_response1.command[7];
          socketnum  = visca_response1.command[8];
          retval = SUCCESS;
        }
      }
    } else {
      ERROR_STREAM("Can't send DeviceTypeInq inquiry to camera " << int(camera_id) << "." << endl);
    }
    return retval;
  } /* DeviceTypeInq */

}; /* namespace NMM */
