/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "YUVtoRGBConverterNode.hpp"
#include "nmm/utils/utils.hpp"
#include "nmm/base/format/Property.hpp"
#include "nmm/base/memorymgr/Buffer.hpp"
#include "nmm/base/registry/Plugin.hpp"

NMM_REGISTER_PLUGIN(NMM::YUVtoRGBConverterNode, "YUVtoRGBConverterNode")

extern "C" {
  void unpack422rgb32_mmx(char *src, int n, char *dst);
}

namespace NMM {

  YUVtoRGBConverterNode::YUVtoRGBConverterNode(const char* name)
    : GenericConverterNode(name)
  {
    width = 0;
    height = 0;
    allocate();
  }

  Buffer* YUVtoRGBConverterNode::UYVY2RGBA32_MMX(Buffer* in_buffer,const GenericConverterNode* node)
  {
    Buffer *out_buffer = const_cast<GenericConverterNode *>(node)->getNewBuffer(in_buffer->getUsed() * 2, in_buffer);

    if (out_buffer) {
      // unpack422rgb32_mmx(in_buffer->getData(), in_buffer->getUsed() , out_buffer->getData());
      out_buffer->setUsed(in_buffer->getUsed()*2);
    }
    else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in_buffer->release();

    // debug-test-instruction (mlohse, repplix)
    // suspendThread(150);

    return out_buffer;
  }

/** UYVYtoRGB24+8bit padding
  */
  Buffer* YUVtoRGBConverterNode::UYVY2RGBA32(Buffer* in_buffer,const GenericConverterNode* node)
  {
    Buffer *out_buffer = const_cast<GenericConverterNode *>(node)->getNewBuffer(in_buffer->getUsed() * 2, in_buffer);

    if (out_buffer) {
      unsigned char *src = (unsigned char*)in_buffer->getData();
      unsigned char *dst = (unsigned char*)out_buffer->getData();
      unsigned char *end=src+(in_buffer->getUsed());
      int r, g, b;
      int r1, g1, b1;
      int r2, g2, b2;

      while(src!=end) {
	r=(((static_cast<int>(src[0])-128)*0x166E9)>>16);
	g=-(((static_cast<int>(src[2])-128)*0x05819)>>16)-(((static_cast<int>(src[0])-128)*0x0B6D1)>>16);
	b=(((static_cast<int>(src[2])-128)*0x1C5A1)>>16);

	r1=r+src[1];
	g1=g+src[1];
	b1=b+src[1];
	r2=r+src[3];
	g2=g+src[3];
	b2=b+src[3];

	dst[0]=CLIPPED(r1);
	dst[1]=CLIPPED(g1);
	dst[2]=CLIPPED(b1);
	dst[3]=0;
	dst[4]=CLIPPED(r2);
	dst[5]=CLIPPED(g2);
	dst[6]=CLIPPED(b2);
	dst[7]=0;

	src+=4;
	dst+=8;
      }
      out_buffer->setUsed(in_buffer->getUsed()*2);
    }
   /* else {
      NODE_ERROR_STREAM(node,"Could not get out buffer." << endl);
    }*/

    in_buffer->release();
    return out_buffer;
  }


  Buffer* YUVtoRGBConverterNode::UYVY2BGR24(Buffer* in_buffer,const GenericConverterNode* node)
  {
    size_t out_size = 3 * (in_buffer->getUsed() / 2);
    Buffer *out_buffer = const_cast<GenericConverterNode *>(node)->getNewBuffer(out_size, in_buffer);

    if(out_buffer) {
      unsigned char* src = (unsigned char*)in_buffer->getData();
      unsigned char* dst = (unsigned char*)out_buffer->getData();
      unsigned char* end = src + (in_buffer->getUsed());

      int r, g, b;
      int r1, g1, b1;
      int r2, g2, b2;
      int u, v;

      while(src!=end) {
	u = src[0];
	v = src[2];

	r=(((static_cast<int>(u)-128)*0x166E9)>>16);
	g=-(((static_cast<int>(v)-128)*0x05819)>>16)-(((static_cast<int>(u)-128)*0x0B6D1)>>16);
	b=(((static_cast<int>(v)-128)*0x1C5A1)>>16);

	r1=r+src[1];
	g1=g+src[1];
	b1=b+src[1];
	r2=r+src[3];
	g2=g+src[3];
	b2=b+src[3];

	dst[0]=CLIPPED(b1);
	dst[1]=CLIPPED(g1);
	dst[2]=CLIPPED(r1);
	dst[3]=CLIPPED(b2);
	dst[4]=CLIPPED(g2);
	dst[5]=CLIPPED(r2);

	src+=4;
	dst+=6;
      }
      out_buffer->setUsed(out_size);
    }
  /*  else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }*/

    in_buffer->release();
    return out_buffer;
  }

  Buffer* YUVtoRGBConverterNode::UYVY2RGB24(Buffer* in_buffer,const GenericConverterNode* node)
  {
    size_t out_size = 3 * (in_buffer->getUsed() / 2);
    Buffer *out_buffer = const_cast<GenericConverterNode *>(node)->getNewBuffer(out_size, in_buffer);

    if (out_buffer) {
      unsigned char* src = (unsigned char*)in_buffer->getData();
      unsigned char* dst = (unsigned char*)out_buffer->getData();
      unsigned char* end=src+in_buffer->getUsed();

      int r, g, b;
      int r1, g1, b1;
      int r2, g2, b2;
      int u, v;

      while(src!=end) {
	u = src[0];
	v = src[2];

	r=(((static_cast<int>(u)-128)*0x166E9)>>16);
	g=-(((static_cast<int>(v)-128)*0x05819)>>16)-(((static_cast<int>(u)-128)*0x0B6D1)>>16);
	b=(((static_cast<int>(v)-128)*0x1C5A1)>>16);

	r1=r+src[1];
	g1=g+src[1];
	b1=b+src[1];
	r2=r+src[3];
	g2=g+src[3];
	b2=b+src[3];

	dst[0]=CLIPPED(r1);
	dst[1]=CLIPPED(g1);
	dst[2]=CLIPPED(b1);
	dst[3]=CLIPPED(r2);
	dst[4]=CLIPPED(g2);
	dst[5]=CLIPPED(b2);

	src+=4;
	dst+=6;
      }
      out_buffer->setUsed(out_size);
    }
    /*else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }*/

    in_buffer->release();
    return out_buffer;
  }


/** UYVYtoRGB24+8bit padding, planar
  */
  Buffer* YUVtoRGBConverterNode::UYVYplan2BGRA32(Buffer* in_buffer,
                                                 const GenericConverterNode *node) {
    // geht, aber unschoen..., oder?
    Buffer *out =
      const_cast<GenericConverterNode *>(node)->getNewBuffer(in_buffer->getUsed()*2, in_buffer);

    if (out) {
      unsigned char *y   = (unsigned char*)in_buffer->getData();
      unsigned char *u   = (unsigned char*)y + (in_buffer->getUsed())/2;
      unsigned char *v   = (unsigned char*)u + (in_buffer->getUsed())/4;
      unsigned char *dst = (unsigned char*)out->getData();

      int r, g, b;
      int r1, g1, b1;
      int r2, g2, b2;
      int y1, y2, u1, v1;

      for(unsigned int i = 0; i < (in_buffer->getUsed())/4; ++i) { //fix it for other resolution!!!!!!!
        y1 = (int) *(y++);
        y2 = (int) *(y++);
        u1 = (int) *(u++);
        v1 = (int) *(v++);
        
        r = ((u1-128)*0x166E9)>>16;
        g = -((v1-128)*0x05819)>>16 - ((u1-128)*0x0B6D1)>>16;
        b = ((v1-128)*0x1C5A1)>>16;
  
        r1 = r + y1;
        g1 = g + y1;
        b1 = b + y1;
        r2 = r + y2;
        g2 = g + y2;
        b2 = b + y2;

	dst[0]=CLIPPED(b1);
	dst[1]=CLIPPED(g1);
	dst[2]=CLIPPED(r1);
	dst[3]=0;
	dst[4]=CLIPPED(b2);
	dst[5]=CLIPPED(g2);
	dst[6]=CLIPPED(r2);
	dst[7]=0;

        dst+=8;      
      }
      out->setUsed(in_buffer->getUsed()*2);
    }
    else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in_buffer->release();
    return out;    
  }

  /** YV12toRGB24, planar -> packet
   */
  Buffer* YUVtoRGBConverterNode::YV12toRGB24(Buffer* in_buffer, const GenericConverterNode* node) {
    size_t out_size = 6 * (in_buffer->getUsed()/3);
    int width = static_cast<const YUVtoRGBConverterNode*>(node)->width;
    int height = static_cast<const YUVtoRGBConverterNode*>(node)->height;
    Buffer *out = const_cast<GenericConverterNode *>(node)->getNewBuffer(out_size, in_buffer);

    if (out) {
      convertYV12ToRGB24((unsigned char*)out->getData(), (unsigned char*)in_buffer->getData(),
		       width, height, false);
      out->setUsed(out_size);
    }
    else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in_buffer->release();
    return out;    
  }

  /** YV12toRGB32, planar -> packet
   */
  Buffer* YUVtoRGBConverterNode::YV12toRGB32(Buffer* in_buffer, const GenericConverterNode* node) {
    size_t out_size = 8 * (in_buffer->getUsed()/3);
    int width = static_cast<const YUVtoRGBConverterNode*>(node)->width;
    int height = static_cast<const YUVtoRGBConverterNode*>(node)->height;
    Buffer *out = const_cast<GenericConverterNode *>(node)->getNewBuffer(out_size, in_buffer);

    if (out) {
      convertYV12ToRGB32((unsigned char*)out->getData(), (unsigned char*)in_buffer->getData(),
		       width, height, false);
      out->setUsed(out_size);
    }
    else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in_buffer->release();
    return out;    
  }

  Result YUVtoRGBConverterNode::doInitOutput()
  {
    width = getInputFormat()->getIntValue(Format::x_resolution_param);
    height = getInputFormat()->getIntValue(Format::y_resolution_param);

    Format* out_format = 0;
    Format* in_format  = 0;

    // find minimum difference between the two possible input formats
    if(getInputFormat()->getStringValue(Format::colorspace_param) == "yuv422") {
      if(getInputFormat()->getStringValue(Format::channelorder_param) == "uyvy") {
	// copy current input format to working input property
	in_format = getInputFormat()->duplicateFormat();
	getOwnWorkingInputProperty()->addNewFormat(in_format);

	// create io-partner with resolved wildcards
	/* Output format for function UYVY2BGR24 */
	out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
	out_format->addWildcard(Format::x_resolution_param);
	out_format->addWildcard(Format::y_resolution_param);
	out_format->addWildcard(Format::framerate_param);
	out_format->addWildcard(Format::ratio_x_param);
	out_format->addWildcard(Format::ratio_y_param);
	out_format->addStringParamValue(Format::channelorder_param,"bgr");
	out_format->addStringParamValue(Format::endian_param, "little");
	out_format->addStringParamValue(Format::colorspace_param,"rgb24");
	out_format->addIntParamValue(Format::bitperpixel_param,24);
	out_format->addStringParamValue(Format::format_param,"packet");
	out_format->resolveMatchingWildcards(in_format);
	out_format->setIOPartner(in_format);
	in_format ->setIOPartner(out_format);
      
	// copy current input format to working input property
	in_format = getInputFormat()->duplicateFormat();
	getOwnWorkingInputProperty()->addNewFormat(in_format);

	// create io-partner with resolved wildcards
	/* Output format for function UYVY2RGB24 */
	out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
	out_format->addWildcard(Format::x_resolution_param);
	out_format->addWildcard(Format::y_resolution_param);
	out_format->addWildcard(Format::ratio_x_param);
	out_format->addWildcard(Format::ratio_y_param);
	out_format->addWildcard(Format::framerate_param);
	out_format->addStringParamValue(Format::channelorder_param,"rgb");
	out_format->addStringParamValue(Format::endian_param, "little");
	out_format->addStringParamValue(Format::colorspace_param,"rgb24");
	out_format->addIntParamValue(Format::bitperpixel_param,24);
	out_format->addStringParamValue(Format::format_param,"packet");
	out_format->resolveMatchingWildcards(in_format);
	out_format->setIOPartner(in_format);
	in_format ->setIOPartner(out_format);

	// copy current input format to working input property
	in_format = getInputFormat()->duplicateFormat();
	getOwnWorkingInputProperty()->addNewFormat(in_format);


	// create io-partner with resolved wildcards
	out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
	out_format->addWildcard(Format::x_resolution_param);
	out_format->addWildcard(Format::y_resolution_param);
	out_format->addWildcard(Format::ratio_x_param);
	out_format->addWildcard(Format::ratio_y_param);
	out_format->addWildcard(Format::framerate_param);
	out_format->addStringParamValue(Format::channelorder_param,"rgb");
	out_format->addStringParamValue(Format::endian_param, "little");
	out_format->addStringParamValue(Format::colorspace_param,"rgb24");
	out_format->addIntParamValue(Format::bitperpixel_param,32);
	out_format->addStringParamValue(Format::format_param,"packet");
	out_format->resolveMatchingWildcards(in_format);
	out_format->setIOPartner(in_format);
	in_format ->setIOPartner(out_format);
      }
      else if(getInputFormat()->getStringValue(Format::channelorder_param) == "yuv") {
	// copy current input format to working input property
	in_format = getInputFormat()->duplicateFormat();
	getOwnWorkingInputProperty()->addNewFormat(in_format);

	// create io-partner with resolved wildcards
	/* Outut format for function UYVYplan2BGR24 */
	out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
	out_format->addWildcard(Format::x_resolution_param);
	out_format->addWildcard(Format::y_resolution_param);
	out_format->addWildcard(Format::ratio_x_param);
	out_format->addWildcard(Format::ratio_y_param);
	out_format->addWildcard(Format::framerate_param);
	out_format->addStringParamValue(Format::channelorder_param,"bgr");
	out_format->addStringParamValue(Format::endian_param, "little");
	out_format->addStringParamValue(Format::colorspace_param,"rgb24");
	out_format->addIntParamValue(Format::bitperpixel_param,32);
	out_format->addStringParamValue(Format::format_param,"packet");
	out_format->resolveMatchingWildcards(in_format);
	out_format->setIOPartner(in_format);
	in_format ->setIOPartner(out_format);
      }
    }
    else if(getInputFormat()->getStringValue(Format::colorspace_param) == "yv12") {
      // copy current input format to working input property
      in_format = getInputFormat()->duplicateFormat();
      getOwnWorkingInputProperty()->addNewFormat(in_format);

      // create io-partner with resolved wildcards
      /* Outut format for function YV12toRGB24 */
      out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
      out_format->addWildcard(Format::x_resolution_param);
      out_format->addWildcard(Format::y_resolution_param);
      out_format->addWildcard(Format::ratio_x_param);
      out_format->addWildcard(Format::ratio_y_param);
      out_format->addWildcard(Format::framerate_param);
      out_format->addStringParamValue(Format::channelorder_param,"rgb");
      out_format->addStringParamValue(Format::endian_param, "little");
      out_format->addStringParamValue(Format::colorspace_param, "rgb24");
      out_format->addIntParamValue(Format::bitperpixel_param, 24);
      out_format->addStringParamValue(Format::format_param,"packet");
      out_format->resolveMatchingWildcards(in_format);
      out_format->setIOPartner(in_format);
      in_format ->setIOPartner(out_format);

      /* Outut format for function YV12toRGB32 */
      out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
      out_format->addWildcard(Format::x_resolution_param);
      out_format->addWildcard(Format::y_resolution_param);
      out_format->addWildcard(Format::ratio_x_param);
      out_format->addWildcard(Format::ratio_y_param);
      out_format->addWildcard(Format::framerate_param);
      out_format->addStringParamValue(Format::channelorder_param,"rgb");
      out_format->addStringParamValue(Format::endian_param, "little");
      out_format->addStringParamValue(Format::colorspace_param, "rgb24");
      out_format->addIntParamValue(Format::bitperpixel_param, 32);
      out_format->addStringParamValue(Format::format_param,"packet");
      out_format->resolveMatchingWildcards(in_format);
      out_format->setIOPartner(in_format);
      in_format ->setIOPartner(out_format);
    }
    else {
      ERROR_STREAM("Something went wrong..." << endl);
      return FAILURE;
    }

    return SUCCESS;
  }

  void YUVtoRGBConverterNode::allocate()
  {
    Format* in_format  = 0;
    Format* out_format = 0;

    /* Input format for function UYVY2BGR24 */
    in_format = getOwnInputProperty()->addNewFormat("video/raw");
    in_format->addWildcard(Format::x_resolution_param);
    in_format->addWildcard(Format::y_resolution_param);
    in_format->addWildcard(Format::framerate_param);
    in_format->addWildcard(Format::ratio_x_param);
    in_format->addWildcard(Format::ratio_y_param);
    in_format->addStringParamValue(Format::channelorder_param,"uyvy");
    in_format->addStringParamValue(Format::endian_param, "little");
    in_format->addStringParamValue(Format::colorspace_param,"yuv422");
    in_format->addIntParamValue(Format::bitperpixel_param,16);
    in_format->addStringParamValue(Format::format_param,"packet");

    /* Output format for function UYVY2BGR24 */
    out_format = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format->addWildcard(Format::x_resolution_param);
    out_format->addWildcard(Format::y_resolution_param);
    out_format->addWildcard(Format::ratio_x_param);
    out_format->addWildcard(Format::ratio_y_param);
    out_format->addWildcard(Format::framerate_param);
    out_format->addStringParamValue(Format::channelorder_param,"bgr");
    out_format->addStringParamValue(Format::endian_param, "little");
    out_format->addStringParamValue(Format::colorspace_param,"rgb24");
    out_format->addIntParamValue(Format::bitperpixel_param,24);
    out_format->addStringParamValue(Format::format_param,"packet");
    /* Add the converter*/
    addConverter(in_format, out_format, &UYVY2BGR24);

    /* Input format for function UYVY2RGB24 */
    in_format = getOwnInputProperty()->addNewFormat("video/raw");
    in_format->addWildcard(Format::x_resolution_param);
    in_format->addWildcard(Format::y_resolution_param);
    in_format->addWildcard(Format::framerate_param);
    in_format->addWildcard(Format::ratio_x_param);
    in_format->addWildcard(Format::ratio_y_param);
    in_format->addStringParamValue(Format::channelorder_param,"uyvy");
    in_format->addStringParamValue(Format::endian_param, "little");
    in_format->addStringParamValue(Format::colorspace_param,"yuv422");
    in_format->addIntParamValue(Format::bitperpixel_param,16);
    in_format->addStringParamValue(Format::format_param,"packet");

    /* Output format for function UYVY2RGB24 */
    out_format = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format->addWildcard(Format::x_resolution_param);
    out_format->addWildcard(Format::y_resolution_param);
    out_format->addWildcard(Format::framerate_param);
    out_format->addWildcard(Format::ratio_x_param);
    out_format->addWildcard(Format::ratio_y_param);
    out_format->addStringParamValue(Format::channelorder_param,"rgb");
    out_format->addStringParamValue(Format::endian_param, "little");
    out_format->addStringParamValue(Format::colorspace_param,"rgb24");
    out_format->addIntParamValue(Format::bitperpixel_param,24);
    out_format->addStringParamValue(Format::format_param,"packet");
    /* Add the converter*/
    addConverter(in_format, out_format, &UYVY2RGB24);

    /* Input format for function UYVY2RGBA32_MMX */
    in_format = getOwnInputProperty()->addNewFormat("video/raw");
    in_format->addWildcard(Format::x_resolution_param);
    in_format->addWildcard(Format::y_resolution_param);
    in_format->addWildcard(Format::framerate_param);
    in_format->addWildcard(Format::ratio_x_param);
    in_format->addWildcard(Format::ratio_y_param);
    in_format->addStringParamValue(Format::channelorder_param,"uyvy");
    in_format->addStringParamValue(Format::endian_param, "little");
    in_format->addStringParamValue(Format::colorspace_param,"yuv422");
    in_format->addIntParamValue(Format::bitperpixel_param,16);
    in_format->addStringParamValue(Format::format_param,"packet");

    /* Output format for function UYVY2RGBA32_MMX and non-mmx*/
    out_format = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format->addWildcard(Format::x_resolution_param);
    out_format->addWildcard(Format::y_resolution_param);
    out_format->addWildcard(Format::framerate_param);
    out_format->addWildcard(Format::ratio_x_param);
    out_format->addWildcard(Format::ratio_y_param);
    out_format->addStringParamValue(Format::channelorder_param,"rgb");
    out_format->addStringParamValue(Format::endian_param, "little");
    out_format->addStringParamValue(Format::colorspace_param,"rgb24");
    out_format->addIntParamValue(Format::bitperpixel_param,32);
    out_format->addStringParamValue(Format::format_param,"packet");
    /* Add the converter*/
    addConverter(in_format, out_format, &UYVY2RGBA32);
    //addConverter(in_format, out_format, &UYVY2RGBA32_MMX);

    /* Input format for function UYVYplan2BGR24 */
    in_format = getOwnInputProperty()->addNewFormat("video/raw");
    in_format->addWildcard(Format::x_resolution_param);
    in_format->addWildcard(Format::y_resolution_param);
    in_format->addWildcard(Format::framerate_param);
    in_format->addWildcard(Format::ratio_x_param);
    in_format->addWildcard(Format::ratio_y_param);
    in_format->addStringParamValue(Format::channelorder_param,"yuv");
    in_format->addStringParamValue(Format::endian_param, "little");
    in_format->addStringParamValue(Format::colorspace_param,"yuv422");
    in_format->addIntParamValue(Format::bitperpixel_param,16);
    in_format->addStringParamValue(Format::format_param,"planar");

    /* Outut format for function UYVYplan2BGR24 */
    out_format = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format->addWildcard(Format::x_resolution_param);
    out_format->addWildcard(Format::y_resolution_param);
    out_format->addWildcard(Format::framerate_param);
    out_format->addWildcard(Format::ratio_x_param);
    out_format->addWildcard(Format::ratio_y_param);
    out_format->addStringParamValue(Format::channelorder_param,"bgr");
    out_format->addStringParamValue(Format::endian_param, "little");
    out_format->addStringParamValue(Format::colorspace_param,"rgb24");
    out_format->addIntParamValue(Format::bitperpixel_param,32);
    out_format->addStringParamValue(Format::format_param,"packet");
    /* Add the converter*/
    addConverter(in_format, out_format, &UYVYplan2BGRA32);

    /* Input format for function YV12toRGB24 */
    in_format = getOwnInputProperty()->addNewFormat("video/raw");
    in_format->addWildcard(Format::x_resolution_param);
    in_format->addWildcard(Format::y_resolution_param);
    in_format->addWildcard(Format::framerate_param);
    in_format->addWildcard(Format::ratio_x_param);
    in_format->addWildcard(Format::ratio_y_param);
    in_format->addStringParamValue(Format::channelorder_param,"yvu");
    in_format->addStringParamValue(Format::endian_param, "little");
    in_format->addStringParamValue(Format::colorspace_param,"yv12");
    in_format->addIntParamValue(Format::bitperpixel_param,12);
    in_format->addStringParamValue(Format::format_param,"planar");

    /* Outut format for function YV12toRGB24 */
    out_format = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format->addWildcard(Format::x_resolution_param);
    out_format->addWildcard(Format::y_resolution_param);
    out_format->addWildcard(Format::framerate_param);
    out_format->addWildcard(Format::ratio_x_param);
    out_format->addWildcard(Format::ratio_y_param);
    out_format->addStringParamValue(Format::channelorder_param,"rgb");
    out_format->addStringParamValue(Format::endian_param, "little");
    out_format->addStringParamValue(Format::colorspace_param,"rgb24");
    out_format->addIntParamValue(Format::bitperpixel_param,24);
    out_format->addStringParamValue(Format::format_param,"packet");
    /* Add the converter*/
    addConverter(in_format, out_format, &YV12toRGB24);

    /* Outut format for function YV12toRGB32 */
    out_format = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format->addWildcard(Format::x_resolution_param);
    out_format->addWildcard(Format::y_resolution_param);
    out_format->addWildcard(Format::framerate_param);
    out_format->addWildcard(Format::ratio_x_param);
    out_format->addWildcard(Format::ratio_y_param);
    out_format->addStringParamValue(Format::channelorder_param,"rgb");
    out_format->addStringParamValue(Format::endian_param, "little");
    out_format->addStringParamValue(Format::colorspace_param,"rgb24");
    out_format->addIntParamValue(Format::bitperpixel_param,32);
    out_format->addStringParamValue(Format::format_param,"packet");
    /* Add the converter*/
    addConverter(in_format, out_format, &YV12toRGB32);
  }
}
