/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_YUVtoRGBCONVERTERNODE_HPP_
#define _NMM_YUVtoRGBCONVERTERNODE_HPP_

#include "nmm/base/GenericConverterNode.hpp"

#define CLIPPED(x) ((x)>255?255:((x)<0?0:(x)))

namespace NMM {
 /** \class YUVtoRGBConverterNode
   * \ingroup VideoConverter
   * This Node converts an incoming YUV image to a BGR0 image
   * (blue, green, red and ignored channel, in this order).
   */
class YUVtoRGBConverterNode : public GenericConverterNode {

public:

  YUVtoRGBConverterNode(const char* name = "YUVtoRGBConverterNode");
  virtual ~YUVtoRGBConverterNode() {};


protected:


  /* YUV422 (in UYVY-format) converter routines*/
  static Buffer* UYVY2BGR24 (Buffer* in_buffer, const GenericConverterNode*);
  static Buffer* UYVY2RGB24 (Buffer* in_buffer, const GenericConverterNode*);
  static Buffer* UYVY2RGBA32(Buffer* in_buffer, const GenericConverterNode*);
  static Buffer* UYVY2RGBA32_MMX(Buffer* in_buffer,const GenericConverterNode* node);
  static Buffer* UYVYplan2BGRA32(Buffer* in_buffer, const GenericConverterNode*);
  static Buffer* YV12toRGB24(Buffer* in_buffer, const GenericConverterNode*);
  static Buffer* YV12toRGB32(Buffer* in_buffer, const GenericConverterNode*);

  virtual Result doInitOutput();

private:
  int width, height;
  
  void allocate();
  static inline int clipTo_0_255(int x) { return ((x)>255?255:((x)<0?0:(x))); }
};

}

#endif //_NMM_YUV2BGR0NODE_HPP_
