/* NMM - Network-Integrated Multimedia Environment
 *
 * Copyright (C) 2002 Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Markus Sand <msand@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "yv12_deinterlace.h"

inline void line_average(unsigned char *row1, unsigned char *row2, unsigned char *out, int bytes)
{
  // calculate the average of each color entry in two arrays and return
  // result in char *out
  int y;
  unsigned int tmp;
  
  for (y = 0; y<bytes; ++y) {
    tmp = ((unsigned int) row2[y] + (unsigned int) row1[y]) >>1;
    out[y] = (unsigned char) (tmp & 0xff);
  }
  return;
}

void yv12_deinterlace(char *image, unsigned int width_, unsigned int height_)
{
  unsigned char *in, *out;
  int y, block; 
  
  block = width_;
  in  = (unsigned char*) image;
  out = (unsigned char*) image;
  
  // convert half frame to full frame by simple interpolation
  out +=block;
  
  for (y = 0; y < (height_>>1)-1; y++) {
    line_average(in, in+(block<<1), out, block);
    in  += block<<1;
    out += block<<1;
  }
  
  if (!(height_ & 1)) {
    // clone last row
    memcpy(out, in, block);
  }
  
  return;
}
