/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "OSDElement.hpp"

namespace NMM {
  
  OSDElement::OSDElement(IOSDManager* _manager, IWidgetFactory* _factory) {
    manager.reset(dynamic_cast<IOSDManager*>(_manager->copy()));
    factory.reset(dynamic_cast<IWidgetFactory*>(_factory->copy()));
    
    win.reset( manager->createWindow());
  }
  
  OSDElement::~OSDElement() {
    if ( win.get() ) {
      manager->deleteWindow( *win );
      win.reset(NULL);
    }
  }
  
  IWindow* OSDElement::getWindow() {
    return win.get();
  }
  
  void OSDElement::show() {
    win->setVisible(true);
  }
  
  void OSDElement::hide() {
    win->setVisible(false);
  }
  
  bool OSDElement::isHidden() {
    return( !win->getVisible() );
  }
  
  void OSDElement::setDimension( unsigned int w, unsigned int h ) {
    width = w;
    height = h;
    win->setDimension( w, h );
  }
  
  void OSDElement::setPosition( unsigned int x, unsigned int y ) {
    win->setPosition( x, y );
  }
}
