/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "flaglistunit.hpp"

namespace NMM {
  
  FlagListUnit::FlagListUnit( IOSDManager* manager, IWidgetFactory* factory, string _font, bool withScrollbar) 
    : OSDElement(manager, factory),
      font(_font) {
    
    columnWidth = 100;

    setColors( 0xff000000, 0xff100000, 0xff707070, 0xff303030, 0xff101010 );
    
    composite.reset(factory->createWidgetComposite());
    text.reset(factory->createTextView());
    text->setFont(font);
    text->setSelectionColor( sel_inactive_color );
    if(!withScrollbar)
      composite->add(*text);
    
    flag.reset(factory->createTextView());
    flag->setFont(font);
    flag->setSelectionColor( sel_inactive_color );
    composite->add(*flag);

    if(withScrollbar){
      scroll.reset(factory->createScrollDecorator( *text ));
      composite->add(*scroll);
    }else{
    }
    plane.reset(factory->createPlaneDecorator( *composite ));
    plane->setPlaneColor( win_inactive_color );

    border.reset(factory->createBorderDecorator( *plane ));

    header.reset(factory->createHeaderDecorator( *border ));
    header->setFont(font);

    win->setContent( *header );
  }
  
  FlagListUnit::~FlagListUnit() {
    deleteWidgets();
  }
  
  void FlagListUnit::deleteWidgets() {
    if ( text.get() ) {
      manager->deleteWidget(*text);
      text.reset(NULL);
    }

    if ( flag.get() ) {
      manager->deleteWidget(*flag);
      flag.reset(NULL);
    }

    if ( composite.get() ) {
      manager->deleteWidget(*composite);
      composite.reset(NULL);
    }
  
    if ( plane.get() ) {
      manager->deleteWidget(*plane);
      plane.reset(NULL);
    }

    if ( border.get() ) {
      manager->deleteWidget(*border);
      border.reset(NULL);
    }

    if ( scroll.get() ) {
      manager->deleteWidget(*scroll);
      scroll.reset(NULL);
    }

    if ( header.get() ) {
      manager->deleteWidget(*header);
      header.reset(NULL);
    }
  }
  
  void FlagListUnit::setColors( unsigned int _win_active_color, unsigned int _win_inactive_color,
                            unsigned int _sel_active_color, unsigned int _sel_active_color2, unsigned int _sel_inactive_color ) {

    win_active_color   = _win_active_color;
    win_inactive_color = _win_inactive_color;
    sel_active_color   = _sel_active_color;
    sel_active_color2  = _sel_active_color2;
    sel_inactive_color = _sel_inactive_color;
  }
  
  void FlagListUnit::selectWindow( unsigned int mode ) {
    if ( mode == 1 ) {
      text->setSelectionColor( sel_active_color2 );
      flag->setSelectionColor( sel_active_color2 );
    } else {
      text->setSelectionColor( sel_active_color );
      flag->setSelectionColor( sel_active_color );
    }
    plane->setPlaneColor( win_active_color );
  }

  void FlagListUnit::deSelectWindow() {
    text->setSelectionColor( sel_inactive_color );
    flag->setSelectionColor( sel_inactive_color );
    plane->setPlaneColor( win_inactive_color );
  }
  
  void FlagListUnit::setHeader( string h ) {
    header->setText( h );
  }
  
  void FlagListUnit::setFirstColumnWidth( unsigned int w ){
    columnWidth = w;
  }


  string FlagListUnit::getLineURL(const unsigned int i) {
    return text->getLineURL(i);
  }

  void FlagListUnit::selectLine(const unsigned int& i){
      text->selectLine(i);
      flag->selectLine(i);
  }

  string FlagListUnit::getLineContent(const unsigned int i){
    return text->getLineContent(i);
  }

  void FlagListUnit::deleteAllLines(){
    selectLine(0);
    update();
    text->deleteAllLines();
    flag->deleteAllLines();
  }

  unsigned int FlagListUnit::addLine(string c, string f, string url){
    unsigned int w, h, w1;
    win->getDimension(w,h);

    w1 = (w * columnWidth)/100;

    flag->setDimension(w-w1, h);
    flag->setPosition(w1+5, 0);

    text->setDimension(w1,h);

    flag->addLine(f);
    
    int ret = text->addLine(c,url);
    return ret;
  }
  
  unsigned int FlagListUnit::addLine(string c, string url){
    return addLine(c, "", url);    
  }
  
  unsigned int FlagListUnit::addLine(string c){
    return addLine(c, "", "");
  }
  
  void FlagListUnit::deSelectLine() {
    text->deSelectLine();
    flag->deSelectLine();
  }

  unsigned int FlagListUnit::getSelectedLine(){
    return text->getSelectedLine();
  }
  
  Result FlagListUnit::selectUp(){
    if(text->getSelectedLine() > 0){
      text->selectUp();
      flag->selectUp();
      
      return SUCCESS;
    }
    return FAILURE;
  }

  Result FlagListUnit::selectDown(){
    unsigned int l = text->getNumLines();
    
    if(text->getSelectedLine() < l){
      text->selectDown();
      flag->selectDown();
      
      return SUCCESS;
    }
    return FAILURE;
  }
}
