/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_LISTUNIT_HPP_
#define _NMM_LISTUNIT_HPP_ _NMM_LISTUNIT_HPP_

#include "OSDElement.hpp"

#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/video/osd/manager/IPlaneDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IBorderDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IScrollDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IHeaderDecorator.hpp"

namespace NMM {
  
  class ListUnit : public OSDElement {
  public:
    
    ListUnit(IOSDManager* manager, IWidgetFactory* factory, string font);
    ~ListUnit();
    
    ITextView* getTextWidget() { return( text.get() ); };
    
    void setColors( unsigned int win_active_color, unsigned int win_inactive_color,
                    unsigned int sel_active_color, unsigned int sel_active_color2, unsigned int sel_inactive_color );
    
    void selectWindow( unsigned int mode=0 );
    void deSelectWindow();
    void setHeader( string h );

    Result selectUp();
    Result selectDown();

    unsigned int getSelectedLine();
    string getLineURL(const unsigned int i);
    void selectLine(const unsigned int& i);
    string getLineContent(int i);
    void deleteAllLines();
    int addLine(string c, string url);
    int addLine(string c);
    void addMultiLine(string c);
    void deSelectLine();

    void markLine(unsigned int line);
    void unmarkLine(unsigned int line);
    list<unsigned int> getMarkedLines();

  protected:
    unsigned int  win_active_color;
    unsigned int  win_inactive_color;
    unsigned int  sel_active_color;
    unsigned int  sel_active_color2;
    unsigned int  sel_inactive_color;
    
    void deleteWidgets();
    
    string            font;
    
    ITextView_var     text;
    
    IPlaneDecorator_var   plane;
    IBorderDecorator_var  border;
    IScrollDecorator_var  scroll;
    IHeaderDecorator_var  header;
  };

}

#endif //_NMM_LISTUNIT_HPP_
