/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "progressbarunit.hpp"

namespace NMM {
  ProgressbarUnit::ProgressbarUnit( IOSDManager* manager, IWidgetFactory* factory, string _font ) : OSDElement(manager, factory),
                                                                                                    font(_font) {
    bar.reset(factory->createProgressBar());
    plane.reset(factory->createPlaneDecorator( *bar ));
    border.reset(factory->createBorderDecorator( *plane ));
    header.reset(factory->createHeaderDecorator( *border ));
    header->setFont(font);
    
    setColors( 0x80ff0000, 0xffffffff );
    setBarSpaces( 0, 8 );
    win->setDimension( 620, 44 );
    win->setContent( *header );
  }
  
  ProgressbarUnit::~ProgressbarUnit() {
    deleteWidgets();
  }
  
  void ProgressbarUnit::setMaxProgress( u_int64_t max ) {
    bar->setMaxProgress( max );
  }
  
  void ProgressbarUnit::setProgress( u_int64_t progress ) {
    bar->setProgress( progress );
  }
  
  void ProgressbarUnit::setBarSpaces( unsigned int space, unsigned int space_step ) {
    bar->setBarSpaces( space, space_step );
  }
  
  void ProgressbarUnit::setHeader( string h ) {
    header->setText( h );
  }
  
  void ProgressbarUnit::setColors( unsigned int color1, unsigned int color2 ) {
    plane->setPlaneColor( color1 );
    bar->setBarColor( color2 );
  }

  void ProgressbarUnit::setWidth(unsigned int width) {
    win->setDimension(width, 44);
  }
  
  void ProgressbarUnit::deleteWidgets() {
    if ( bar.get() ) {
      manager->deleteWidget(*bar);
      bar.reset(NULL);
    }
    
    if ( plane.get() ) {
      manager->deleteWidget(*plane);
      plane.reset(NULL);
    }
    
    if ( border.get() ) {
      manager->deleteWidget(*border);
      border.reset(NULL);
    }
    
    if ( header.get() ) {
      manager->deleteWidget(*header);
      header.reset(NULL);
    }
  }
}
