/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "scroll.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* ScrollDecorator									 */
  /*****************************************************************************************/
  ScrollDecorator::ScrollDecorator() {
    setBarWidth( 20 );
    setBackgroundColor( 0xffa0a0a0 );
    setBorderColor( 0xff4e4e4e );
    setBarColor( 0xffe1e1e1 );
    setBarBorderColors( 0xffffffff, 0xffb7b7b7 );
  }
  
  void ScrollDecorator::setBarWidth( const unsigned int& bw ) {
    barwidth = bw;
  }
  
  void ScrollDecorator::setBackgroundColor( const u_int32_t& bcolor ) {
    backgroundcolor = bcolor;
  }
  
  void ScrollDecorator::setBorderColor( const u_int32_t& bcolor ) {
    bordercolor = bcolor;
  }
  
  void ScrollDecorator::setBarColor( const u_int32_t& bcolor ) {
    barcolor = bcolor;
  }
  
  void ScrollDecorator::setBarBorderColors( const u_int32_t& bcolor1, const u_int32_t& bcolor2 ) {
    barbordercolor1 = bcolor1;
    barbordercolor2 = bcolor2;
  }
  
  void ScrollDecorator::draw( workspace_t* ws ) {
    workspace_t workspace;
    unsigned int w, h;

    /* we must adjust the workspace to be sure that the object dosn't overwrite the scrollbar */
    workspace.data   = ws->data;

    if ( width > 0 ) w = xpos+width;
    else w = ws->width;

    if ( w > ws->width ) w = ws->width;
    workspace.width  = w  - barwidth;

    if ( height > 0 ) h = ypos+height;
    else height = ws->height;

    if ( h > ws->height ) h = ws->height;
    workspace.height = h;

    workspace.stride = ws->stride;

    Decorator::draw( &workspace );

    drawScrollBar( ws );
  }

  void ScrollDecorator::drawFilledBox( unsigned int* frame, unsigned int stride,
				       unsigned int xs, unsigned int ys,
				       unsigned int width, unsigned int height,
				       unsigned int color ) {

    for( unsigned int y=ys; y < (ys+height); y++ ) {
      for( unsigned int x=xs; x < (xs+width); x++ ) {
	frame[x+y*stride]  = color;
      }
    }

  }

  void ScrollDecorator::drawBox( unsigned int* frame, unsigned int stride,
				 unsigned int xs, unsigned int ys,
				 unsigned int width, unsigned int height,
				 unsigned int color1, unsigned int color2 ) {
    unsigned int  x, y;
    unsigned int  xd, yd;

    xd = xs+width-1;
    yd = ys+height-1;

    for(x=xs; x <= xd; x++) {
      frame[x+ys*stride]     = color1;
      frame[x+(ys+1)*stride] = color1;

      frame[x+(yd*stride)]     = color2;
      frame[x+((yd-1)*stride)] = color2;
    }

    for(y=ys+1; y <= yd; y++) {
      frame[xs+(y*stride)]   = color1;
      frame[xs+1+(y*stride)] = color1;

      frame[xd+(y*stride)]   = color2;
      frame[xd-1+(y*stride)] = color2;
    }

  }


  void ScrollDecorator::drawScrollBar( workspace_t* ws ) {

    unsigned int v_width, v_height;
    unsigned int sh, sy;
    v_width  = adjustDimension( xpos, width, ws->width );
    v_height = adjustDimension( ypos, height, ws->height );

    /* get scrollbar height information from widget and adjust it to the scrollbar height */
    sh = (vcomponent->getScrollBarHeight() * v_height)/10000;

    /* get scrollbar position information from widget and adjust it */
    sy = (vcomponent->getScrollBarYPosition() * v_height)/10000 + ypos;
    /* don't set it to 0 */
    if (sy==0) sy=1;

    /* draw the scrollbar background */
    drawFilledBox( (unsigned int*) ws->data, ws->stride, xpos+v_width-barwidth, ypos,
		   barwidth, v_height, backgroundcolor );

    drawBox( (unsigned int*) ws->data, ws->stride, xpos+v_width-barwidth, ypos,
	     barwidth, v_height, bordercolor, bordercolor );

    drawFilledBox( (unsigned int*) ws->data, ws->stride, xpos+v_width-barwidth+2, sy,
		   barwidth-4, sh, barcolor );

    drawBox( (unsigned int*) ws->data, ws->stride, xpos+v_width-barwidth+2, sy,
	     barwidth-4, sh, barbordercolor1, barbordercolor2  );


  }

}
