/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_WGBUTTONS_HPP_
#define _NMM_WGBUTTONS_HPP_ _NMM_WGBUTTONS_HPP_

#include "imageview.hpp"

#include "nmm/interfaces/video/osd/manager/IButtonsImpl.hpp"

namespace NMM {
  
  /**
   * Buttons widget.
   */
  class Buttons : virtual public ImageView,
                  virtual public IButtonsImpl {
    
    typedef enum { INACTIVE=0, ACTIVE, SELECTED } buttonstate_t;
    typedef struct {
      string        label;     // additional description of the button
      string        id[3];	 // Bitmap identifiers fo INACTIVE, ACTIVE and SELECTED state
      int           x;	 // x position of upper left corner
      int           y;	 // y position of upper left corner
      buttonstate_t state;	 // actual state (INACTIVE or ACTIVE)
    } button_t;

    typedef map<int, button_t> buttonlist_t;

  protected:
    buttonlist_t           buttonlist;	         // stores all Buttons
    int                    selected_button;	 // id of selected button
    unsigned int	   step;		 // number of buttons to get into next line

  public:
    Buttons();
    virtual ~Buttons();
    
    /** IButtons interface. */
    virtual void selectRight();
    virtual void selectLeft();
    virtual void selectUp();
    virtual void selectDown();
    virtual void selectPush();
    virtual void addButton(const int& buttonid, const string& label, const int& x, const int& y,
                           const string& inactive_id, const string& active_id, const string& select_id);
    virtual string getButtonLabel(const int& button_id);
    virtual string getSelectedButtonLabel();
    virtual bool getButtonState(const int& button_id);
    virtual void deleteButton(const int& button_id);
    virtual void deleteAllButtons();
    virtual int selectButton(const int& buttonid);
    virtual void setStep(const unsigned int& value);
    virtual void deselectButton();
    virtual void activateButton(const int& button_id);
    virtual void deactivateButton(const int& button_id);
    virtual void deactivateAllButtons();
    
    void draw( workspace_t* ws );
  }; // Buttons
}

#endif //_NMM_WGBUTTONS_HPP_
