/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "timeview.hpp"
#include "../OSDManagerNode.hpp"

#include "nmm/misc/overlay/overlay.h"

namespace NMM {

  /*****************************************************************************************/
  /* TimeView                                                                              */
  /*****************************************************************************************/

  TimeView::TimeView( BitmapFont* bfont ) {
    font = bfont;
    
    reset();
    setPrefix( "" );
  }
  
  TimeView::~TimeView() {
  }
  
  void TimeView::setFont( const string& fontname ) {
    font = manager->getFont( fontname );
  }
  
  void TimeView::reset() {
    current_time = 0;
    setTimeOffset( 0 );
  }
  
  void TimeView::setPrefix( const string& p ) {
    prefix = p;
  }

  void TimeView::setTimeOffset( const int& to ) {
    timeoffset = to;
  }

  void TimeView::setCurrentTime( const int& t ) {
    current_time = t;
  }

  int TimeView::getCurrentTime() {
    return current_time;
  }
  
  
  void TimeView::draw( workspace_t* ws ) {
    unsigned int v_width, v_height;
    unsigned int h, m, s;
    long         sec;
    char         timestr[200];
    
    v_width  = adjustDimension( xpos, width, ws->width );
    v_height = adjustDimension( ypos, height, ws->height );
    
    unsigned int* frame = (unsigned int*) ws->data;
    
    sec = current_time + timeoffset;
    h   = sec/3600;
    
    sec = sec - 3600*h;
    m   = sec/60;

    s   = sec - 60*m;

    sprintf( timestr, "%s%02d:%02d:%02d", prefix.c_str(), h, m, s );
    textdraw.draw( timestr, font, frame, ws->stride,
		   xpos, ypos,
		   v_width+xpos, v_height+ypos );

  }

}
