/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_WGTIMEVIEW_HPP_
#define _NMM_WGTIMEVIEW_HPP_ _NMM_WGTIMEVIEW_HPP_

#include "widgets.hpp"
#include "textdraw.hpp"

#include "nmm/interfaces/video/osd/manager/ITimeViewImpl.hpp"

namespace NMM {
  
  /**
   * Time view widget.
   * Displays a time.
   */
  class TimeView : virtual public Widget,
                   virtual public ITimeViewImpl {
    
  protected:
    /** font that is used to draw the text */
    BitmapFont*    font;
    
    /** class that contains some functions to draw text */
    TextDraw       textdraw;
    
    /** text that should be placed in front of the time */
    string prefix;
    
    /** offset that is added to the time */
    long   timeoffset;
    
    time_t current_time;
    
  public:
    TimeView( BitmapFont* bfont = NULL );
    virtual ~TimeView();
    
    /** ITimeView interface. */
    void setFont(const string& value);
    void setPrefix(const string& value);
    void setTimeOffset(const int& value);
    void setCurrentTime(const int& value);
    int getCurrentTime();
    void reset();
    
    void draw( workspace_t* ws );
  }; // TextView

}

#endif //_NMM_WGTIMEVIEW_HPP_
