/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:	       
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_SPUDECODENODE_HPP
#define NMM_SPUDECODENODE_HPP NMM_SPUDECODENODE_HPP

#include "nmm/base/GenericProcessorNode.hpp"
#include "nmm/interfaces/video/spudec/ISPUDecoderImpl.hpp"

#include "SPUClass.hpp"

namespace NMM {
  
  /**
   * Subtitle Picture Unit (SPU) Decoder node.
   * This node decodes subtitle picture unit streams into raw video streams.
   * Palette and button information is provided through instream events.
   */
  class SPUDecodeNode
    : public GenericProcessorNode
    , public ISPUDecoderImpl
  {
    typedef struct {
      unsigned int x_start;
      unsigned int y_start;
      unsigned int x_end;
      unsigned int y_end;
      unsigned int scheme;
    } button_t;
    
  private:
    bool           send_lastbuffer;
    bool           w_flag;
    Buffer*        out;
    unsigned int   std_palette[16];
    unsigned int*  palette;
    SPUDecoder*    decoder;
    SPUDecoder*    menu_decoder;
    unsigned int   offset;
    Buffer*        old_in_buffer;
    unsigned char  temp[0x10000];
    unsigned char  menu[0x10000];
    unsigned char* data;
    unsigned int   size;
    unsigned int   width;
    unsigned int   height;
    unsigned int   x_offset;
    unsigned int   y_offset;
    
    bool           saved_working_flag;
    bool           send_button_info;
    bool           send_menu;
    bool           have_menu;
    unsigned int   button_index;
    unsigned int   scheme_index;
    unsigned int   button_state;
    unsigned int   selected_button;
    button_t       button[32];
    
    ThreadMutex    event_mutex;
    
    void setPalette( unsigned int* palette );
    
  protected:
    virtual Result doInit();
    virtual Result doDeinit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    
    virtual Message* processBuffer( Buffer* in_buffer );
    
    // ISPUDecoderImpl
    virtual Result setPaletteItem(const int& index,
                                  const int& color_r, const int& color_g, const int& color_b, const int& alpha);
    virtual Result initColorScheme(const int& scheme_nr, const int& selection, const int& activation);
    virtual Result setDVDSelectedButton(const int& button);
    virtual Result setDVDButtonCount(const int& buttons);
    virtual Result setDVDButtonCoords(const int& start_x, const int& start_y, const int& end_x, const int& end_y);
    virtual Result setDVDButtonScheme(const int& scheme);
    
  public:
    SPUDecodeNode( const char* = "SPUDecodeNode", StreamQueue::Mode = StreamQueue::MODE_SUSPEND, int = 20);
    ~SPUDecodeNode();
  };

}

#endif //NMM_SPUDECODENODE_HPP

