/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *                     
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef __NMMCONFIG_H__
#define __NMMCONFIG_H__ __NMMCONFIG_H__

#include <map>
#include <string>
#include <list>
#include <iomanip>
#include <fstream>

using std::string;
using std::map;
using std::list;

extern "C" {
#include <stdlib.h>
}

namespace NMM { 

  class NMMConfig {
  
  public:
    class Setting {
    private:
      string desc;
      list<string> values;
    
    public:
      Setting(const string& d = string(""), const string& v = string(""));
    
      const string& getDescription() const {
	return desc;
      }
    
      const list<string>& getValues() const {
	return values;
      }
    };  
  
    NMMConfig();
    NMMConfig(const string);
    ~NMMConfig();
  
    bool getValue(const string id, string& value);
    bool getValue(const string id, bool& value);
    bool getValue(const string id, int& value);
    bool getValue(const string id, float& value);
  
    void setValue( string id, string value );
  
    void saveFile();
  
    bool getSetting(const string& id, const Setting*& setting) const;
  
    void addSetting(const string& d = string(""), const string& s = string(""), const string& v = string(""));
  
    void parseFile(const string);
  protected:
    map<string,string> config_table;
    map<string,Setting> settings;
  
    string rc_file;
  };

}
#endif //__MMBOXCONFIG_H__
