/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef SECURITYCHECK_HPP
#define SECURITYCHECK_HPP SECURITYCHECK_HPP

#include "NMMConfig.hpp"
#include "Singleton.hpp"
#include "NamedObject.hpp"

namespace NMM {

  class SecurityCheck : public NamedObject {
  public:
    SecurityCheck();
    ~SecurityCheck() {};

    /**
     * Check if filename for read permission.
     * This method checks if an NMM plugin is allowed to read from the given file.
     * \return true when given filename (may include full path) passes security checks
     * \param dir_plus_filename the filename to check
     */
    bool checkFilenameForReading(const string& dir_plus_filename);

    /**
     * Check if filename for write permission
     * This method checks if an NMM plugin is allowed to write to the given file.
     * \return true when given filename (may include full path) passes security checks
     * \param dir_plus_filename the filename to check
     */
    bool checkFilenameForWriting(const string& dir_plus_filename);

    /**
     * \return the password as set in this environment; needed for UserInfo
     */
    const string getPasswd();

    /**
     * \return true if the given password matches the password of this environment
     */
    bool checkPasswd(const string& passwd);

    /**
     * Reads a new config file 
     */
    void parseFile(const string& filename);
    
  private:
    NMMConfig nmmconfig;
  };

  typedef Singleton<SecurityCheck, CreateUsingNew > SSecurityCheck; 
}
#endif
