/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "WebServer.hpp"

#include <iostream>

namespace NMM{
  WebServer::WebServer(unsigned int port)
    :serverPort(port),
     clientHandlers(3),
     allowAll(false),
   running(true)
  {
    
    //avoid setting a privileged port
    if(serverPort <= 1024 || serverPort > 65535){
      serverPort = 8090;
      cerr << "ERROR: port " << serverPort << "is reserved. setting port to 8090" << endl; 
    }
    

    INetAddr adress;
    adress.setPort(port);
    adress.setTransportProtocol("tcp");
    
    server = new SockAcceptor();
    server->open(adress, "tcp");
  }
  
  WebServer::~WebServer(){
    stop();
    if(server) delete server;
  }
  
  void WebServer::start(){
    GLOBAL_DEBUG_STREAM("starting WebServer" << endl);
    for(int i = 0; i < clientHandlers; i++){
      WebServerProcessor *wspObj = new WebServerProcessor(i);
      handlerThreads.push_back(wspObj);
      wspObj->start();
    }
    
    activateThread();
  }
  
  void WebServer::stop(){
    running = false;
    deactivateThread();
    while(!handlerThreads.empty()){
      WebServerProcessor *wspObj = handlerThreads.front();
      handlerThreads.pop_front();
      if(wspObj) delete wspObj;
    }

  }
  
  void WebServer::run(){

    while(running){
      try{
        Socket *sock = new Socket();
        server->accept(*sock);
        WebServerProcessor::processRequest(sock);
      }catch(const Exception &e){}
    }
    server->close();
  }
  
  void WebServer::setNumberOfClientHandlers(int number){
    clientHandlers = number;
  }
  int WebServer::getNumberOfClientHandlers(){
    return clientHandlers;
  }
  void WebServer::enableAllowAll(){
    allowAll = true;
  }
  void WebServer::disableAllowAll(){
    allowAll = false;
  }
  bool WebServer::getAllowAll(){
    return allowAll;
  }
  
  void WebServer::registerHandler(WebRequestHandler *handler){
    WebServerProcessor::registerHandler(handler);
  }
}
