/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "WebServerRequest.hpp"

#include <sstream>

namespace NMM{

  WebServerRequest::WebServerRequest(Socket *clientSocket)
    :buffer_size(1000),
     mMethod(""),
     mProtocol(""),
     mRequestURI(""),
     mQueryString(""),
     mRemoteAddr("")
  {
    char buffer[buffer_size];
    size_t len = clientSocket->recv(buffer, buffer_size,0);
    buffer[len] = '\0';

    string tmpAddr = clientSocket->getPeerAddress().toString();
    mRemoteAddr = tmpAddr.substr(0, tmpAddr.find_first_of(':'));
    
    stringstream s(buffer);
    //if(s.getCount() != 3) throw Exception("Incorrect HTTP Request");
    
    //get Request-Type
    s >> mMethod;
    
    string fullRequest;
    s >> fullRequest;
    
    if(fullRequest[0] == '/'){
      fullRequest.erase(0,1);
    }
    
    size_t queryStringSeperator = fullRequest.find_first_of('?');
    if(queryStringSeperator == string::npos){
      mRequestURI = fullRequest;
    }else{
      mRequestURI  = fullRequest.substr(0,queryStringSeperator);
      mQueryString = fullRequest.substr(queryStringSeperator + 1);
    }
    s >> mProtocol;
    
  }
  
  WebServerRequest::~WebServerRequest(){}
  string WebServerRequest::getMethod(){
    return mMethod;
  }
  string WebServerRequest::getProtocol(){
    return mProtocol;
  }
  string WebServerRequest::getRequestURI(){
    return mRequestURI;
  }
  string WebServerRequest::getQueryString(){
    return mQueryString;
  }
  string WebServerRequest::getRemoteAddress(){
    return mRemoteAddr;
  }
}
