/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Reppliner <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef _EVENTRECORDER_HPP_
#define _EVENTRECORDER_HPP_

#include "nmm/base/IReceiver.hpp"
#include "nmm/base/sync/TimedElement.hpp"
#include <string>

class ostream;

namespace NMM {

class OValueStream;
class StdOStream;

/** An instance of this class can be used to record events and write them into a file.
  * The file can be read from an EventPlayer which reconsturct the event and send them to an object.
  */
class EventRecorder : public IEventReceiver, public TimedElement {
public:
/** Constructor
  * \param output_file name of the output file
  * \param ov OValueStream that should be used for event serialization.
  */
  EventRecorder(const string& output_file);

/** Constructor
  * \param output_file name of the output file
  * \param ov OValueStream that should be used for event serialization.
  */
  EventRecorder(const string& output_file, const OValueStream& ov);

/** Destructor
  */
  virtual ~EventRecorder();

/** Implementation of the IEventReceiver interface to receive events
  */
  virtual Result receiveEvent(const CEvent&, const IEventSender* = 0 );

/** Implementation of the IEventReceiver interface to receive events
  */
  virtual Result receiveEvent(const Event&, const IEventSender* = 0 ) ;

/** Implementation of the IEventReceiver interface to receive events
  */
  virtual Result receiveEvent(const CEvent&, const IEventSender* = 0 ) const;

/** Implementation of the IEventReceiver interface to receive events
  */
  virtual Result receiveEvent(const Event&, const IEventSender* = 0 )  const;

private:
  Result writeValue(CEvent&) const;

private: //member
  OValueStream* __value_stream;     ///used OValueStream to serialize the event
  StdOStream*   __std_o_stream;     ///wrapper for the std::ostream, required to set the ostream as partner of the OValueStream.
  std::ostream* __o_stream;         ///std::ostream to write the data into a file
};



}

#endif

