#include "IClicValueStream.hpp"
#include "nmm/base/Event.hpp"

#include <ctype.h>

namespace NMM {

  IClicValueStream::IClicValueStream(const string& event_key, const vector<string>& argument_stack)
    : __current_pos(0), __event_key(event_key), __argument_stack(argument_stack)
 {
//    size_t pos = event_string.find("(", 0);
//    __event_key = event_string.substr(0, pos);
//
//    size_t start_pos = pos + 1; // skip the '('
//
//    size_t end_pos   = event_string.find(")", start_pos);
//
//    string argument_string = event_string.substr(start_pos, end_pos-start_pos);
//
//    start_pos = end_pos = 0;
//
//    while( end_pos < argument_string.size()) {
//      end_pos   = argument_string.find(",", start_pos);
//      if (end_pos < argument_string.size()) {
//        /* store the last element */
//        __argument_stack.push_back(argument_string.substr(start_pos, end_pos - start_pos ));
//        cutBlanks(__argument_stack.back());
//      } else {
//        __argument_stack.push_back(argument_string.substr(start_pos, end_pos-start_pos ));
//        cutBlanks(__argument_stack.back());
//      }
//      start_pos = end_pos + 1;
//    }
  }

  IClicValueStream* IClicValueStream::copy() const
  {
    return new IClicValueStream(__event_key, __argument_stack);
  }


  IClicValueStream& IClicValueStream::operator>>(u_int16_t& value)       {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }

  IClicValueStream& IClicValueStream::operator>>(int16_t& value)         {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }

  IClicValueStream& IClicValueStream::operator>>(u_int32_t& value  )      {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }

  IClicValueStream& IClicValueStream::operator>>(int32_t& value  )        {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }
  
  IClicValueStream& IClicValueStream::operator>>(u_int64_t& value  )      {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }

  IClicValueStream& IClicValueStream::operator>>(int64_t& value  )        {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }

  IClicValueStream& IClicValueStream::operator>>(bool& value  )           {
    if ( __argument_stack[__current_pos].size() > 1 ) {
      string lower_value;
      /* convert each character to a lower character */
      for (unsigned int i = 0; i< __argument_stack[__current_pos].size(); ++i ) {
        lower_value.push_back( tolower(__argument_stack[__current_pos][i]) );
      }
      if (lower_value == "true") {
        value = true;
      } else if (lower_value == "false") {
        value = false;
      } else {
        throw Exception(string("Error in argument parsing! Can not convert '") +__argument_stack[__current_pos] + string("' to a bool type!"));
      }
    } else {
      strstream s;
      s << __argument_stack[__current_pos];
      s >> value;
      ++__current_pos;
    }

    return *this;
  };

  IClicValueStream& IClicValueStream::operator>>(unsigned long& value)    {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }
  
  IClicValueStream& IClicValueStream::operator>>(long& value )            {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }
  
  IClicValueStream& IClicValueStream::operator>>(double& value )          {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }

  IClicValueStream& IClicValueStream::operator>>(float& value )           {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }

  IClicValueStream& IClicValueStream::operator>>(char& value )            {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }

  IClicValueStream& IClicValueStream::operator>>(unsigned char& value)    {
    strstream s;
    s << __argument_stack[__current_pos];
    s >> value;
    ++__current_pos;
    return *this;
  }

  IClicValueStream& IClicValueStream::operator>>(string& value)           {
    value = __argument_stack[__current_pos];
    ++__current_pos;
    return *this;
  }

  bool IClicValueStream::goDown()    {
    return true;
  }

  void IClicValueStream::goUp()      {
    return;
  }

  bool IClicValueStream::isEmpty()    const {
    if (__current_pos >= __argument_stack.size())
      return true;
    return false;
  }

  bool IClicValueStream::nextElement() {
    return true;
  }

  const TypeInfo& IClicValueStream::getCurrentTypeInfo() const {
    return __empty_type_info;
  }

  const string& IClicValueStream::getCurrentType() const {
    return __empty_type_info.t_name;
  }

  const string& IClicValueStream::getCurrentNamespace() const {
    return __empty_type_info.t_namespace;
  }
  const string& IClicValueStream::getIValueStreamType() const {
    return __stream_type;
  }

  void IClicValueStream::readBuffer(char * buffer, unsigned int size) {
    memcpy( buffer, __argument_stack[__current_pos].c_str(), size);
    ++__current_pos;
  }

  void IClicValueStream::readBuffer(unsigned char * buffer, unsigned int size){
    memcpy( buffer, __argument_stack[__current_pos].c_str(), size);
    ++__current_pos;
  }

  void IClicValueStream::cutBlanks(string& s ) {
    /* cut the leading blanks */
    while (s[0] == ' ') {
      s = s.substr(1, s.size()-1);
    }

    /* cut the end blanks */
    while (s[s.size()-1] == ' ') {
      s = s.substr(0, s.size()-1);
    }

  }

  void IClicValueStream::readEvent(Event& event) {
    for (unsigned int i = 0; i< event.getArgSize(); ++i) {
      if ( event.getValue(i) )
        if ( !isEmpty() ) {
          (*this) >> *(event.getValue(i));
        } else {
          GLOBAL_WARNING_STREAM( i << ".th argument could not be read. Stream is empty! ");
          strstream s; string count;
          s << event.getArgSize();
          s >> count;
          throw Exception(string("Event ") + getEventKey() + string (" expects ") + count + string (" arguments"));
        }
      else
        throw Exception("Event " + getEventKey() + " has 0-ptr as arguments");
    }
  }

  const string    IClicValueStream::__stream_type("IClicValueStream");
  const TypeInfo  IClicValueStream::__empty_type_info("IClicValueStream");

}


